@@ -2,6 +2,7 @@
  * Copyright (c) 2015-2025 Valve Corporation
  * Copyright (c) 2015-2025 LunarG, Inc.
  * Copyright (C) 2015-2024 Google Inc.
+ * Copyright (c) 2025 Arm Limited.
  * Modifications Copyright (C) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -31,6 +32,7 @@
 #include "state_tracker/buffer_state.h"
 #include "state_tracker/event_map.h"
 #include "state_tracker/cmd_buffer_state.h"
+#include "state_tracker/tensor_state.h"
 
 // Holds common information between all command buffers being submitted
 struct CommandBufferSubmitState {
@@ -94,6 +96,23 @@ struct CommandBufferSubmitState {
                 skip |= function(video_session_state.get(), local_state_it->second, /*do_validate*/ true);
             }
         }
+
+        // Validate TensorMemoryBarrierARM
+        for (auto &barrier : cb_state.tensor_barriers) {
+            auto tensor_state = core.Get<vvl::Tensor>(barrier.tensor);
+            ASSERT_AND_RETURN_SKIP(tensor_state);
+            if (VK_SHARING_MODE_EXCLUSIVE == tensor_state->create_info.sharingMode &&
+                barrier.srcQueueFamilyIndex != VK_QUEUE_FAMILY_IGNORED && barrier.dstQueueFamilyIndex != VK_QUEUE_FAMILY_IGNORED &&
+                barrier.srcQueueFamilyIndex != queue_state.queue_family_index &&
+                barrier.dstQueueFamilyIndex != queue_state.queue_family_index) {
+                skip |= core.LogError("VUID-VkTensorMemoryBarrierARM-tensor-09757", cb_state.Handle(), loc,
+                                      "Tensor (%s) used in barrier has sharing mode VK_SHARING_MODE_EXCLUSIVE but neither "
+                                      "srcQueueFamilyIndex (%d) or dstQueueFamilyIndex (%d) are VK_QUEUE_FAMILY_IGNORED or "
+                                      "the same queue family as this queue which is executing the barrier (%d)",
+                                      core.FormatHandle(barrier.tensor).c_str(), barrier.srcQueueFamilyIndex,
+                                      barrier.dstQueueFamilyIndex, queue_state.queue_family_index);
+            }
+        }
         return skip;
     }
 
@@ -180,15 +199,16 @@ bool CoreChecks::PreCallValidateQueueSubmit(VkQueue queue, uint32_t submitCount,
             if ((cb_state->unprotected == true) && (protected_submit == true)) {
                 const LogObjectList objlist(cb_state->Handle(), queue);
                 skip |= LogError("VUID-VkSubmitInfo-pNext-04148", objlist, cb_loc,
-                                 "(%s) is unprotected while queue %s pSubmits[%u] has "
+                                 "(%s) is unprotected while queue %s pSubmits[%" PRIu32
+                                 "] has "
                                  "VkProtectedSubmitInfo:protectedSubmit set to VK_TRUE",
                                  FormatHandle(cb_state->Handle()).c_str(), FormatHandle(queue).c_str(), submit_idx);
             }
             if ((cb_state->unprotected == false) && (protected_submit == false)) {
                 const LogObjectList objlist(cb_state->Handle(), queue);
                 skip |= LogError("VUID-VkSubmitInfo-pNext-04120", objlist, cb_loc,
-                                 "(%s) is protected while queue %s pSubmits[%u] has %s", FormatHandle(cb_state->Handle()).c_str(),
-                                 FormatHandle(queue).c_str(), submit_idx,
+                                 "(%s) is protected while queue %s pSubmits[%" PRIu32 "] has %s",
+                                 FormatHandle(cb_state->Handle()).c_str(), FormatHandle(queue).c_str(), submit_idx,
                                  protected_submit_info ? "VkProtectedSubmitInfo:protectedSubmit set to VK_FALSE"
                                                        : "no VkProtectedSubmitInfo in the pNext chain");
             }
@@ -535,7 +555,6 @@ bool CoreChecks::ValidateCommandBufferState(const vvl::CommandBuffer &cb_state,
 
     // Validate that cmd buffers have been updated
     switch (cb_state.state) {
-        case CbState::InvalidIncomplete:
         case CbState::InvalidComplete:
             skip |= ReportInvalidCommandBuffer(cb_state, loc, vuid);
             break;
@@ -545,13 +564,15 @@ bool CoreChecks::ValidateCommandBufferState(const vvl::CommandBuffer &cb_state,
                              FormatHandle(cb_state).c_str());
             break;
 
+        case CbState::InvalidIncomplete:
         case CbState::Recording:
-            skip |= LogError(vuid, cb_state.Handle(), loc, "You must call vkEndCommandBuffer() on %s before this call.",
+            skip |= LogError(vuid, cb_state.Handle(), loc,
+                             "You must call vkEndCommandBuffer() on %s before this call can use the command buffer.",
                              FormatHandle(cb_state).c_str());
             break;
 
-        default: /* recorded */
-            break;
+        case CbState::Recorded:
+            break;  // only valid option
     }
     return skip;
 }
@@ -596,7 +617,7 @@ bool CoreChecks::ValidatePrimaryCommandBufferState(
                                  FormatHandle(sub_cb->primary_command_buffer).c_str());
             }
 
-            if (sub_cb->state != CbState::Recorded) {
+            if (!IsRecorded(sub_cb->state)) {
                 const char *const finished_cb_vuid = (loc.function == Func::vkQueueSubmit)
                                                          ? "VUID-vkQueueSubmit-pCommandBuffers-00072"
                                                          : "VUID-vkQueueSubmit2-commandBuffer-03876";
