@@ -583,6 +583,14 @@ void CommandBufferSubState::RecordBarriers2(const VkDependencyInfo& dep_info, co
         // Update layouts at the end. Submit time enqueuing logic above needs pre-update layout map.
         validator.RecordTransitionImageLayout(base, img_barrier, *image_state);
     }
+    if (const auto tensor_barrier_dep_info = vku::FindStructInPNextChain<VkTensorDependencyInfoARM>(dep_info.pNext)) {
+        const Location tensor_dep_info_loc(loc.function, vvl::Struct::VkTensorDependencyInfoARM, vvl::Field::pNext);
+        for (uint32_t i = 0; i < tensor_barrier_dep_info->tensorMemoryBarrierCount; ++i) {
+            const TensorBarrier barrier(tensor_barrier_dep_info->pTensorMemoryBarriers[i]);
+            base.tensor_barriers.emplace_back(barrier);
+        }
+    }
+
 }
 
 static void SetQueryState(const QueryObject& object, QueryState value, QueryMap* local_query_to_state_map) {
@@ -775,7 +783,7 @@ void CommandBufferSubState::RecordBeginVideoCoding(vvl::VideoSession& vs_state,
                     } else if (slot.resource && !dev_state.IsSlotPicture(slot.index, slot.resource)) {
                         skip |= validator.LogError("VUID-vkCmdBeginVideoCodingKHR-pPictureResource-07265", vs_state->Handle(), loc,
                                                    "DPB slot index %d of %s is not currently associated with the specified "
-                                                   "video picture resource: %s, layer %u, offset (%s), extent (%s).",
+                                                   "video picture resource: %s, layer %" PRIu32 ", offset (%s), extent (%s).",
                                                    slot.index, validator.FormatHandle(*vs_state).c_str(),
                                                    validator.FormatHandle(slot.resource.image_state->Handle()).c_str(),
                                                    slot.resource.range.baseArrayLayer,
@@ -856,7 +864,7 @@ void CommandBufferSubState::RecordDecodeVideo(vvl::VideoSession& vs_state, const
                                                         const char* picture_kind) -> bool {
                     return validator.LogError(vuid, vs_state->Handle(), loc,
                                               "DPB slot index %d of %s does not currently contain a %s with the specified "
-                                              "video picture resource: %s, layer %u, offset (%s), extent (%s).",
+                                              "video picture resource: %s, layer %" PRIu32 ", offset (%s), extent (%s).",
                                               slot.index, validator.FormatHandle(*vs_state).c_str(), picture_kind,
                                               validator.FormatHandle(slot.resource.image_state->Handle()).c_str(),
                                               slot.resource.range.baseArrayLayer,
@@ -907,8 +915,9 @@ void CommandBufferSubState::RecordEncodeVideo(vvl::VideoSession& vs_state, const
                     if (vsp_state->GetEncodeQualityLevel() != dev_state.GetEncodeQualityLevel()) {
                         const LogObjectList objlist(vs_state->Handle(), vsp_state->Handle());
                         skip |= validator.LogError("VUID-vkCmdEncodeVideoKHR-None-08318", objlist, loc,
-                                                   "The currently configured encode quality level (%u) for %s "
-                                                   "does not match the encode quality level (%u) %s was created with.",
+                                                   "The currently configured encode quality level (%" PRIu32
+                                                   ") for %s "
+                                                   "does not match the encode quality level (%" PRIu32 ") %s was created with.",
                                                    dev_state.GetEncodeQualityLevel(), validator.FormatHandle(*vs_state).c_str(),
                                                    vsp_state->GetEncodeQualityLevel(), validator.FormatHandle(*vsp_state).c_str());
                     }
