@@ -88,6 +88,7 @@ struct DrawDispatchVuid {
     const char* logic_op_04878 = kVUIDUndefined;
     const char* primitive_restart_enable_04879 = kVUIDUndefined;
     const char* primitive_restart_list_09637 = kVUIDUndefined;
+    const char* primitive_restart_patch_10909 = kVUIDUndefined;
     const char* vertex_input_binding_stride_04913 = kVUIDUndefined;
     const char* vertex_input_04914 = kVUIDUndefined;
     const char* vertex_input_08734 = kVUIDUndefined;
@@ -123,6 +124,8 @@ struct DrawDispatchVuid {
     const char* dynamic_rendering_07287 = kVUIDUndefined;
     const char* dynamic_rendering_local_location_09548 = kVUIDUndefined;
     const char* dynamic_rendering_local_index_09549 = kVUIDUndefined;
+    const char* dynamic_rendering_local_index_10927 = kVUIDUndefined;
+    const char* dynamic_rendering_local_index_10928 = kVUIDUndefined;
     const char* dynamic_rendering_dithering_09642 = kVUIDUndefined;
     const char* dynamic_rendering_dithering_09643 = kVUIDUndefined;
     const char* image_view_access_64_04470 = kVUIDUndefined;
@@ -151,14 +154,15 @@ struct DrawDispatchVuid {
     const char* dynamic_alpha_to_one_enable_07625 = kVUIDUndefined;
     const char* dynamic_logic_op_enable_07626 = kVUIDUndefined;
     const char* dynamic_color_blend_enable_07476 = kVUIDUndefined;
-    const char* dynamic_color_blend_equation_07477 = kVUIDUndefined;
     const char* dynamic_color_write_mask_07478 = kVUIDUndefined;
+    const char* dynamic_color_blend_equation_10862 = kVUIDUndefined;
+    const char* dynamic_color_blend_equation_10863 = kVUIDUndefined;
+    const char* dynamic_color_blend_equation_10864 = kVUIDUndefined;
     const char* dynamic_rasterization_stream_07630 = kVUIDUndefined;
     const char* dynamic_conservative_rasterization_mode_07631 = kVUIDUndefined;
     const char* dynamic_extra_primitive_overestimation_size_07632 = kVUIDUndefined;
     const char* dynamic_depth_clip_enable_07633 = kVUIDUndefined;
     const char* dynamic_sample_locations_enable_07634 = kVUIDUndefined;
-    const char* dynamic_color_blend_advanced_07479 = kVUIDUndefined;
     const char* dynamic_provoking_vertex_mode_07636 = kVUIDUndefined;
     const char* dynamic_depth_clip_negative_one_to_one_07639 = kVUIDUndefined;
     const char* dynamic_viewport_w_scaling_enable_07640 = kVUIDUndefined;
@@ -175,7 +179,6 @@ struct DrawDispatchVuid {
     const char* dynamic_viewport_07831 = kVUIDUndefined;
     const char* dynamic_scissor_07832 = kVUIDUndefined;
     const char* dynamic_depth_bias_07834 = kVUIDUndefined;
-    const char* dynamic_line_width_07833 = kVUIDUndefined;
     const char* dynamic_line_stipple_ext_07849 = kVUIDUndefined;
     const char* dynamic_blend_constants_07835 = kVUIDUndefined;
     const char* dynamic_depth_bounds_07836 = kVUIDUndefined;
@@ -219,20 +222,9 @@ struct DrawDispatchVuid {
     const char* external_format_resolve_09368 = kVUIDUndefined;
     const char* external_format_resolve_09369 = kVUIDUndefined;
     const char* external_format_resolve_09372 = kVUIDUndefined;
-    const char* external_format_resolve_09366 = kVUIDUndefined;
-    const char* external_format_resolve_09367 = kVUIDUndefined;
-    const char* external_format_resolve_09370 = kVUIDUndefined;
-    const char* external_format_resolve_09371 = kVUIDUndefined;
     const char* set_rasterization_samples_08644 = kVUIDUndefined;
-    const char* set_color_write_enable_08647 = kVUIDUndefined;
-    const char* set_color_blend_equation_08658 = kVUIDUndefined;
-    const char* set_blend_operation_advance_09416 = kVUIDUndefined;
     const char* set_line_rasterization_mode_08666 = kVUIDUndefined;
-    const char* set_line_rasterization_mode_08667 = kVUIDUndefined;
-    const char* set_line_rasterization_mode_08668 = kVUIDUndefined;
     const char* set_line_stipple_enable_08669 = kVUIDUndefined;
-    const char* set_line_stipple_enable_08670 = kVUIDUndefined;
-    const char* set_line_stipple_enable_08671 = kVUIDUndefined;
     const char* next_stage_10745 = kVUIDUndefined;
     const char* vertex_shader_08684 = kVUIDUndefined;
     const char* tessellation_control_shader_08685 = kVUIDUndefined;
@@ -252,8 +244,6 @@ struct DrawDispatchVuid {
     const char* shaders_descriptor_layouts_08879 = kVUIDUndefined;
     const char* draw_shaders_no_task_mesh_08885 = kVUIDUndefined;
     const char* set_line_width_08617 = kVUIDUndefined;
-    const char* set_line_width_08618 = kVUIDUndefined;
-    const char* set_line_width_08619 = kVUIDUndefined;
     const char* set_viewport_with_count_08642 = kVUIDUndefined;
     const char* alpha_component_word_08920 = kVUIDUndefined;
     const char* color_write_mask_09116 = kVUIDUndefined;
@@ -279,9 +269,6 @@ struct DrawDispatchVuid {
     const char* sample_locations_enable_07936 = kVUIDUndefined;
     const char* sample_locations_enable_07937 = kVUIDUndefined;
     const char* sample_locations_enable_07938 = kVUIDUndefined;
-    const char* set_blend_advanced_09417 = kVUIDUndefined;
-    const char* set_blend_equation_09418 = kVUIDUndefined;
-    const char* set_color_write_09419 = kVUIDUndefined;
     const char* set_coverage_to_color_location_09420 = kVUIDUndefined;
     const char* set_coverage_to_color_location_07490 = kVUIDUndefined;
     const char* set_viewport_swizzle_09421 = kVUIDUndefined;
@@ -301,9 +288,7 @@ struct DrawDispatchVuid {
     const char* vertex_input_09461 = kVUIDUndefined;
     const char* vertex_input_09462 = kVUIDUndefined;
     const char* color_blend_enable_07627 = kVUIDUndefined;
-    const char* color_blend_equation_07628 = kVUIDUndefined;
     const char* color_write_mask_07629 = kVUIDUndefined;
-    const char* color_blend_advanced_07635 = kVUIDUndefined;
     const char* image_layout_09600 = kVUIDUndefined;
     const char* rendering_contents_10582 = kVUIDUndefined;
     const char* line_rasterization_10608 = kVUIDUndefined;
