@@ -29,11 +29,8 @@
 #include "gpuav/validation_cmd/gpuav_dispatch.h"
 #include "gpuav/validation_cmd/gpuav_draw.h"
 #include "gpuav/validation_cmd/gpuav_trace_rays.h"
-#include "utils/assert_utils.h"
 #include "utils/math_utils.h"
 
-#include <cmath>
-
 namespace gpuav {
 
 void Validator::PreCallRecordCreateBuffer(VkDevice device, const VkBufferCreateInfo *pCreateInfo,
@@ -85,7 +82,7 @@ void Validator::PreCallRecordBeginCommandBuffer(VkCommandBuffer commandBuffer, c
 }
 
 void Instance::InternalWarning(LogObjectList objlist, const Location &loc, const char *const specific_message) const {
-    char const *vuid = gpuav_settings.debug_printf_only ? "WARNING-DEBUG-PRINTF" : "WARNING-GPU-Assisted-Validation";
+    const char *vuid = gpuav_settings.debug_printf_only ? "WARNING-DEBUG-PRINTF" : "WARNING-GPU-Assisted-Validation";
     LogWarning(vuid, objlist, loc, "Internal Warning: %s", specific_message);
 }
 
@@ -627,4 +624,38 @@ bool Validator::ValidateUnprotectedImage(const vvl::CommandBuffer &cb_state, con
     return skip;
 }
 
+// Validates the buffer is allowed to be protected
+bool Validator::ValidateProtectedTensor(const vvl::CommandBuffer &cb_state, const vvl::Tensor &tensor_state,
+                                        const Location &tensor_loc, const char *vuid, const char *more_message) const {
+    /* don't use on an unprotected tensor */
+    assert(tensor_state.unprotected == false);
+
+    bool skip = false;
+
+    // if driver supports protectedNoFault the operation is valid, just has undefined values
+    if ((!phys_dev_props_core11.protectedNoFault) && (cb_state.unprotected == true)) {
+        const LogObjectList objlist(cb_state.Handle(), tensor_state.Handle());
+        skip |= LogError(vuid, objlist, tensor_loc, "(%s) is a protected tensor, but command buffer (%s) is unprotected.%s",
+                         FormatHandle(tensor_state).c_str(), FormatHandle(cb_state).c_str(), more_message);
+    }
+    return skip;
+}
+
+// Validates the buffer is allowed to be unprotected
+bool Validator::ValidateUnprotectedTensor(const vvl::CommandBuffer &cb_state, const vvl::Tensor &tensor_state,
+                                          const Location &tensor_loc, const char *vuid, const char *more_message) const {
+    /* don't use on a protected tensor */
+    assert(tensor_state.unprotected == true);
+
+    bool skip = false;
+
+    // if driver supports protectedNoFault the operation is valid, just has undefined values
+    if ((!phys_dev_props_core11.protectedNoFault) && (cb_state.unprotected == false)) {
+        const LogObjectList objlist(cb_state.Handle(), tensor_state.Handle());
+        skip |= LogError(vuid, objlist, tensor_loc, "(%s) is an unprotected tensor, but command buffer (%s) is protected.%s",
+                         FormatHandle(tensor_state).c_str(), FormatHandle(cb_state).c_str(), more_message);
+    }
+    return skip;
+}
+
 }  // namespace gpuav
