@@ -17,12 +17,10 @@
 
 #include <vulkan/vulkan.h>
 #include "gpuav/debug_printf/debug_printf.h"
-#include "chassis/dispatch_object.h"
 #include "gpuav/shaders/gpuav_error_header.h"
 #include "gpuav/shaders/gpuav_shaders_constants.h"
 #include "gpuav/resources/gpuav_state_trackers.h"
 #include "gpuav/core/gpuav.h"
-#include "state_tracker/shader_instruction.h"
 #include "error_message/spirv_logging.h"
 
 #include <iostream>
@@ -69,10 +67,11 @@ struct Substring {
     bool needs_value = false;  // if value from buffer needed to print arguments
     NumericType type = NumericTypeUnknown;
     bool is_64_bit = false;
+    bool is_pointer = false;
 };
 
 static std::vector<Substring> ParseFormatString(const std::string &format_string) {
-    const char types[] = {'d', 'i', 'o', 'u', 'x', 'X', 'a', 'A', 'e', 'E', 'f', 'F', 'g', 'G', 'v', '\0'};
+    const char types[] = {'d', 'i', 'o', 'u', 'x', 'X', 'a', 'A', 'e', 'E', 'f', 'F', 'g', 'G', 'v', 'p', '\0'};
     std::vector<Substring> parsed_strings;
     size_t pos = 0;
     size_t begin = 0;
@@ -145,6 +144,11 @@ static std::vector<Substring> ParseFormatString(const std::string &format_string
                     substring.is_64_bit = true;
                     pos++;  // Save long size
                 }
+                if (format_string[pos] == 'p') {
+                    substring.is_64_bit = true;
+                    substring.is_pointer = true;
+                }
+
                 substring.string = format_string.substr(begin, pos - begin + 1);
                 substring.type = NumericTypeLookup(format_string[pos]);
                 parsed_strings.emplace_back(substring);
@@ -166,7 +170,7 @@ static std::vector<Substring> ParseFormatString(const std::string &format_string
 struct OutputRecord {
     uint32_t size;
     uint32_t shader_id;
-    uint32_t instruction_position;
+    uint32_t instruction_position_offset;
     uint32_t format_string_id;
     uint32_t double_bitmask;     // used to distinguish if float is 1 or 2 dwords
     uint32_t signed_8_bitmask;   // used to distinguish if signed int is a int8_t
@@ -178,9 +182,23 @@ struct OutputRecord {
     uint32_t values;  // place holder to be casted to get rest of items in record
 };
 
-void AnalyzeAndGenerateMessage(Validator &gpuav, VkCommandBuffer command_buffer, VkQueue queue,
-                               gpuav::DebugPrintfBufferInfo &buffer_info, uint32_t *const debug_output_buffer,
-                               const Location &loc) {
+struct DebugPrintfBufferInfo {
+    vko::BufferRange output_mem_buffer;
+    VkPipelineBindPoint pipeline_bind_point;
+    uint32_t action_command_index;
+    DebugPrintfBufferInfo(vko::BufferRange output_mem_buffer, VkPipelineBindPoint pipeline_bind_point,
+                          uint32_t action_command_index)
+        : output_mem_buffer(output_mem_buffer),
+          pipeline_bind_point(pipeline_bind_point),
+          action_command_index(action_command_index){};
+};
+
+struct DebugPrintfCbState {
+    std::vector<DebugPrintfBufferInfo> buffer_infos;
+};
+
+void AnalyzeAndGenerateMessage(Validator &gpuav, VkCommandBuffer command_buffer, DebugPrintfBufferInfo &buffer_info,
+                               uint32_t *const debug_output_buffer, const Location &loc) {
     uint32_t output_buffer_dwords_counts = debug_output_buffer[gpuav::kDebugPrintfOutputBufferDWordsCount];
     if (!output_buffer_dwords_counts) return;
 
@@ -198,14 +216,14 @@ void AnalyzeAndGenerateMessage(Validator &gpuav, VkCommandBuffer command_buffer,
         }
 
         // without the instrumented spirv, there is nothing valuable to print out
-        if (!instrumented_shader || instrumented_shader->instrumented_spirv.empty()) {
-            gpuav.InternalWarning(queue, loc, "Can't find instructions from any handles in shader_map");
+        if (!instrumented_shader || instrumented_shader->original_spirv.empty()) {
+            gpuav.InternalWarning(LogObjectList(), loc, "Can't find instructions from any handles in shader_map");
             return;
         }
 
         // Search through the shader source for the printf format string for this invocation
         std::string format_string;
-        const char *op_string = ::spirv::GetOpString(instrumented_shader->instrumented_spirv, debug_record->format_string_id);
+        const char *op_string = ::spirv::GetOpString(instrumented_shader->original_spirv, debug_record->format_string_id);
         if (op_string) {
             format_string = std::string(op_string);
         } else {
@@ -333,14 +351,20 @@ void AnalyzeAndGenerateMessage(Validator &gpuav, VkCommandBuffer command_buffer,
 
         const bool use_stdout = gpuav.gpuav_settings.debug_printf_to_stdout;
         if (gpuav.gpuav_settings.debug_printf_verbose) {
-            std::string debug_info_message = gpuav.GenerateDebugInfoMessage(
-                command_buffer, debug_record->stage_id, debug_record->stage_info_0, debug_record->stage_info_1,
-                debug_record->stage_info_2, debug_record->instruction_position, instrumented_shader, debug_record->shader_id,
-                buffer_info.pipeline_bind_point, buffer_info.action_command_index);
+            GpuShaderInstrumentor::ShaderMessageInfo shader_info{debug_record->stage_id,
+                                                                 debug_record->stage_info_0,
+                                                                 debug_record->stage_info_1,
+                                                                 debug_record->stage_info_2,
+                                                                 debug_record->instruction_position_offset,
+                                                                 debug_record->shader_id};
+
+            std::string debug_info_message =
+                gpuav.GenerateDebugInfoMessage(command_buffer, shader_info, instrumented_shader, buffer_info.pipeline_bind_point,
+                                               buffer_info.action_command_index);
             if (use_stdout) {
                 std::cout << "VVL-DEBUG-PRINTF " << shader_message.str() << '\n' << debug_info_message;
             } else {
-                LogObjectList objlist(queue, command_buffer);
+                LogObjectList objlist(command_buffer);
                 gpuav.LogInfo("VVL-DEBUG-PRINTF", objlist, loc, "DebugPrintf:\n%s\n%s", shader_message.str().c_str(),
                               debug_info_message.c_str());
             }
@@ -373,43 +397,42 @@ void AnalyzeAndGenerateMessage(Validator &gpuav, VkCommandBuffer command_buffer,
 #pragma GCC diagnostic pop
 #endif
 
-bool UpdateInstrumentationDescSet(Validator &gpuav, CommandBuffer &cb_state, VkDescriptorSet instrumentation_desc_set,
-                                  VkPipelineBindPoint bind_point, const Location &loc) {
-    gpuav::vko::Buffer debug_printf_output_buffer(gpuav);
-
-    // Allocate memory for the output block that the gpu will use to return values for printf
-    VkBufferCreateInfo buffer_info = vku::InitStructHelper();
-    buffer_info.size = gpuav.gpuav_settings.debug_printf_buffer_size;
-    buffer_info.usage = VK_BUFFER_USAGE_STORAGE_BUFFER_BIT;
-    VmaAllocationCreateInfo alloc_info = {};
-    alloc_info.requiredFlags = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
-    alloc_info.preferredFlags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
-    const bool success = debug_printf_output_buffer.Create(loc, &buffer_info, &alloc_info);
-    if (!success) {
-        return false;
+void RegisterDebugPrintf(Validator &gpuav, CommandBufferSubState &cb_state) {
+    if (!gpuav.gpuav_settings.debug_printf_enabled) {
+        return;
     }
 
-    // Clear the output block to zeros so that only printf values from the gpu will be present
-    auto printf_output_ptr = (uint32_t *)debug_printf_output_buffer.MapMemory(loc);
-    memset(printf_output_ptr, 0, gpuav.gpuav_settings.debug_printf_buffer_size);
-    debug_printf_output_buffer.UnmapMemory();
+    cb_state.on_instrumentation_desc_set_update_functions.emplace_back(
+        [debug_printf_buffer_size = gpuav.gpuav_settings.debug_printf_buffer_size](
+            CommandBufferSubState &cb, VkPipelineBindPoint bind_point, VkDescriptorBufferInfo &out_buffer_info,
+            uint32_t &out_dst_binding) {
+            vko::BufferRange debug_printf_output_buffer =
+                cb.gpu_resources_manager.GetHostVisibleBufferRange(debug_printf_buffer_size);
+            std::memset(debug_printf_output_buffer.offset_mapped_ptr, 0, (size_t)debug_printf_buffer_size);
 
-    VkDescriptorBufferInfo debug_printf_desc_buffer_info = {};
-    debug_printf_desc_buffer_info.range = gpuav.gpuav_settings.debug_printf_buffer_size;
-    debug_printf_desc_buffer_info.buffer = debug_printf_output_buffer.VkHandle();
-    debug_printf_desc_buffer_info.offset = 0;
+            out_buffer_info.buffer = debug_printf_output_buffer.buffer;
+            out_buffer_info.offset = debug_printf_output_buffer.offset;
+            out_buffer_info.range = debug_printf_output_buffer.size;
 
-    VkWriteDescriptorSet wds = vku::InitStructHelper();
-    wds.dstBinding = glsl::kBindingInstDebugPrintf;
-    wds.descriptorCount = 1;
-    wds.descriptorType = VK_DESCRIPTOR_TYPE_STORAGE_BUFFER;
-    wds.pBufferInfo = &debug_printf_desc_buffer_info;
-    wds.dstSet = instrumentation_desc_set;
+            out_dst_binding = glsl::kBindingInstDebugPrintf;
 
-    DispatchUpdateDescriptorSets(gpuav.device, 1, &wds, 0, nullptr);
+            DebugPrintfCbState &debug_printf_cb_state = cb.shared_resources_cache.GetOrCreate<DebugPrintfCbState>();
+            debug_printf_cb_state.buffer_infos.emplace_back(debug_printf_output_buffer, bind_point, cb.action_command_count);
+        });
 
-    cb_state.debug_printf_buffer_infos.emplace_back(debug_printf_output_buffer, bind_point, cb_state.action_command_count);
-    return true;
+    cb_state.on_cb_completion_functions.emplace_back([](Validator &gpuav, CommandBufferSubState &cb,
+                                                        const CommandBufferSubState::LabelLogging &label_logging,
+                                                        const Location &loc) {
+        DebugPrintfCbState *debug_printf_cb_state = cb.shared_resources_cache.TryGet<DebugPrintfCbState>();
+        if (!debug_printf_cb_state) {
+            return true;
+        }
+        for (DebugPrintfBufferInfo &printf_buffer_info : debug_printf_cb_state->buffer_infos) {
+            auto printf_output_ptr = (char *)printf_buffer_info.output_mem_buffer.offset_mapped_ptr;
+            debug_printf::AnalyzeAndGenerateMessage(gpuav, cb.VkHandle(), printf_buffer_info, (uint32_t *)printf_output_ptr, loc);
+        }
+        return true;
+    });
 }
 
 }  // namespace debug_printf
