@@ -34,6 +34,7 @@
 #include "state_tracker/last_bound_state.h"
 #include "state_tracker/shader_object_state.h"
 #include "state_tracker/pipeline_state.h"
+#include "state_tracker/pipeline_layout_state.h"
 #include "state_tracker/shader_module.h"
 #include "utils/action_command_utils.h"
 
@@ -41,10 +42,12 @@ namespace gpuav {
 
 // If application is using shader objects, bindings count will be computed from bound shaders
 static uint32_t LastBoundPipelineOrShaderDescSetBindingsCount(const LastBound &last_bound) {
-    if (last_bound.pipeline_state && last_bound.pipeline_state->PreRasterPipelineLayoutState()) {
-        return static_cast<uint32_t>(last_bound.pipeline_state->PreRasterPipelineLayoutState()->set_layouts.size());
+    // App uses pipeline or graphics pipeline libraries
+    if (last_bound.pipeline_state && last_bound.pipeline_state->PipelineLayoutState()) {
+        return uint32_t(last_bound.pipeline_state->PipelineLayoutState()->set_layouts.size());
     }
 
+    // App uses shader objects
     if (const vvl::ShaderObject *main_bound_shader = last_bound.GetFirstShader()) {
         return static_cast<uint32_t>(main_bound_shader->set_layouts.size());
     }
@@ -213,22 +216,33 @@ static std::pair<std::optional<VertexAttributeFetchLimit>, std::optional<VertexA
             }
             const VkDeviceSize attribute_size = GetVertexInputFormatSize(attrib.desc.format);
 
-            const VkDeviceSize stride =
-                vbb->stride != 0 ? vbb->stride : attribute_size;  // Tracked stride should already handle all possible value origin
-
             VkDeviceSize vertex_buffer_remaining_size =
                 vbb->effective_size > attrib.desc.offset ? vbb->effective_size - attrib.desc.offset : 0;
 
-            VkDeviceSize vertex_attributes_count = vertex_buffer_remaining_size / stride;
-            if (vertex_buffer_remaining_size > vertex_attributes_count * stride) {
-                vertex_buffer_remaining_size -= vertex_attributes_count * stride;
-            } else {
-                vertex_buffer_remaining_size = 0;
-            }
+            VkDeviceSize vertex_attributes_count = 0;
+            if (vbb->stride > 0) {
+                vertex_attributes_count = vertex_buffer_remaining_size / vbb->stride;
 
-            // maybe room for one more attribute but not full stride - not having stride space does not matter for last element
-            if (vertex_buffer_remaining_size >= attribute_size) {
-                vertex_attributes_count += 1;
+                if (vertex_buffer_remaining_size > vertex_attributes_count * vbb->stride) {
+                    vertex_buffer_remaining_size -= vertex_attributes_count * vbb->stride;
+                } else {
+                    vertex_buffer_remaining_size = 0;
+                }
+
+                // maybe room for one more attribute but not full stride - not having stride space does not matter for last element
+                if (vertex_buffer_remaining_size >= attribute_size) {
+                    vertex_attributes_count += 1;
+                }
+            } else {
+                // For the current attribute type, if stride is 0, the same vertex data chunk will be accessed by all vertex shader
+                // instances See https://docs.vulkan.org/spec/latest/chapters/fxvertex.html#fxvertex-input-address-calculation
+                if (vertex_buffer_remaining_size >= attribute_size) {
+                    // attribute won't be limiting
+                    continue;
+                } else {
+                    // Vertex attribute does not fit in vertex buffer
+                    vertex_attributes_count = 0;
+                }
             }
 
             if (vertex_binding_desc.desc.inputRate == VK_VERTEX_INPUT_RATE_VERTEX) {
@@ -471,57 +485,67 @@ void PreCallSetupShaderInstrumentationResources(Validator &gpuav, CommandBufferS
 
     // Pathetic way of trying to make sure we take care of updating all
     // bindings of the instrumentation descriptor set
-    assert(gpuav.instrumentation_bindings_.size() == 9);
+    assert(gpuav.instrumentation_bindings_.size() == glsl::kTotalBindings);
 
     InstrumentationErrorBlob instrumentation_error_blob;
     UpdateInstrumentationDescSet(gpuav, cb_state, bind_point, instrumentation_desc_set, loc, instrumentation_error_blob);
 
-    instrumentation_error_blob.operation_index = (bind_point == VK_PIPELINE_BIND_POINT_GRAPHICS)  ? cb_state.draw_index
-                                                 : (bind_point == VK_PIPELINE_BIND_POINT_COMPUTE) ? cb_state.compute_index
-                                                 : (bind_point == VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR)
-                                                     ? cb_state.trace_rays_index
-                                                     : 0;
-
+    instrumentation_error_blob.action_command_i = (bind_point == VK_PIPELINE_BIND_POINT_GRAPHICS)  ? cb_state.draw_index
+                                                  : (bind_point == VK_PIPELINE_BIND_POINT_COMPUTE) ? cb_state.compute_index
+                                                  : (bind_point == VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR)
+                                                      ? cb_state.trace_rays_index
+                                                      : 0;
     instrumentation_error_blob.pipeline_bind_point = bind_point;
     instrumentation_error_blob.uses_shader_object = last_bound.pipeline_state == nullptr;
 
     // Bind instrumentation descriptor set, using an appropriate pipeline layout
     // ---
 
-    // First find this appropriate pipeline layout.
-    // Always try to grab pipeline layout from last bound pipeline. Looking at PreRasterPipelineLayoutState
-    // is enough to get the layout whether the application is using standard pipelines or GPL.
-    // If GPU-AV failed to get a pipeline layout this way, fall back to pipeline layout specified in last
-    // vkCmdBindDescriptorSets, or in last vkCmdPushConstantRanges.
-
     enum class PipelineLayoutSource { NoPipelineLayout, LastBoundPipeline, LastBoundDescriptorSet, LastPushedConstants };
     std::shared_ptr<const vvl::PipelineLayout> inst_binding_pipe_layout_state;
+    VkPipelineLayout inst_binding_pipe_layout = VK_NULL_HANDLE;
     PipelineLayoutSource inst_binding_pipe_layout_src = PipelineLayoutSource::NoPipelineLayout;
-    if (last_bound.pipeline_state && !last_bound.pipeline_state->PreRasterPipelineLayoutState()->Destroyed()) {
-        inst_binding_pipe_layout_state = last_bound.pipeline_state->PreRasterPipelineLayoutState();
-        inst_binding_pipe_layout_src = PipelineLayoutSource::LastBoundPipeline;
 
-        // One exception when using GPL is we need to look out for INDEPENDENT_SETS_BIT which will have null sets inside them.
-        // We have a fake merged_graphics_layout to mimic the complete layout, but the app must bind it to descriptor set
-        if (inst_binding_pipe_layout_state->IsIndependentSets()) {
+    // App uses regular pipelines or graphics pipeline libraries
+    if (last_bound.pipeline_state) {
+        // Regular pipeline, and its pipeline layout has not been destroyed: pick it
+        const auto last_bound_pipe_layout = last_bound.pipeline_state->PipelineLayoutState();
+        if (!last_bound.pipeline_state->library_create_info && last_bound_pipe_layout && !last_bound_pipe_layout->Destroyed()) {
+            inst_binding_pipe_layout = last_bound.pipeline_state->PipelineLayoutState()->VkHandle();
+        }
+        // Pipeline layout has been destroyed, or pipeline is a graphics pipeline library
+        else {
+            const PipelineSubState &pipeline_sub_state = SubState(*last_bound.pipeline_state);
+            inst_binding_pipe_layout = pipeline_sub_state.GetPipelineLayoutUnion(loc);
+            assert(inst_binding_pipe_layout != VK_NULL_HANDLE);
+            if (gpuav.aborted_) {
+                return;
+            }
+        }
+        inst_binding_pipe_layout_src = PipelineLayoutSource::LastBoundPipeline;
+    }
+    // App uses shader objects
+    else {
+        if (last_bound.desc_set_pipeline_layout) {
             inst_binding_pipe_layout_state = last_bound.desc_set_pipeline_layout;
+            inst_binding_pipe_layout = inst_binding_pipe_layout_state->VkHandle();
             inst_binding_pipe_layout_src = PipelineLayoutSource::LastBoundDescriptorSet;
+        } else if (cb_state.push_constant_latest_used_layout[vvl_bind_point] != VK_NULL_HANDLE) {
+            inst_binding_pipe_layout_state =
+                gpuav.Get<vvl::PipelineLayout>(cb_state.push_constant_latest_used_layout[vvl_bind_point]);
+            inst_binding_pipe_layout = inst_binding_pipe_layout_state->VkHandle();
+            inst_binding_pipe_layout_src = PipelineLayoutSource::LastPushedConstants;
         }
-    } else if (last_bound.desc_set_pipeline_layout) {
-        inst_binding_pipe_layout_state = last_bound.desc_set_pipeline_layout;
-        inst_binding_pipe_layout_src = PipelineLayoutSource::LastBoundDescriptorSet;
-    } else if (cb_state.push_constant_latest_used_layout[vvl_bind_point] != VK_NULL_HANDLE) {
-        inst_binding_pipe_layout_state = gpuav.Get<vvl::PipelineLayout>(cb_state.push_constant_latest_used_layout[vvl_bind_point]);
-        inst_binding_pipe_layout_src = PipelineLayoutSource::LastPushedConstants;
     }
 
     // TODO: Using cb_state.per_command_resources.size() is kind of a hack? Worth considering passing the resource index as a
     // parameter
-    const uint32_t error_logger_i = static_cast<uint32_t>(cb_state.per_command_error_loggers.size());
-    const std::array<uint32_t, 2> dynamic_offsets = {{instrumentation_error_blob.operation_index * gpuav.indices_buffer_alignment_,
+    const uint32_t error_logger_i = static_cast<uint32_t>(cb_state.command_error_loggers.size());
+    const std::array<uint32_t, 2> dynamic_offsets = {{instrumentation_error_blob.action_command_i * gpuav.indices_buffer_alignment_,
                                                       error_logger_i * gpuav.indices_buffer_alignment_}};
-    if (inst_binding_pipe_layout_state) {
-        if ((uint32_t)inst_binding_pipe_layout_state->set_layouts.size() > gpuav.instrumentation_desc_set_bind_index_) {
+    if (inst_binding_pipe_layout) {
+        if (inst_binding_pipe_layout_state &&
+            (uint32_t)inst_binding_pipe_layout_state->set_layouts.size() > gpuav.instrumentation_desc_set_bind_index_) {
             gpuav.InternalWarning(cb_state.Handle(), loc,
                                   "Unable to bind instrumentation descriptor set, it would override application's bound set");
             return;
@@ -533,7 +557,7 @@ void PreCallSetupShaderInstrumentationResources(Validator &gpuav, CommandBufferS
                 assert(false);
                 break;
             case PipelineLayoutSource::LastBoundPipeline:
-                DispatchCmdBindDescriptorSets(cb_state.VkHandle(), bind_point, inst_binding_pipe_layout_state->VkHandle(),
+                DispatchCmdBindDescriptorSets(cb_state.VkHandle(), bind_point, inst_binding_pipe_layout,
                                               gpuav.instrumentation_desc_set_bind_index_, 1, &instrumentation_desc_set,
                                               static_cast<uint32_t>(dynamic_offsets.size()), dynamic_offsets.data());
                 break;
@@ -586,7 +610,7 @@ void PreCallSetupShaderInstrumentationResources(Validator &gpuav, CommandBufferS
                                       static_cast<uint32_t>(dynamic_offsets.size()), dynamic_offsets.data());
     }
 
-    // We want to grab the last (current) element in descriptor_binding_commands, but as a std::vector, the refernce might be
+    // We want to grab the last (current) element in descriptor_binding_commands, but as a std::vector, the reference might be
     // garbage later, so just hold the index for later. It is possible to have no descriptor sets bound, for example if using push
     // constants.
     instrumentation_error_blob.descriptor_binding_index = vvl::kU32Max;
@@ -599,8 +623,9 @@ void PreCallSetupShaderInstrumentationResources(Validator &gpuav, CommandBufferS
     instrumentation_error_blob.label_command_i =
         !cb_state.base.GetLabelCommands().empty() ? uint32_t(cb_state.base.GetLabelCommands().size() - 1) : vvl::kU32Max;
 
-    CommandBufferSubState::ErrorLoggerFunc error_logger = [&gpuav, &cb_state, loc, instrumentation_error_blob](
-                                                              const uint32_t *error_record, const LogObjectList &objlist,
+    CommandBufferSubState::ErrorLoggerFunc error_logger = [&gpuav, &cb_state, instrumentation_error_blob](
+                                                              const uint32_t *error_record, const Location &loc,
+                                                              const LogObjectList &objlist,
                                                               const std::vector<std::string> &initial_label_stack) {
         bool skip = false;
         skip |=
@@ -608,8 +633,9 @@ void PreCallSetupShaderInstrumentationResources(Validator &gpuav, CommandBufferS
         return skip;
     };
 
-    cb_state.action_cmd_i_to_label_cmd_i_map[cb_state.action_command_count] = instrumentation_error_blob.label_command_i;
-    cb_state.per_command_error_loggers.emplace_back(error_logger);
+    cb_state.command_error_loggers.emplace_back(CommandBufferSubState::CommandErrorLogger{
+        loc, last_bound.pipeline_state ? LogObjectList(last_bound.pipeline_state->VkHandle()) : LogObjectList{},
+        std::move(error_logger), int32_t(instrumentation_error_blob.label_command_i)});
 }
 
 void PostCallSetupShaderInstrumentationResources(Validator &gpuav, CommandBufferSubState &cb_state, const LastBound &last_bound,
@@ -1133,16 +1159,16 @@ bool LogInstrumentationError(Validator &gpuav, const CommandBufferSubState &cb_s
         Location loc_with_debug_region(loc, debug_region_name);
 
         const uint32_t stage_id = error_record[glsl::kHeaderStageInstructionIdOffset] >> glsl::kStageIdShift;
-        const uint32_t instruction_position = error_record[glsl::kHeaderStageInstructionIdOffset] & glsl::kInstructionIdMask;
+        const uint32_t instruction_position_offset = error_record[glsl::kHeaderStageInstructionIdOffset] & glsl::kInstructionIdMask;
         GpuShaderInstrumentor::ShaderMessageInfo shader_info{stage_id,
                                                              error_record[glsl::kHeaderStageInfoOffset_0],
                                                              error_record[glsl::kHeaderStageInfoOffset_1],
                                                              error_record[glsl::kHeaderStageInfoOffset_2],
-                                                             instruction_position,
+                                                             instruction_position_offset,
                                                              shader_id};
         std::string debug_info_message = gpuav.GenerateDebugInfoMessage(cb_state.VkHandle(), shader_info, instrumented_shader,
                                                                         instrumentation_error_blob.pipeline_bind_point,
-                                                                        instrumentation_error_blob.operation_index);
+                                                                        instrumentation_error_blob.action_command_i);
 
         gpuav.LogError(vuid_msg.c_str(), objlist, loc_with_debug_region, "%s\n%s", error_msg.c_str(), debug_info_message.c_str());
     }
