@@ -85,10 +85,10 @@ void RegisterPostProcessingValidation(Validator& gpuav, CommandBufferSubState& c
 
             // For each unique bound descriptor set in this command buffer,
             // create an appropriate post processing buffer,
-            // and update the "per CB submission desciptor set to post process buffers" LUT
+            // and update the "per CB submission descriptor set to post process buffers" LUT
 
             // For each CB submission, and for each descriptor binding command,
-            // a "desciptor set to post process buffers LUT" is allocated and updated in a VkBuffer.
+            // a "descriptor set to post process buffers LUT" is allocated and updated in a VkBuffer.
             // When executing, this CB submission will access its own private
             // post processing buffers, preventing concurrent use by another CB
             for (size_t ds_i = 0; ds_i < desc_binding_cmd.bound_descriptor_sets.size(); ds_i++) {
@@ -101,7 +101,7 @@ void RegisterPostProcessingValidation(Validator& gpuav, CommandBufferSubState& c
                 if (auto found = bound_desc_sets_to_pp_buffer_map->find(desc_binding_cmd.bound_descriptor_sets[ds_i]);
                     found == bound_desc_sets_to_pp_buffer_map->end()) {
                     // DescriptorSetSubState::GetPostProcessBufferSize() used to do a "auto guard = Lock()"
-                    // But the lock was only guarding againg GPU-AV sub state, not the base state, so
+                    // But the lock was only guarding against GPU-AV sub state, not the base state, so
                     // base.GetNonInlineDescriptorCount() access were not fully protected
                     const VkDeviceSize pp_buffer_size =
                         desc_set_state.base.GetNonInlineDescriptorCount() * sizeof(glsl::PostProcessDescriptorIndexSlot);
@@ -167,18 +167,16 @@ void RegisterPostProcessingValidation(Validator& gpuav, CommandBufferSubState& c
     // Validate descriptor set accesses done by command buffer submission
     cb.on_cb_completion_functions.emplace_back([bound_desc_sets_to_pp_buffer_map](
                                                    Validator& gpuav, CommandBufferSubState& cb,
-                                                   const CommandBufferSubState::LabelLogging& label_logging, const Location& loc) {
+                                                   const CommandBufferSubState::LabelLogging& label_logging,
+                                                   const Location& submission_loc) {
         VVL_ZoneScoped;
-        // TODO - Currently we don't know the actual call that triggered this, but without just giving "vkCmdDraw" we
-        // will get VUID_Undefined We now have the DescriptorValidator::action_index, just need to hook it up!
-        Location draw_loc(vvl::Func::vkCmdDraw);
 
         // We loop each vkCmdBindDescriptorSet, find each VkDescriptorSet that was used in the command buffer, and check
         // its post process buffer for which descriptor was accessed Only check a VkDescriptorSet once, might be bound
         // multiple times in a single command buffer
         for (auto& [desc_set, staging_buffer] : *bound_desc_sets_to_pp_buffer_map) {
             // We build once here, but will update the set_index and shader_handle when found
-            vvl::DescriptorValidator context(gpuav, cb.base, *desc_set, 0, VK_NULL_HANDLE, nullptr, draw_loc);
+            vvl::DescriptorValidator context(gpuav, cb.base, *desc_set, 0, VK_NULL_HANDLE, nullptr, Location(vvl::Func::Empty));
 
             // We create a map with the |unique_shader_id| as the key so we can only do the state object lookup once per
             // pipeline/shaderModule/shaderObject
@@ -194,10 +192,10 @@ void RegisterPostProcessingValidation(Validator& gpuav, CommandBufferSubState& c
                         const glsl::PostProcessDescriptorIndexSlot slot = slot_ptr[binding_layout.start + descriptor_i];
                         if (slot.meta_data & glsl::kPostProcessMetaMaskAccessed) {
                             const uint32_t shader_id = slot.meta_data & glsl::kShaderIdMask;
-                            const uint32_t action_index =
-                                (slot.meta_data & glsl::kPostProcessMetaMaskActionIndex) >> glsl::kPostProcessMetaShiftActionIndex;
-                            descriptor_access_map[shader_id].emplace_back(
-                                DescriptorAccess{binding, descriptor_i, slot.variable_id, action_index});
+                            const uint32_t error_logger_i = (slot.meta_data & glsl::kPostProcessMetaMaskErrorLoggerIndex) >>
+                                                            glsl::kPostProcessMetaShiftErrorLoggerIndex;
+                            descriptor_access_map[shader_id].emplace_back(DescriptorAccess{
+                                binding, descriptor_i, slot.variable_id, slot.instruction_position_offset, error_logger_i});
                         }
                     }
                 }
@@ -217,16 +215,16 @@ void RegisterPostProcessingValidation(Validator& gpuav, CommandBufferSubState& c
                 if (it->second.pipeline != VK_NULL_HANDLE) {
                     // We use pipeline over vkShaderModule as likely they will have been destroyed by now
                     pipeline_state = gpuav.Get<vvl::Pipeline>(it->second.pipeline).get();
-                    context.SetShaderHandleForGpuAv(&pipeline_state->Handle());
                 } else if (it->second.shader_object != VK_NULL_HANDLE) {
                     shader_object_state = gpuav.Get<vvl::ShaderObject>(it->second.shader_object).get();
                     ASSERT_AND_CONTINUE(shader_object_state->entrypoint);
-                    context.SetShaderHandleForGpuAv(&shader_object_state->Handle());
                 } else {
                     assert(false);
                     continue;
                 }
 
+                context.SetOriginalSpirv(&it->second.original_spirv);
+
                 for (const DescriptorAccess& descriptor_access : descriptor_accesses) {
                     auto descriptor_binding = desc_set->GetBinding(descriptor_access.binding);
                     ASSERT_AND_CONTINUE(descriptor_binding);
@@ -258,17 +256,21 @@ void RegisterPostProcessingValidation(Validator& gpuav, CommandBufferSubState& c
                         continue;
                     }
 
+                    context.SetInstructionPositionOffset(descriptor_access.instruction_position_offset);
+
                     // This will represent the Set that was accessed in the shader, which might not match the
                     // vkCmdBindDescriptorSet index if sets are aliased
                     context.SetSetIndexForGpuAv(resource_variable->decorations.set);
 
+                    const CommandBufferSubState::CommandErrorLogger& cmd_error_logger =
+                        cb.command_error_loggers[descriptor_access.error_logger_i];
+                    context.SetObjlistForGpuAv(&cmd_error_logger.objlist);
                     std::string debug_region_name;
-                    if (auto found_label_cmd_i = label_logging.action_cmd_i_to_label_cmd_i_map.find(descriptor_access.action_index);
-                        found_label_cmd_i != label_logging.action_cmd_i_to_label_cmd_i_map.end()) {
-                        debug_region_name = cb.GetDebugLabelRegion(found_label_cmd_i->second, label_logging.initial_label_stack);
+                    if (cmd_error_logger.label_cmd_i >= 0) {
+                        debug_region_name = cb.GetDebugLabelRegion(cmd_error_logger.label_cmd_i, label_logging.initial_label_stack);
                     }
 
-                    Location access_loc(loc, debug_region_name);
+                    Location access_loc(cmd_error_logger.loc.Get(), debug_region_name);
                     context.SetLocationForGpuAv(access_loc);
                     context.ValidateBindingDynamic(*resource_variable, *descriptor_binding, descriptor_access.index);
                 }
