@@ -1,6 +1,7 @@
 /* Copyright (c) 2018-2025 The Khronos Group Inc.
  * Copyright (c) 2018-2025 Valve Corporation
  * Copyright (c) 2018-2025 LunarG, Inc.
+ * Copyright (c) 2025 Arm Limited.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,14 +26,17 @@
 
 // We pull in most the core state tracking files
 // gpuav_state_trackers.h should NOT be included by any other header file
+
 #include "state_tracker/buffer_state.h"
-#include "state_tracker/image_state.h"
 #include "state_tracker/cmd_buffer_state.h"
-#include "state_tracker/queue_state.h"
-#include "state_tracker/sampler_state.h"
-#include "state_tracker/ray_tracing_state.h"
-#include "state_tracker/shader_object_state.h"
+#include "state_tracker/image_state.h"
+#include "state_tracker/pipeline_state.h"
 #include "state_tracker/push_constant_data.h"
+#include "state_tracker/queue_state.h"
+#include "state_tracker/ray_tracing_state.h"
+#include "state_tracker/sampler_state.h"
+#include "state_tracker/shader_object_state.h"
+#include "state_tracker/tensor_state.h"
 
 namespace gpuav {
 
@@ -43,7 +47,6 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
   public:
     struct LabelLogging {
         const std::vector<std::string> &initial_label_stack;
-        const vvl::unordered_map<uint32_t, uint32_t> &action_cmd_i_to_label_cmd_i_map;
     };
 
     using OnInstrumentionDescSetUpdate =
@@ -54,7 +57,7 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
         stdext::inplace_function<void(Validator &gpuav, CommandBufferSubState &cb, VkCommandBuffer per_submission_cb)>;
     using OnCommandBufferCompletion =
         stdext::inplace_function<bool(Validator &gpuav, CommandBufferSubState &cb,
-                                      const CommandBufferSubState::LabelLogging &label_logging, const Location &loc),
+                                      const CommandBufferSubState::LabelLogging &label_logging, const Location &submission_loc),
                                  64>;
     using OnPreCommandBufferSubmission =
         stdext::inplace_function<void(Validator &gpuav, CommandBufferSubState &cb, VkCommandBuffer per_pre_submission_cb)>;
@@ -119,13 +122,19 @@ class CommandBufferSubState : public vvl::CommandBufferSubState {
     void RecordEndRenderPass(const VkSubpassEndInfo *subpass_end_info, const Location &loc) final;
 
     vko::GpuResourcesManager gpu_resources_manager;
+
     // Using stdext::inplace_function over std::function to allocate memory in place
     using ErrorLoggerFunc =
-        stdext::inplace_function<bool(const uint32_t *error_record, const LogObjectList &objlist,
+        stdext::inplace_function<bool(const uint32_t *error_record, const Location &loc, const LogObjectList &objlist,
                                       const std::vector<std::string> &initial_label_stack),
                                  288 /*lambda storage size (bytes), large enough to store biggest error lambda*/>;
-    std::vector<ErrorLoggerFunc> per_command_error_loggers;
-    vvl::unordered_map<uint32_t, uint32_t> action_cmd_i_to_label_cmd_i_map;
+    struct CommandErrorLogger {
+        vvl::LocationCapture loc;
+        LogObjectList objlist;
+        ErrorLoggerFunc error_logger_func;
+        int32_t label_cmd_i = -1;
+    };
+    std::vector<CommandErrorLogger> command_error_loggers;
 
     // Buffer storing GPU-AV errors
     vko::BufferRange error_output_buffer_range_;
@@ -300,6 +309,38 @@ static inline const AccelerationStructureKHRSubState &SubState(const vvl::Accele
     return *static_cast<const AccelerationStructureKHRSubState *>(obj.SubState(LayerObjectTypeGpuAssisted));
 }
 
+class TensorSubState : public vvl::TensorSubState {
+  public:
+    TensorSubState(vvl::Tensor &obj, DescriptorHeap &heap);
+    void Destroy() override;
+    void NotifyInvalidate(const vvl::StateObject::NodeList &invalid_nodes, bool unlink) override;
+
+    DescriptorId Id() const { return id_tracker ? id_tracker->id : 0; }
+    std::optional<DescriptorIdTracker> id_tracker;
+};
+static inline TensorSubState &SubState(vvl::Tensor &obj) {
+    return *static_cast<TensorSubState *>(obj.SubState(LayerObjectTypeGpuAssisted));
+}
+static inline const TensorSubState &SubState(const vvl::Tensor &obj) {
+    return *static_cast<const TensorSubState *>(obj.SubState(LayerObjectTypeGpuAssisted));
+}
+
+class TensorViewSubState : public vvl::TensorViewSubState {
+  public:
+    TensorViewSubState(vvl::TensorView &obj, DescriptorHeap &heap);
+    void Destroy() override;
+    void NotifyInvalidate(const vvl::StateObject::NodeList &invalid_nodes, bool unlink) override;
+
+    DescriptorId Id() const { return id_tracker ? id_tracker->id : 0; }
+    std::optional<DescriptorIdTracker> id_tracker;
+};
+static inline TensorViewSubState &SubState(vvl::TensorView &obj) {
+    return *static_cast<TensorViewSubState *>(obj.SubState(LayerObjectTypeGpuAssisted));
+}
+static inline const TensorViewSubState &SubState(const vvl::TensorView &obj) {
+    return *static_cast<const TensorViewSubState *>(obj.SubState(LayerObjectTypeGpuAssisted));
+}
+
 class ShaderObjectSubState : public vvl::ShaderObjectSubState {
   public:
     explicit ShaderObjectSubState(vvl::ShaderObject &obj);
@@ -318,4 +359,24 @@ static inline const ShaderObjectSubState &SubState(const vvl::ShaderObject &obj)
     return *static_cast<const ShaderObjectSubState *>(obj.SubState(LayerObjectTypeGpuAssisted));
 }
 
+class PipelineSubState : public vvl::PipelineSubState {
+  public:
+    explicit PipelineSubState(Validator &gpuav, vvl::Pipeline &pipeline);
+
+    void Destroy() override;
+
+    VkPipelineLayout GetPipelineLayoutUnion(const Location &loc) const;
+
+  private:
+    mutable VkPipelineLayout recreated_layout = VK_NULL_HANDLE;
+    Validator &gpuav_;
+};
+
+static inline PipelineSubState &SubState(vvl::Pipeline &pipeline) {
+    return *static_cast<PipelineSubState *>(pipeline.SubState(LayerObjectTypeGpuAssisted));
+}
+static inline const PipelineSubState &SubState(const vvl::Pipeline &pipeline) {
+    return *static_cast<const PipelineSubState *>(pipeline.SubState(LayerObjectTypeGpuAssisted));
+}
+
 }  // namespace gpuav
