@@ -1,6 +1,7 @@
 // Copyright (c) 2021-2025 The Khronos Group Inc.
 // Copyright (c) 2021-2025 Valve Corporation
 // Copyright (c) 2021-2025 LunarG, Inc.
+// Copyright (c) 2025 Arm Limited.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -60,6 +61,7 @@ const int kBindingInstActionIndex = 5;
 const int kBindingInstCmdResourceIndex = 6;
 const int kBindingInstCmdErrorsCount = 7;
 const int kBindingInstVertexAttributeFetchLimits = 8;
+const int kTotalBindings = 9;
 
 // Validation pipelines
 // ---
@@ -101,6 +103,7 @@ const uint kTexelDesc = 4;
 const uint kBufferDesc = 5;
 const uint kInlineUniformDesc = 6;
 const uint kAccelDesc = 7;
+const uint kTensorDesc = 8;
 
 // Buffer Device Address Input Buffer Format
 //
@@ -140,8 +143,8 @@ const uint kShaderIdMask = 0x3FFFF;
 const uint kMaxActionsPerCommandBuffer = 1u << 13;  // 8,192
 // We use a single bit mark if this descriptor was accessed or not
 const uint kPostProcessMetaMaskAccessed = 1u << 31;
-const uint kPostProcessMetaShiftActionIndex = 18;
-const uint kPostProcessMetaMaskActionIndex = 0x1FFF << kPostProcessMetaShiftActionIndex;
+const uint kPostProcessMetaShiftErrorLoggerIndex = 18;
+const uint kPostProcessMetaMaskErrorLoggerIndex = 0x1FFF << kPostProcessMetaShiftErrorLoggerIndex;
 
 #ifdef __cplusplus
 }  // namespace glsl
