@@ -17,6 +17,7 @@
 #extension GL_EXT_buffer_reference : require
 #extension GL_EXT_buffer_reference2 : require
 #extension GL_EXT_buffer_reference_uvec2 : require
+#extension GL_EXT_scalar_block_layout : require
 #if defined(GL_ARB_gpu_shader_int64)
 #extension GL_ARB_gpu_shader_int64 : require
 #else
@@ -26,22 +27,22 @@
 #include "gpuav_error_header.h"
 #include "gpuav_shaders_constants.h"
 
-layout(set = kInstDefaultDescriptorSet, binding = kBindingInstErrorBuffer, std430) buffer OutputBuffer {
+layout(set = kInstDefaultDescriptorSet, binding = kBindingInstErrorBuffer, scalar) buffer OutputBuffer {
     uint flags;
     uint written_count;
     uint data[];
 }
 inst_errors_buffer;
 
-layout(set = kInstDefaultDescriptorSet, binding = kBindingInstActionIndex, std430) buffer ActionIndexBuffer { uint index[]; }
+layout(set = kInstDefaultDescriptorSet, binding = kBindingInstActionIndex, scalar) buffer ActionIndexBuffer { uint index[]; }
 inst_action_index_buffer;
 
-layout(set = kInstDefaultDescriptorSet, binding = kBindingInstCmdResourceIndex, std430) buffer CmdResourceIndexBuffer {
+layout(set = kInstDefaultDescriptorSet, binding = kBindingInstCmdResourceIndex, scalar) buffer ErrorLoggerIndexBuffer {
     uint index[];
 }
-inst_cmd_resource_index_buffer;
+inst_error_logger_index_buffer;
 
-layout(set = kInstDefaultDescriptorSet, binding = kBindingInstCmdErrorsCount, std430) buffer CmdErrorsCountBuffer {
+layout(set = kInstDefaultDescriptorSet, binding = kBindingInstCmdErrorsCount, scalar) buffer CmdErrorsCountBuffer {
     uint errors_count[];
 }
 inst_cmd_errors_count_buffer;
