@@ -1,6 +1,6 @@
-/* Copyright (c) 2018-2024 The Khronos Group Inc.
- * Copyright (c) 2018-2024 Valve Corporation
- * Copyright (c) 2018-2024 LunarG, Inc.
+/* Copyright (c) 2018-2025 The Khronos Group Inc.
+ * Copyright (c) 2018-2025 Valve Corporation
+ * Copyright (c) 2018-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -24,7 +24,9 @@ struct Location;
 namespace gpuav {
 class Validator;
 
-void InsertIndirectTraceRaysValidation(Validator &gpuav, const Location &loc, CommandBuffer &cb_state,
-                                       VkDeviceAddress indirect_data_address);
+namespace valcmd {
+void TraceRaysIndirect(Validator &gpuav, const Location &loc, CommandBufferSubState &cb_state,
+                       VkDeviceAddress indirect_data_address);
 
+}
 }  // namespace gpuav
