@@ -25,14 +25,14 @@ static VkExternalFenceHandleTypeFlags GetExportHandleTypes(const VkFenceCreateIn
     return export_info ? export_info->handleTypes : 0;
 }
 
-vvl::Fence::Fence(vvl::Device &dev, VkFence handle, const VkFenceCreateInfo *pCreateInfo)
+vvl::Fence::Fence(Logger &logger, VkFence handle, const VkFenceCreateInfo *pCreateInfo)
     : RefcountedStateObject(handle, kVulkanObjectTypeFence),
       flags(pCreateInfo->flags),
       export_handle_types(GetExportHandleTypes(pCreateInfo)),
       state_((pCreateInfo->flags & VK_FENCE_CREATE_SIGNALED_BIT) ? kRetired : kUnsignaled),
       completed_(),
       waiter_(completed_.get_future()),
-      dev_data_(dev) {}
+      logger_(logger) {}
 
 const VulkanTypedHandle *vvl::Fence::InUse() const {
     auto guard = ReadLock();
@@ -67,7 +67,7 @@ bool vvl::Fence::EnqueueSignal(vvl::Queue *queue_state, uint64_t next_seq) {
 // Called from a non-queue operation, such as vkWaitForFences()|
 void vvl::Fence::NotifyAndWait(const Location &loc) {
     std::shared_future<void> waiter;
-    AcquireFenceSync acquire_fence_sync;
+    std::optional<SubmissionReference> present_submission_ref;
     {
         // Hold the lock only while updating members, but not
         // while waiting
@@ -82,20 +82,27 @@ void vvl::Fence::NotifyAndWait(const Location &loc) {
                 queue_ = nullptr;
                 seq_ = 0;
             }
-            acquire_fence_sync = std::move(acquire_fence_sync_);
-            acquire_fence_sync_ = AcquireFenceSync{};
+            present_submission_ref = std::move(present_submission_ref_);
+            present_submission_ref_.reset();
         }
+        // Cleanup wait semaphores.
+        // NOTE: Functions like QueueWaitIdle put fence in the retired state, still it can have
+        // the list of present semaphores, which are not cleared by QueueWaitIdle when swapchain
+        // maintenance extension is enabled. That's the reason this code is not under kInflight condition.
+        for (auto &semaphore : present_wait_semaphores_) {
+            semaphore->ClearSwapchainWaitInfo();
+        }
+        present_wait_semaphores_.clear();
     }
     if (waiter.valid()) {
         auto result = waiter.wait_until(GetCondWaitTimeout());
         if (result != std::future_status::ready) {
-            dev_data_.LogError(
-                "INTERNAL-ERROR-VkFence-state-timeout", Handle(), loc,
-                "The Validation Layers hit a timeout waiting for fence state to update (this is most likely a validation bug).");
+            logger_.LogError("INTERNAL-ERROR-VkFence-state-timeout", Handle(), loc,
+                             "The Validation Layers hit a timeout waiting for fence state to update.");
         }
     }
-    for (const auto &submission_ref : acquire_fence_sync.submission_refs) {
-        submission_ref.queue->NotifyAndWait(loc, submission_ref.seq);
+    if (present_submission_ref.has_value()) {
+        present_submission_ref->queue->NotifyAndWait(loc, present_submission_ref->seq);
     }
 }
 
@@ -124,7 +131,11 @@ void vvl::Fence::Reset() {
     state_ = kUnsignaled;
     completed_ = std::promise<void>();
     waiter_ = std::shared_future<void>(completed_.get_future());
-    acquire_fence_sync_ = AcquireFenceSync{};
+    present_submission_ref_.reset();
+
+    // Do not reset swapchain-in-use state of each semaphore here, only stop the tracking.
+    // In order to reset swapchain-in-use state we need to wait on the fence.
+    present_wait_semaphores_.clear();
 }
 
 void vvl::Fence::Import(VkExternalFenceHandleTypeFlagBits handle_type, VkFenceImportFlags flags) {
@@ -165,11 +176,28 @@ std::optional<VkExternalFenceHandleTypeFlagBits> vvl::Fence::ImportedHandleType(
     return imported_handle_type_;
 }
 
-void vvl::Fence::SetAcquireFenceSync(const AcquireFenceSync &acquire_fence_sync) {
+void vvl::Fence::SetPresentSubmissionRef(const SubmissionReference &present_submission_ref) {
     auto guard = WriteLock();
+    // In an error-free scenario, "present_submission_ref_" member has no value (as optional).
+    // If the fence is reused without being waited on/reset (which causes a validation error),
+    // then we may find a stale ref value here. Simply overwrite it with a new ref.
+    //
+    // VVL INSIGHT: Interestingly, stale ref value can't happen in VVL tests because this function
+    // (from the Record phase) is not called if validation fails. However, the Record phase is
+    // always called for regular applications even in the case of validation errors.
+    // This means we cannot assert that submission ref member is empty. Such an assert would be
+    // valid for VVL tests but not for regular apps. It's not a big deal, but this observation
+    // may be useful to get better understanding of VVL internals.
+    //
+    // assert(!present_submission_ref_.has_value()); - only valid for VVL tests
 
-    // An attempt to overwrite existing acquire fence sync is a bug
-    assert(acquire_fence_sync.submission_refs.empty() || acquire_fence_sync_.submission_refs.empty());
+    assert(present_submission_ref.queue != nullptr);
+    present_submission_ref_ = present_submission_ref;
+}
 
-    acquire_fence_sync_ = acquire_fence_sync;
+void vvl::Fence::SetPresentWaitSemaphores(vvl::span<std::shared_ptr<vvl::Semaphore>> present_wait_semaphores) {
+    present_wait_semaphores_.clear();
+    for (const auto &semaphore : present_wait_semaphores) {
+        present_wait_semaphores_.emplace_back(semaphore);
+    }
 }
