@@ -21,12 +21,10 @@
 
 #include <vulkan/utility/vk_safe_struct.hpp>
 
-#include "state_tracker/pipeline_sub_state.h"
+#include "state_tracker/pipeline_library_state.h"
 #include "generated/dynamic_state_helper.h"
-#include "utils/shader_utils.h"
 #include "state_tracker/state_tracker.h"
 #include "state_tracker/shader_stage_state.h"
-#include "utils/vk_layer_utils.h"
 
 // Fwd declarations -- including descriptor_set.h creates an ugly include loop
 namespace vvl {
@@ -34,7 +32,7 @@ class DescriptorSetLayoutDef;
 class DescriptorSetLayout;
 class DescriptorSet;
 class Descriptor;
-class Device;
+class DeviceState;
 class RenderPass;
 class CommandBuffer;
 class Pipeline;
@@ -68,7 +66,9 @@ class PipelineCache : public StateObject {
     }
 };
 
-class Pipeline : public StateObject {
+class PipelineSubState;
+
+class Pipeline : public StateObject, public SubStateManager<PipelineSubState> {
   protected:
     // NOTE: The style guide suggests private data appear at the end, but we need this populated first, so placing it here
 
@@ -79,7 +79,7 @@ class Pipeline : public StateObject {
 
   public:
     const std::variant<vku::safe_VkGraphicsPipelineCreateInfo, vku::safe_VkComputePipelineCreateInfo,
-                       vku::safe_VkRayTracingPipelineCreateInfoCommon>
+                       vku::safe_VkRayTracingPipelineCreateInfoCommon, vku::safe_VkDataGraphPipelineCreateInfoARM>
         create_info;
 
     // Pipeline cache state
@@ -90,18 +90,22 @@ class Pipeline : public StateObject {
     const VkPipelineLibraryCreateInfoKHR *library_create_info = nullptr;
     VkGraphicsPipelineLibraryFlagsEXT graphics_lib_type = static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0);
     VkPipelineBindPoint pipeline_type;
-    VkPipelineCreateFlags2KHR create_flags;
+    VkPipelineCreateFlags2 create_flags;
     vvl::span<const vku::safe_VkPipelineShaderStageCreateInfo> shader_stages_ci;
     const vku::safe_VkPipelineLibraryCreateInfoKHR *ray_tracing_library_ci = nullptr;
     // If using a shader module identifier, the module itself is not validated, but the shader stage is still known
     const bool uses_shader_module_id;
 
-    // State split up based on library types
-    const std::shared_ptr<VertexInputState> vertex_input_state;  // VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT
-    const std::shared_ptr<PreRasterState> pre_raster_state;      // VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT
-    const std::shared_ptr<FragmentShaderState> fragment_shader_state;  // VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT
-    const std::shared_ptr<FragmentOutputState>
-        fragment_output_state;  // VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT
+    // State split up based on the 4 library states
+    //
+    // VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT
+    const std::shared_ptr<VertexInputState> vertex_input_state;
+    // VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT
+    const std::shared_ptr<PreRasterState> pre_raster_state;
+    // VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT
+    const std::shared_ptr<FragmentShaderState> fragment_shader_state;
+    // VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT
+    const std::shared_ptr<FragmentOutputState> fragment_output_state;
 
     // Additional metadata needed by pipeline_state initialization and validation
     const std::vector<ShaderStageState> stage_states;
@@ -123,20 +127,20 @@ class Pipeline : public StateObject {
     const ActiveSlotMap active_slots;
     const uint32_t max_active_slot = 0;  // the highest set number in active_slots for pipeline layout compatibility checks
 
-    // Which state is dynamic from pipeline creation, factors in GPL sub state as well
+    // Which state is dynamic from pipeline creation, factors in GPL library state as well
     CBDynamicFlags dynamic_state;
 
     const VkPrimitiveTopology topology_at_rasterizer = VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
     const bool descriptor_buffer_mode = false;
-    const bool uses_pipeline_robustness;
-    const bool uses_pipeline_vertex_robustness;
-    bool ignore_color_attachments;
+    const bool uses_pipeline_robustness = false;
+    const bool uses_pipeline_vertex_robustness = false;
+    bool ignore_color_attachments = true;
 
     mutable bool binary_data_released = false;
 
     // TODO - Because we have hack to create a pipeline at PreCallValidate time (for GPL) we have no proper way to create inherited
     // state objects of the pipeline This is to make it clear that while currently everyone has to allocate this memory, it is only
-    // ment for GPU-AV
+    // meant for GPU-AV
     struct InstrumentationData {
         // < unique_shader_id, instrumented_shader_module_handle >
         // We create a VkShaderModule that is instrumented and need to delete before leaving the pipeline call
@@ -148,23 +152,28 @@ class Pipeline : public StateObject {
         VkPipeline frag_out_lib = VK_NULL_HANDLE;
     } instrumentation_data;
 
-    Pipeline(const Device &state_data, const VkGraphicsPipelineCreateInfo *pCreateInfo,
+    Pipeline(const DeviceState &state_data, const VkGraphicsPipelineCreateInfo *pCreateInfo,
              std::shared_ptr<const vvl::PipelineCache> pipe_cache, std::shared_ptr<const vvl::RenderPass> &&rpstate,
              std::shared_ptr<const vvl::PipelineLayout> &&layout,
              spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]);
 
-    Pipeline(const Device &state_data, const VkComputePipelineCreateInfo *pCreateInfo,
+    Pipeline(const DeviceState &state_data, const VkComputePipelineCreateInfo *pCreateInfo,
              std::shared_ptr<const vvl::PipelineCache> &&pipe_cache, std::shared_ptr<const vvl::PipelineLayout> &&layout,
              spirv::StatelessData *stateless_data);
 
-    Pipeline(const Device &state_data, const VkRayTracingPipelineCreateInfoKHR *pCreateInfo,
+    Pipeline(const DeviceState &state_data, const VkRayTracingPipelineCreateInfoKHR *pCreateInfo,
              std::shared_ptr<const vvl::PipelineCache> &&pipe_cache, std::shared_ptr<const vvl::PipelineLayout> &&layout,
              spirv::StatelessData *stateless_data);
 
-    Pipeline(const Device &state_data, const VkRayTracingPipelineCreateInfoNV *pCreateInfo,
+    Pipeline(const DeviceState &state_data, const VkRayTracingPipelineCreateInfoNV *pCreateInfo,
              std::shared_ptr<const vvl::PipelineCache> &&pipe_cache, std::shared_ptr<const vvl::PipelineLayout> &&layout,
              spirv::StatelessData *stateless_data);
 
+    Pipeline(const DeviceState &state_data, const VkDataGraphPipelineCreateInfoARM *pCreateInfo,
+             std::shared_ptr<const vvl::PipelineCache> &&pipe_cache, std::shared_ptr<const vvl::PipelineLayout> &&layout,
+             spirv::StatelessData *stateless_data);
+
+    void Destroy() override;
     VkPipeline VkHandle() const { return handle_.Cast<VkPipeline>(); }
 
     void SetHandle(VkPipeline p) { handle_.handle = CastToUint64(p); }
@@ -174,8 +183,7 @@ class Pipeline : public StateObject {
         if (pipeline_type != VK_PIPELINE_BIND_POINT_GRAPHICS) {
             return true;
         }
-        // First make sure that this pipeline is a "classic" pipeline, or is linked together with the appropriate sub-state
-        // libraries
+        // First make sure that this pipeline is a "classic" pipeline, or is linked together with the appropriate library state
         if (graphics_lib_type && (graphics_lib_type != (VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT |
                                                         VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT |
                                                         VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT |
@@ -203,39 +211,42 @@ class Pipeline : public StateObject {
     }
 
     template <VkGraphicsPipelineLibraryFlagBitsEXT type_flag>
-    struct SubStateTraits {};
+    struct LibraryStateTraits {};
 
     template <VkGraphicsPipelineLibraryFlagBitsEXT type_flag>
-    static inline typename SubStateTraits<type_flag>::type GetSubState(const Pipeline &) {
+    static inline typename LibraryStateTraits<type_flag>::type GetLibraryState(const Pipeline &) {
         return {};
     }
 
-    std::shared_ptr<const vvl::ShaderModule> GetSubStateShader(VkShaderStageFlagBits state) const;
+    std::shared_ptr<const vvl::ShaderModule> GetLibraryStateShader(VkShaderStageFlagBits state) const;
 
     template <VkGraphicsPipelineLibraryFlagBitsEXT type_flag>
-    static inline typename SubStateTraits<type_flag>::type GetLibSubState(const Device &state,
-                                                                          const VkPipelineLibraryCreateInfoKHR &link_info) {
+    static inline typename LibraryStateTraits<type_flag>::type GetLibraryState(const DeviceState &device_state,
+                                                                               const VkPipelineLibraryCreateInfoKHR &link_info) {
         for (uint32_t i = 0; i < link_info.libraryCount; ++i) {
-            const auto lib_state = state.Get<vvl::Pipeline>(link_info.pLibraries[i]);
+            const auto lib_state = device_state.Get<vvl::Pipeline>(link_info.pLibraries[i]);
             if (lib_state && ((lib_state->graphics_lib_type & type_flag) != 0)) {
-                return GetSubState<type_flag>(*lib_state);
+                return GetLibraryState<type_flag>(*lib_state);
             }
         }
         return {};
     }
 
-    // Used to know if the pipeline substate is being created (as opposed to being linked)
-    // Important as some pipeline checks need pipeline state that won't be there if the substate is from linking
+    // Used to know if the pipeline library is being created (as opposed to being linked)
+    // Important as some pipeline checks need pipeline state that won't be there if the library is from linking
     // Many VUs say "the pipeline require" which means "not being linked in as a library"
     // If the VUs says "created with" then you should NOT use this function
     // TODO - This could probably just be a check to VkGraphicsPipelineLibraryCreateInfoEXT::flags
-    bool OwnsSubState(const std::shared_ptr<PipelineSubState> sub_state) const { return sub_state && (&sub_state->parent == this); }
+    bool OwnsLibState(const std::shared_ptr<PipelineLibraryState> lib_state) const {
+        return lib_state && (&lib_state->parent == this);
+    }
 
     // This grabs the render pass at pipeline creation time, if you are inside a command buffer, use the vvl::RenderPass inside the
     // command buffer! (The render pass can be different as they just have to be compatible, see
     // vkspec.html#renderpass-compatibility)
     const std::shared_ptr<const vvl::RenderPass> RenderPassState() const {
-        // TODO A render pass object is required for all of these sub-states. Which one should be used for an "executable pipeline"?
+        // TODO A render pass object is required for all of these library states. Which one should be used for an "executable
+        // pipeline"?
         if (fragment_output_state && fragment_output_state->rp_state) {
             return fragment_output_state->rp_state;
         } else if (fragment_shader_state && fragment_shader_state->rp_state) {
@@ -248,14 +259,15 @@ class Pipeline : public StateObject {
 
     // A pipeline does not "require" state that is specified in a library.
     bool IsRenderPassStateRequired() const {
-        return OwnsSubState(pre_raster_state) || OwnsSubState(fragment_shader_state) || OwnsSubState(fragment_output_state);
+        return OwnsLibState(pre_raster_state) || OwnsLibState(fragment_shader_state) || OwnsLibState(fragment_output_state);
     }
 
-    // There could be an invalid RenderPass which will not come as as null, need to check RenderPassState() if it is valid
+    // There could be an invalid RenderPass which will not come as null, need to check RenderPassState() if it is valid
     bool IsRenderPassNull() const { return GraphicsCreateInfo().renderPass == VK_NULL_HANDLE; }
 
     const std::shared_ptr<const vvl::PipelineLayout> PipelineLayoutState() const {
-        // TODO A render pass object is required for all of these sub-states. Which one should be used for an "executable pipeline"?
+        // TODO A render pass object is required for all of these library states. Which one should be used for an "executable
+        // pipeline"?
         if (merged_graphics_layout) {
             return merged_graphics_layout;
         } else if (pre_raster_state) {
@@ -284,7 +296,8 @@ class Pipeline : public StateObject {
 
     // the VkPipelineMultisampleStateCreateInfo need to be identically defined so can safely grab both
     const vku::safe_VkPipelineMultisampleStateCreateInfo *MultisampleState() const {
-        // TODO A render pass object is required for all of these sub-states. Which one should be used for an "executable pipeline"?
+        // TODO A render pass object is required for all of these library states. Which one should be used for an "executable
+        // pipeline"?
         if (fragment_shader_state && fragment_shader_state->ms_state &&
             (fragment_shader_state->ms_state->rasterizationSamples >= VK_SAMPLE_COUNT_1_BIT) &&
             (fragment_shader_state->ms_state->rasterizationSamples < VK_SAMPLE_COUNT_FLAG_BITS_MAX_ENUM)) {
@@ -298,7 +311,8 @@ class Pipeline : public StateObject {
     }
 
     const vku::safe_VkPipelineRasterizationStateCreateInfo *RasterizationState() const {
-        // TODO A render pass object is required for all of these sub-states. Which one should be used for an "executable pipeline"?
+        // TODO A render pass object is required for all of these library states. Which one should be used for an "executable
+        // pipeline"?
         if (pre_raster_state) {
             return pre_raster_state->raster_state;
         }
@@ -322,7 +336,8 @@ class Pipeline : public StateObject {
     }
 
     const vku::safe_VkPipelineViewportStateCreateInfo *ViewportState() const {
-        // TODO A render pass object is required for all of these sub-states. Which one should be used for an "executable pipeline"?
+        // TODO A render pass object is required for all of these library states. Which one should be used for an "executable
+        // pipeline"?
         if (pre_raster_state) {
             return pre_raster_state->viewport_state;
         }
@@ -358,7 +373,8 @@ class Pipeline : public StateObject {
     }
 
     uint32_t Subpass() const {
-        // TODO A render pass object is required for all of these sub-states. Which one should be used for an "executable pipeline"?
+        // TODO A render pass object is required for all of these library states. Which one should be used for an "executable
+        // pipeline"?
         if (pre_raster_state) {
             return pre_raster_state->subpass;
         } else if (fragment_shader_state) {
@@ -392,6 +408,9 @@ class Pipeline : public StateObject {
     const vku::safe_VkRayTracingPipelineCreateInfoCommon &RayTracingCreateInfo() const {
         return std::get<vku::safe_VkRayTracingPipelineCreateInfoCommon>(create_info);
     }
+    const vku::safe_VkDataGraphPipelineCreateInfoARM &DataGraphCreateInfo() const {
+        return std::get<vku::safe_VkDataGraphPipelineCreateInfoARM>(create_info);
+    }
 
     VkStructureType GetCreateInfoSType() const {
         const auto *gfx = std::get_if<vku::safe_VkGraphicsPipelineCreateInfo>(&create_info);
@@ -403,7 +422,14 @@ class Pipeline : public StateObject {
             return cmp->sType;
         }
         const auto *rt = std::get_if<vku::safe_VkRayTracingPipelineCreateInfoCommon>(&create_info);
-        return rt->sType;
+        if (rt) {
+            return rt->sType;
+        }
+        const auto *dg = std::get_if<vku::safe_VkDataGraphPipelineCreateInfoARM>(&create_info);
+        if (dg) {
+            return dg->sType;
+        }
+        return VK_STRUCTURE_TYPE_MAX_ENUM;
     }
 
     const void *GetCreateInfoPNext() const {
@@ -419,11 +445,9 @@ class Pipeline : public StateObject {
         return rt->pNext;
     }
 
-    bool BlendConstantsEnabled() const { return fragment_output_state && fragment_output_state->blend_constants_enabled; }
+    const Location GetCreateFlagsLoc(const Location &create_info_loc) const;
 
-    bool SampleLocationEnabled() const { return fragment_output_state && fragment_output_state->sample_location_enabled; }
-
-    static std::vector<ShaderStageState> GetStageStates(const Device &state_data, const Pipeline &pipe_state,
+    static std::vector<ShaderStageState> GetStageStates(const DeviceState &state_data, const Pipeline &pipe_state,
                                                         spirv::StatelessData *stateless_data);
 
     // Return true if for a given PSO, the given state enum is dynamic, else return false
@@ -446,14 +470,14 @@ class Pipeline : public StateObject {
     }
 
     template <typename CreateInfo>
-    static bool EnablesRasterizationStates(const vvl::Device &vo, const CreateInfo &create_info) {
+    static bool EnablesRasterizationStates(const vvl::DeviceState &device_state, const CreateInfo &create_info) {
         // If this is an executable pipeline created from linking graphics libraries, we need to find the pre-raster library to
         // check if rasterization is enabled
         auto link_info = vku::FindStructInPNextChain<VkPipelineLibraryCreateInfoKHR>(create_info.pNext);
         if (link_info) {
             const auto libs = vvl::make_span(link_info->pLibraries, link_info->libraryCount);
             for (const auto handle : libs) {
-                auto lib = vo.template Get<vvl::Pipeline>(handle);
+                auto lib = device_state.template Get<vvl::Pipeline>(handle);
                 if (lib && lib->pre_raster_state) {
                     return EnablesRasterizationStates(lib->pre_raster_state);
                 }
@@ -479,21 +503,18 @@ class Pipeline : public StateObject {
     }
 
     template <typename CreateInfo>
-    static bool ContainsSubState(const vvl::Device *vo, const CreateInfo &create_info,
-                                 VkGraphicsPipelineLibraryFlagsEXT sub_state) {
+    static bool ContainsLibraryState(const vvl::DeviceState &device_state, const CreateInfo &create_info,
+                                     VkGraphicsPipelineLibraryFlagsEXT lib_flags) {
         constexpr VkGraphicsPipelineLibraryFlagsEXT null_lib = static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0);
         VkGraphicsPipelineLibraryFlagsEXT current_state = null_lib;
 
         // Check linked libraries
         auto link_info = vku::FindStructInPNextChain<VkPipelineLibraryCreateInfoKHR>(create_info.pNext);
         if (link_info) {
-            auto state_tracker = dynamic_cast<const Device *>(vo);
-            if (state_tracker) {
-                const auto libs = vvl::make_span(link_info->pLibraries, link_info->libraryCount);
-                for (const auto handle : libs) {
-                    auto lib = state_tracker->Get<vvl::Pipeline>(handle);
-                    current_state |= lib->graphics_lib_type;
-                }
+            const auto libs = vvl::make_span(link_info->pLibraries, link_info->libraryCount);
+            for (const auto handle : libs) {
+                auto lib = device_state.Get<vvl::Pipeline>(handle);
+                current_state |= lib->graphics_lib_type;
             }
         }
 
@@ -508,12 +529,12 @@ class Pipeline : public StateObject {
             return true;
         }
 
-        return (current_state & sub_state) != null_lib;
+        return (current_state & lib_flags) != null_lib;
     }
 
     // Version used at dispatch time for stateless VOs
     template <typename CreateInfo>
-    static bool ContainsSubState(const CreateInfo &create_info, VkGraphicsPipelineLibraryFlagsEXT sub_state) {
+    static bool ContainsLibraryState(const CreateInfo &create_info, VkGraphicsPipelineLibraryFlagsEXT lib_flags) {
         constexpr VkGraphicsPipelineLibraryFlagsEXT null_lib = static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0);
         VkGraphicsPipelineLibraryFlagsEXT current_state = null_lib;
 
@@ -531,19 +552,18 @@ class Pipeline : public StateObject {
             return true;
         }
 
-        return (current_state & sub_state) != null_lib;
+        return (current_state & lib_flags) != null_lib;
     }
 
     // This is a helper that is meant to be used during safe_VkPipelineRenderingCreateInfo construction to determine whether or not
     // certain fields should be ignored based on graphics pipeline state
-    // TODO - This is only a pointer to Device  because we are trying to do state tracking outside the state tracker
-    static bool PnextRenderingInfoCustomCopy(const Device *state_data, const VkGraphicsPipelineCreateInfo &graphics_info,
+    static bool PnextRenderingInfoCustomCopy(const DeviceState &device_state, const VkGraphicsPipelineCreateInfo &graphics_info,
                                              VkBaseOutStructure *safe_struct, const VkBaseOutStructure *in_struct) {
         // "safe_struct" is assumed to be non-null as it should be the "this" member of calling class instance
         assert(safe_struct);
         if (safe_struct->sType == VK_STRUCTURE_TYPE_PIPELINE_RENDERING_CREATE_INFO) {
-            const bool has_fo_state = Pipeline::ContainsSubState(state_data, graphics_info,
-                                                                 VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT);
+            const bool has_fo_state = Pipeline::ContainsLibraryState(
+                device_state, graphics_info, VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT);
             if (!has_fo_state) {
                 // Clear out all pointers except for viewMask. Since viewMask is a scalar, it has already been copied at this point
                 // in vku::safe_VkPipelineRenderingCreateInfo construction.
@@ -561,17 +581,17 @@ class Pipeline : public StateObject {
     }
 
   protected:
-    static std::shared_ptr<VertexInputState> CreateVertexInputState(const Pipeline &p, const Device &state,
+    static std::shared_ptr<VertexInputState> CreateVertexInputState(const Pipeline &p, const DeviceState &state,
                                                                     const vku::safe_VkGraphicsPipelineCreateInfo &create_info);
     static std::shared_ptr<PreRasterState> CreatePreRasterState(
-        const Pipeline &p, const Device &state, const vku::safe_VkGraphicsPipelineCreateInfo &create_info,
+        const Pipeline &p, const DeviceState &state, const vku::safe_VkGraphicsPipelineCreateInfo &create_info,
         const std::shared_ptr<const vvl::RenderPass> &rp, spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]);
     static std::shared_ptr<FragmentShaderState> CreateFragmentShaderState(
-        const Pipeline &p, const Device &state, const VkGraphicsPipelineCreateInfo &create_info,
+        const Pipeline &p, const DeviceState &state, const VkGraphicsPipelineCreateInfo &create_info,
         const vku::safe_VkGraphicsPipelineCreateInfo &safe_create_info, const std::shared_ptr<const vvl::RenderPass> &rp,
         spirv::StatelessData stateless_data[kCommonMaxGraphicsShaderStages]);
     static std::shared_ptr<FragmentOutputState> CreateFragmentOutputState(
-        const Pipeline &p, const Device &state, const VkGraphicsPipelineCreateInfo &create_info,
+        const Pipeline &p, const DeviceState &state, const VkGraphicsPipelineCreateInfo &create_info,
         const vku::safe_VkGraphicsPipelineCreateInfo &safe_create_info, const std::shared_ptr<const vvl::RenderPass> &rp);
 
     template <typename CreateInfo>
@@ -605,264 +625,70 @@ class Pipeline : public StateObject {
         return EnablesRasterizationStates(pre_raster_state->parent.GraphicsCreateInfo());
     }
 
-    // Merged layouts
     std::shared_ptr<const vvl::PipelineLayout> merged_graphics_layout;
 };
 
+class PipelineSubState {
+  public:
+    explicit PipelineSubState(Pipeline &pipeline) : base(pipeline) {}
+    PipelineSubState(const PipelineSubState &) = delete;
+    PipelineSubState &operator=(const PipelineSubState &) = delete;
+    virtual ~PipelineSubState() {}
+    virtual void Destroy() {}
+
+    Pipeline &base;
+};
+
 template <>
-struct Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT> {
+struct Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT> {
     using type = std::shared_ptr<VertexInputState>;
 };
 
 // static
 template <>
-inline Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT>::type
-Pipeline::GetSubState<VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT>(const Pipeline &pipe_state) {
+inline Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT>::type
+Pipeline::GetLibraryState<VK_GRAPHICS_PIPELINE_LIBRARY_VERTEX_INPUT_INTERFACE_BIT_EXT>(const Pipeline &pipe_state) {
     return pipe_state.vertex_input_state;
 }
 
 template <>
-struct Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT> {
+struct Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT> {
     using type = std::shared_ptr<PreRasterState>;
 };
 
 // static
 template <>
-inline Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT>::type
-Pipeline::GetSubState<VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT>(const Pipeline &pipe_state) {
+inline Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT>::type
+Pipeline::GetLibraryState<VK_GRAPHICS_PIPELINE_LIBRARY_PRE_RASTERIZATION_SHADERS_BIT_EXT>(const Pipeline &pipe_state) {
     return pipe_state.pre_raster_state;
 }
 
 template <>
-struct Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT> {
+struct Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT> {
     using type = std::shared_ptr<FragmentShaderState>;
 };
 
 // static
 template <>
-inline Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT>::type
-Pipeline::GetSubState<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT>(const Pipeline &pipe_state) {
+inline Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT>::type
+Pipeline::GetLibraryState<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_SHADER_BIT_EXT>(const Pipeline &pipe_state) {
     return pipe_state.fragment_shader_state;
 }
 
 template <>
-struct Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT> {
+struct Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT> {
     using type = std::shared_ptr<FragmentOutputState>;
 };
 
 // static
 template <>
-inline Pipeline::SubStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT>::type
-Pipeline::GetSubState<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT>(const Pipeline &pipe_state) {
+inline Pipeline::LibraryStateTraits<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT>::type
+Pipeline::GetLibraryState<VK_GRAPHICS_PIPELINE_LIBRARY_FRAGMENT_OUTPUT_INTERFACE_BIT_EXT>(const Pipeline &pipe_state) {
     return pipe_state.fragment_output_state;
 }
 
 }  // namespace vvl
 
-// Track last states that are bound per pipeline bind point (Gfx & Compute)
-struct LastBound {
-    LastBound(vvl::CommandBuffer &cb) : cb_state(cb) {}
-
-    vvl::CommandBuffer &cb_state;
-    vvl::Pipeline *pipeline_state = nullptr;
-    // All shader stages for a used pipeline bind point must be bound to with a valid shader or VK_NULL_HANDLE
-    // We have to track shader_object_bound, because shader_object_states will be nullptr when VK_NULL_HANDLE is used
-    bool shader_object_bound[kShaderObjectStageCount]{false};
-    vvl::ShaderObject *shader_object_states[kShaderObjectStageCount]{nullptr};
-    // The compatible layout used binding descriptor sets (track location to provide better error message)
-    VkPipelineLayout desc_set_pipeline_layout = VK_NULL_HANDLE;
-    vvl::Func desc_set_bound_command = vvl::Func::Empty;  // will be something like vkCmdBindDescriptorSets
-    std::shared_ptr<vvl::DescriptorSet> push_descriptor_set;
-
-    struct DescriptorBufferBinding {
-        uint32_t index = 0;
-        VkDeviceSize offset = 0;
-    };
-
-    // Each command buffer has a "slot" to hold a descriptor set binding. This "slot" also might be empty
-    struct DescriptorSetSlot {
-        std::shared_ptr<vvl::DescriptorSet> ds_state;
-        std::optional<DescriptorBufferBinding> descriptor_buffer_binding;
-
-        // one dynamic offset per dynamic descriptor bound to this CB
-        std::vector<uint32_t> dynamic_offsets;
-        PipelineLayoutCompatId compat_id_for_set{0};
-
-        // Cache most recently validated descriptor state for ValidateActionState/UpdateImageLayoutDrawState
-        const vvl::DescriptorSet *validated_set{nullptr};
-        uint64_t validated_set_change_count{~0ULL};
-        uint64_t validated_set_image_layout_change_count{~0ULL};
-
-        void Reset() {
-            ds_state.reset();
-            descriptor_buffer_binding.reset();
-            dynamic_offsets.clear();
-        }
-    };
-
-    // Ordered bound set tracking where index is set# that given set is bound to
-    std::vector<DescriptorSetSlot> ds_slots;
-
-    void Reset();
-
-    void UnbindAndResetPushDescriptorSet(std::shared_ptr<vvl::DescriptorSet> &&ds);
-
-    // Dynamic State helpers that require both the Pipeline and CommandBuffer state are here
-    bool IsDepthTestEnable() const;
-    bool IsDepthBoundTestEnable() const;
-    bool IsDepthWriteEnable() const;
-    bool IsDepthBiasEnable() const;
-    bool IsDepthClampEnable() const;
-    bool IsStencilTestEnable() const;
-    VkStencilOpState GetStencilOpStateFront() const;
-    VkStencilOpState GetStencilOpStateBack() const;
-    VkSampleCountFlagBits GetRasterizationSamples() const;
-    bool IsRasterizationDisabled() const;
-    bool IsLogicOpEnabled() const;
-    VkColorComponentFlags GetColorWriteMask(uint32_t i) const;
-    bool IsColorWriteEnabled(uint32_t i) const;
-    VkPrimitiveTopology GetPrimitiveTopology() const;
-    VkCullModeFlags GetCullMode() const;
-    VkConservativeRasterizationModeEXT GetConservativeRasterizationMode() const;
-    bool IsSampleLocationsEnable() const;
-    bool IsExclusiveScissorEnabled() const;
-    bool IsCoverageToColorEnabled() const;
-    bool IsCoverageModulationTableEnable() const;
-    bool IsDiscardRectangleEnable() const;
-    bool IsStippledLineEnable() const;
-    bool IsShadingRateImageEnable() const;
-    bool IsViewportWScalingEnable() const;
-    bool IsPrimitiveRestartEnable() const;
-    bool IsAlphaToCoverage() const;
-    VkCoverageModulationModeNV GetCoverageModulationMode() const;
-
-    bool ValidShaderObjectCombination(const VkPipelineBindPoint bind_point, const DeviceFeatures &device_features) const;
-    VkShaderEXT GetShader(ShaderObjectStage stage) const;
-    vvl::ShaderObject *GetShaderState(ShaderObjectStage stage) const;
-    const vvl::ShaderObject *GetShaderStateIfValid(ShaderObjectStage stage) const;
-    // Return compute shader for compute pipeline, vertex or mesh shader for graphics
-    const vvl::ShaderObject *GetFirstShader(VkPipelineBindPoint bind_point) const;
-    bool HasShaderObjects() const;
-    bool IsValidShaderBound(ShaderObjectStage stage) const;
-    bool IsValidShaderOrNullBound(ShaderObjectStage stage) const;
-    std::vector<vvl::ShaderObject *> GetAllBoundGraphicsShaders();
-    bool IsAnyGraphicsShaderBound() const;
-    VkShaderStageFlags GetAllActiveBoundStages() const;
-
-    bool IsBoundSetCompatible(uint32_t set, const vvl::PipelineLayout &pipeline_layout) const;
-    bool IsBoundSetCompatible(uint32_t set, const vvl::ShaderObject &shader_object_state) const;
-    std::string DescribeNonCompatibleSet(uint32_t set, const vvl::PipelineLayout &pipeline_layout) const;
-    std::string DescribeNonCompatibleSet(uint32_t set, const vvl::ShaderObject &shader_object_state) const;
-
-    const spirv::EntryPoint *GetVertexEntryPoint() const;
-    const spirv::EntryPoint *GetFragmentEntryPoint() const;
-
-    // For GPU-AV
-    bool WasInstrumented() const;
-};
-
 // Used to compare 2 layouts independently when not tied to the last bound object
 bool IsPipelineLayoutSetCompatible(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b);
 std::string DescribePipelineLayoutSetNonCompatible(uint32_t set, const vvl::PipelineLayout *a, const vvl::PipelineLayout *b);
-
-enum LvlBindPoint {
-    BindPoint_Graphics = VK_PIPELINE_BIND_POINT_GRAPHICS,
-    BindPoint_Compute = VK_PIPELINE_BIND_POINT_COMPUTE,
-    BindPoint_Ray_Tracing = 2,
-    BindPoint_Count = 3,
-};
-
-static VkPipelineBindPoint inline ConvertToPipelineBindPoint(LvlBindPoint bind_point) {
-    switch (bind_point) {
-        case BindPoint_Graphics:
-            return VK_PIPELINE_BIND_POINT_GRAPHICS;
-        case BindPoint_Compute:
-            return VK_PIPELINE_BIND_POINT_COMPUTE;
-        case BindPoint_Ray_Tracing:
-            return VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR;
-        default:
-            break;
-    }
-    assert(false);
-    return VK_PIPELINE_BIND_POINT_GRAPHICS;
-}
-
-static LvlBindPoint inline ConvertToLvlBindPoint(VkPipelineBindPoint bind_point) {
-    switch (bind_point) {
-        case VK_PIPELINE_BIND_POINT_GRAPHICS:
-            return BindPoint_Graphics;
-        case VK_PIPELINE_BIND_POINT_COMPUTE:
-            return BindPoint_Compute;
-        case VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR:
-            return BindPoint_Ray_Tracing;
-        default:
-            break;
-    }
-    assert(false);
-    return BindPoint_Graphics;
-}
-
-static VkPipelineBindPoint inline ConvertToPipelineBindPoint(VkShaderStageFlagBits stage) {
-    switch (stage) {
-        case VK_SHADER_STAGE_VERTEX_BIT:
-        case VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT:
-        case VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT:
-        case VK_SHADER_STAGE_GEOMETRY_BIT:
-        case VK_SHADER_STAGE_FRAGMENT_BIT:
-        case VK_SHADER_STAGE_TASK_BIT_EXT:
-        case VK_SHADER_STAGE_MESH_BIT_EXT:
-            return VK_PIPELINE_BIND_POINT_GRAPHICS;
-        case VK_SHADER_STAGE_COMPUTE_BIT:
-            return VK_PIPELINE_BIND_POINT_COMPUTE;
-        case VK_SHADER_STAGE_RAYGEN_BIT_KHR:
-        case VK_SHADER_STAGE_ANY_HIT_BIT_KHR:
-        case VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR:
-        case VK_SHADER_STAGE_MISS_BIT_KHR:
-        case VK_SHADER_STAGE_INTERSECTION_BIT_KHR:
-        case VK_SHADER_STAGE_CALLABLE_BIT_KHR:
-            return VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR;
-        default:
-            break;
-    }
-    assert(false);
-    return VK_PIPELINE_BIND_POINT_GRAPHICS;
-}
-
-static VkPipelineBindPoint inline ConvertToPipelineBindPoint(VkShaderStageFlags stage) {
-    // Assumes the call has checked stages have not been mixed
-    if (stage & kShaderStageAllGraphics) {
-        return VK_PIPELINE_BIND_POINT_GRAPHICS;
-    } else if (stage & VK_SHADER_STAGE_COMPUTE_BIT) {
-        return VK_PIPELINE_BIND_POINT_COMPUTE;
-    } else if (stage & kShaderStageAllRayTracing) {
-        return VK_PIPELINE_BIND_POINT_RAY_TRACING_KHR;
-    } else {
-        assert(false);
-        return VK_PIPELINE_BIND_POINT_GRAPHICS;
-    }
-}
-static LvlBindPoint inline ConvertToLvlBindPoint(VkShaderStageFlagBits stage) {
-    switch (stage) {
-        case VK_SHADER_STAGE_VERTEX_BIT:
-        case VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT:
-        case VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT:
-        case VK_SHADER_STAGE_GEOMETRY_BIT:
-        case VK_SHADER_STAGE_FRAGMENT_BIT:
-        case VK_SHADER_STAGE_TASK_BIT_EXT:
-        case VK_SHADER_STAGE_MESH_BIT_EXT:
-            return BindPoint_Graphics;
-        case VK_SHADER_STAGE_COMPUTE_BIT:
-            return BindPoint_Compute;
-        case VK_SHADER_STAGE_RAYGEN_BIT_KHR:
-        case VK_SHADER_STAGE_ANY_HIT_BIT_KHR:
-        case VK_SHADER_STAGE_CLOSEST_HIT_BIT_KHR:
-        case VK_SHADER_STAGE_MISS_BIT_KHR:
-        case VK_SHADER_STAGE_INTERSECTION_BIT_KHR:
-        case VK_SHADER_STAGE_CALLABLE_BIT_KHR:
-            return BindPoint_Ray_Tracing;
-        default:
-            break;
-    }
-    assert(false);
-    return BindPoint_Graphics;
-}
