@@ -15,11 +15,12 @@
  */
 
 #include "shader_object_state.h"
+#include <vulkan/vulkan_core.h>
 #include "shader_module.h"
 #include "state_tracker/state_tracker.h"
 
 namespace vvl {
-static ShaderObject::SetLayoutVector GetSetLayouts(Device &dev_data, const VkShaderCreateInfoEXT &pCreateInfo) {
+static ShaderObject::SetLayoutVector GetSetLayouts(DeviceState &dev_data, const VkShaderCreateInfoEXT &pCreateInfo) {
     ShaderObject::SetLayoutVector set_layouts(pCreateInfo.setLayoutCount);
 
     for (uint32_t i = 0; i < pCreateInfo.setLayoutCount; ++i) {
@@ -28,8 +29,8 @@ static ShaderObject::SetLayoutVector GetSetLayouts(Device &dev_data, const VkSha
     return set_layouts;
 }
 
-ShaderObject::ShaderObject(Device &dev_data, const VkShaderCreateInfoEXT &create_info_i, VkShaderEXT handle,
-                           std::shared_ptr<spirv::Module> &spirv_module, uint32_t createInfoCount, VkShaderEXT *pShaders)
+ShaderObject::ShaderObject(DeviceState &dev_data, const VkShaderCreateInfoEXT &create_info_i, VkShaderEXT handle,
+                           std::shared_ptr<spirv::Module> &spirv_module)
     : StateObject(handle, kVulkanObjectTypeShaderEXT),
       safe_create_info(&create_info_i),
       create_info(*safe_create_info.ptr()),
@@ -39,23 +40,11 @@ ShaderObject::ShaderObject(Device &dev_data, const VkShaderCreateInfoEXT &create
       max_active_slot(GetMaxActiveSlot(active_slots)),
       set_layouts(GetSetLayouts(dev_data, create_info)),
       push_constant_ranges(GetCanonicalId(create_info.pushConstantRangeCount, create_info.pPushConstantRanges)),
-      set_compat_ids(GetCompatForSet(set_layouts, push_constant_ranges)) {
-    if ((create_info.flags & VK_SHADER_CREATE_LINK_STAGE_BIT_EXT) != 0) {
-        for (uint32_t i = 0; i < createInfoCount; ++i) {
-            if (pShaders[i] != handle) {
-                linked_shaders.push_back(pShaders[i]);
-            }
-        }
+      set_compat_ids(GetCompatForSet(set_layouts, push_constant_ranges, 0)) {
+    // We need to update handle, but if using VK_SHADER_CODE_TYPE_SPIRV_EXT, it will be null
+    if (spirv_module) {
+        spirv_module->handle_ = handle_;
     }
 }
 
-VkPrimitiveTopology ShaderObject::GetTopology() const {
-    if (spirv) {
-        const auto topology = spirv->GetTopology(*entrypoint);
-        if (topology) {
-            return *topology;
-        }
-    }
-    return VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
-}
 }  // namespace vvl
