@@ -94,84 +94,68 @@ bool ForEachEntryInRangesUntil(const RangeMap &map, RangeGen &range_gen, Action
     return false;
 }
 
-struct NoopBarrierAction {
-    explicit NoopBarrierAction() {}
-    void operator()(ResourceAccessState *access) const {}
-    const bool layout_transition = false;
-};
+// This functor changes layout of access map as part of infill_update_range traversal:
+//    * infills gaps within a specified input range (only for layout transition use case)
+//    * if existing ranges intersect begin/end of the input range then existing ranges are split
+//
+// Besides layout changes this functor does not update access state.
+// The subsequent functors that will run over the same input range will have a guarantee that access
+// map structure won't be modified, so they can collect persistent references to access states.
+//
+// NOTE: notice, when this functor is used for not layout transtiion barrier it is not a noop: yes,
+//  Infill operation is used only for layout transition, but infill_update_range can additionally
+//  perform up to two splits if input range intersects access map existing ranges.
+struct ApplyMarkupFunctor {
+    ApplyMarkupFunctor(bool layout_transition) : layout_transition(layout_transition) {}
 
-// This functor applies a collection of barriers, updating the "pending state" in each touched memory range, and optionally
-// resolves the pending state. Suitable for processing Global memory barriers, or Subpass Barriers when the "final" barrier
-// of a collection is known/present.
-template <typename BarrierOp, typename OpVector = std::vector<BarrierOp>>
-class ApplyBarrierOpsFunctor {
-  public:
     using Iterator = ResourceAccessRangeMap::iterator;
-    // Only called with a gap, and pos at the lower_bound(range)
-    inline Iterator Infill(ResourceAccessRangeMap *accesses, const Iterator &pos, const ResourceAccessRange &range) const {
-        if (!infill_default_) {
-            return pos;
+    Iterator Infill(ResourceAccessRangeMap *accesses, const Iterator &pos_hint, const ResourceAccessRange &range) const {
+        if (!layout_transition) {
+            return pos_hint;
         }
         ResourceAccessState default_state;
-        auto inserted = accesses->insert(pos, std::make_pair(range, default_state));
+        auto inserted = accesses->insert(pos_hint, std::make_pair(range, default_state));
         return inserted;
     }
+    void operator()(const Iterator &pos) const {}
+    const bool layout_transition;
+};
+
+// This functor populates PendingBarriers with the results of independent barrier appication (pending barriers).
+// After this functor finished its work then PendingBarriers::Apply() can be used to update the access states.
+struct CollectBarriersFunctor {
+    CollectBarriersFunctor(const BarrierScope &barrier_scope, const SyncBarrier &barrier, bool layout_transition,
+                           uint32_t layout_transition_handle_index, PendingBarriers &pending_barriers)
+        : barrier_scope(barrier_scope),
+          barrier(barrier),
+          layout_transition(layout_transition),
+          layout_transition_handle_index(layout_transition_handle_index),
+          pending_barriers(pending_barriers) {
+        // Suppress layout transition during submit time application.
+        // It add write access but this is necessary only during recording.
+        if (barrier_scope.scope_queue != kQueueIdInvalid) {
+            this->layout_transition = false;
+            this->layout_transition_handle_index = vvl::kNoIndex32;
+        }
+    }
+
+    using Iterator = ResourceAccessRangeMap::iterator;
+    Iterator Infill(ResourceAccessRangeMap *accesses, const Iterator &pos_hint, const ResourceAccessRange &range) const {
+        assert(!layout_transition);  // MarkupFunctor infills gaps for layout transtion, so we should never get here in that case
+        return pos_hint;
+    }
 
     void operator()(const Iterator &pos) const {
-        auto &access_state = pos->second;
-        for (const auto &op : barrier_ops_) {
-            op(&access_state);
-        }
-
-        if (resolve_) {
-            // If this is the last (or only) batch, we can do the pending resolve as the last step in this operation to avoid
-            // another walk
-            access_state.ApplyPendingBarriers(tag_);
-        }
+        ResourceAccessState &access_state = pos->second;
+        access_state.CollectPendingBarriers(barrier_scope, barrier, layout_transition, layout_transition_handle_index,
+                                            pending_barriers);
     }
 
-    // A valid tag is required IFF layout_transition is true, as transitions are write ops
-    ApplyBarrierOpsFunctor(bool resolve, typename OpVector::size_type size_hint, ResourceUsageTag tag)
-        : resolve_(resolve), infill_default_(false), barrier_ops_(), tag_(tag) {
-        barrier_ops_.reserve(size_hint);
-    }
-    void EmplaceBack(const BarrierOp &op) {
-        barrier_ops_.emplace_back(op);
-        infill_default_ |= op.layout_transition;
-    }
-
-  private:
-    bool resolve_;
-    bool infill_default_;
-    OpVector barrier_ops_;
-    const ResourceUsageTag tag_;
-};
-
-// This functor applies a single barrier, updating the "pending state" in each touched memory range, but does not
-// resolve the pendinging state. Suitable for processing Image and Buffer barriers from PipelineBarriers or Events
-template <typename BarrierOp>
-class ApplyBarrierFunctor : public ApplyBarrierOpsFunctor<BarrierOp, small_vector<BarrierOp, 1>> {
-    using Base = ApplyBarrierOpsFunctor<BarrierOp, small_vector<BarrierOp, 1>>;
-
-  public:
-    ApplyBarrierFunctor(const BarrierOp &barrier_op) : Base(false, 1, kInvalidTag) { Base::EmplaceBack(barrier_op); }
-};
-
-// This functor resolves the pendinging state.
-class ResolvePendingBarrierFunctor : public ApplyBarrierOpsFunctor<NoopBarrierAction, small_vector<NoopBarrierAction, 1>> {
-    using Base = ApplyBarrierOpsFunctor<NoopBarrierAction, small_vector<NoopBarrierAction, 1>>;
-
-  public:
-    ResolvePendingBarrierFunctor(ResourceUsageTag tag) : Base(true, 0, tag) {}
-};
-
-struct ApplySubpassTransitionBarriersAction {
-    explicit ApplySubpassTransitionBarriersAction(const std::vector<SyncBarrier> &barriers_) : barriers(barriers_) {}
-    void operator()(ResourceAccessState *access) const {
-        assert(access);
-        access->ApplyBarriers(barriers, true);
-    }
-    const std::vector<SyncBarrier> &barriers;
+    const BarrierScope barrier_scope;
+    const SyncBarrier barrier;
+    bool layout_transition;
+    uint32_t layout_transition_handle_index;
+    PendingBarriers &pending_barriers;
 };
 
 struct QueueTagOffsetBarrierAction {
@@ -190,10 +174,7 @@ struct ApplyTrackbackStackAction {
         : barriers(barriers_), previous_barrier(previous_barrier_) {}
     void operator()(ResourceAccessState *access) const {
         assert(access);
-        assert(!access->HasPendingState());
-        access->ApplyBarriers(barriers, false);
-        // NOTE: We can use invalid tag, as these barriers do no include layout transitions (would assert in SetWrite)
-        access->ApplyPendingBarriers(kInvalidTag);
+        ApplyBarriers(*access, barriers);
         if (previous_barrier) {
             assert(bool(*previous_barrier));
             (*previous_barrier)(access);
@@ -203,24 +184,21 @@ struct ApplyTrackbackStackAction {
     const ResourceAccessStateFunction *previous_barrier;
 };
 
-template <typename SubpassNode>
+class AccessContext;
+
 struct SubpassBarrierTrackback {
     std::vector<SyncBarrier> barriers;
-    const SubpassNode *source_subpass = nullptr;
+    const AccessContext *source_subpass = nullptr;
     SubpassBarrierTrackback() = default;
-    SubpassBarrierTrackback(const SubpassBarrierTrackback &) = default;
-    SubpassBarrierTrackback(const SubpassNode *source_subpass_, VkQueueFlags queue_flags_,
-                            const std::vector<const VkSubpassDependency2 *> &subpass_dependencies_)
-        : barriers(), source_subpass(source_subpass_) {
-        barriers.reserve(subpass_dependencies_.size());
-        for (const VkSubpassDependency2 *dependency : subpass_dependencies_) {
+    SubpassBarrierTrackback(const AccessContext *source_subpass, VkQueueFlags queue_flags,
+                            const std::vector<const VkSubpassDependency2 *> &subpass_dependencies)
+        : source_subpass(source_subpass) {
+        barriers.reserve(subpass_dependencies.size());
+        for (const VkSubpassDependency2 *dependency : subpass_dependencies) {
             assert(dependency);
-            barriers.emplace_back(queue_flags_, *dependency);
+            barriers.emplace_back(queue_flags, *dependency);
         }
     }
-    SubpassBarrierTrackback(const SubpassNode *source_subpass_, const SyncBarrier &barrier_)
-        : barriers(1, barrier_), source_subpass(source_subpass_) {}
-    SubpassBarrierTrackback &operator=(const SubpassBarrierTrackback &) = default;
 };
 
 class AttachmentViewGen {
@@ -250,8 +228,6 @@ class AccessContext {
         kDetectAll = (kDetectPrevious | kDetectAsync)
     };
 
-    using TrackBack = SubpassBarrierTrackback<AccessContext>;
-
     HazardResult DetectHazard(const vvl::Buffer &buffer, SyncAccessIndex access_index, const ResourceAccessRange &range) const;
     HazardResult DetectHazard(const vvl::Image &image, SyncAccessIndex current_usage,
                               const VkImageSubresourceRange &subresource_range, bool is_depth_sliced) const;
@@ -276,19 +252,21 @@ class AccessContext {
                                           DetectOptions options) const;
     HazardResult DetectImageBarrierHazard(const vvl::Image &image, VkPipelineStageFlags2 src_exec_scope,
                                           const SyncAccessFlags &src_access_scope, const VkImageSubresourceRange &subresource_range,
-                                          DetectOptions options) const;
-    HazardResult DetectSubpassTransitionHazard(const TrackBack &track_back, const AttachmentViewGen &attach_view) const;
+                                          bool is_depth_sliced, DetectOptions options) const;
+    HazardResult DetectSubpassTransitionHazard(const SubpassBarrierTrackback &track_back,
+                                               const AttachmentViewGen &attach_view) const;
 
     HazardResult DetectFirstUseHazard(QueueId queue_id, const ResourceUsageRange &tag_range,
                                       const AccessContext &access_context) const;
+    HazardResult DetectMarkerHazard(const vvl::Buffer &buffer, const ResourceAccessRange &range) const;
 
-    const TrackBack &GetDstExternalTrackBack() const { return dst_external_; }
+    const SubpassBarrierTrackback &GetDstExternalTrackBack() const { return dst_external_; }
     void Reset() {
         prev_.clear();
         prev_by_subpass_.clear();
         async_.clear();
         src_external_ = nullptr;
-        dst_external_ = TrackBack();
+        dst_external_ = SubpassBarrierTrackback();
         start_tag_ = ResourceUsageTag();
         access_state_map_.clear();
     }
@@ -305,7 +283,7 @@ class AccessContext {
                             bool recur_to_infill = false);
 
     void UpdateAccessState(const vvl::Buffer &buffer, SyncAccessIndex current_usage, SyncOrdering ordering_rule,
-                           const ResourceAccessRange &range, ResourceUsageTagEx tag_ex);
+                           const ResourceAccessRange &range, ResourceUsageTagEx tag_ex, SyncFlags flags = 0);
     void UpdateAccessState(const vvl::Image &image, SyncAccessIndex current_usage, SyncOrdering ordering_rule,
                            const VkImageSubresourceRange &subresource_range, const ResourceUsageTag &tag);
     void UpdateAccessState(const vvl::Image &image, SyncAccessIndex current_usage, SyncOrdering ordering_rule,
@@ -327,10 +305,6 @@ class AccessContext {
 
     void ImportAsyncContexts(const AccessContext &from);
     void ClearAsyncContexts() { async_.clear(); }
-    template <typename Action>
-    void ApplyUpdateAction(const AttachmentViewGen &view_gen, AttachmentViewGen::Gen gen_type, const Action &action);
-    template <typename Action>
-    void ApplyToContext(const Action &barrier_action);
 
     AccessContext(uint32_t subpass, VkQueueFlags queue_flags, const std::vector<SubpassDependencyGraphNode> &dependencies,
                   const std::vector<AccessContext> &contexts, const AccessContext *external_context);
@@ -343,7 +317,7 @@ class AccessContext {
 
     ResourceAccessRangeMap &GetAccessStateMap() { return access_state_map_; }
     const ResourceAccessRangeMap &GetAccessStateMap() const { return access_state_map_; }
-    const TrackBack *GetTrackBackFromSubpass(uint32_t subpass) const {
+    const SubpassBarrierTrackback *GetTrackBackFromSubpass(uint32_t subpass) const {
         if (subpass == VK_SUBPASS_EXTERNAL) {
             return src_external_;
         } else {
@@ -381,16 +355,17 @@ class AccessContext {
         QueueId queue_id_;
     };
 
+    template <typename Action>
+    void UpdateMemoryAccessRangeState(Action &action, const ResourceAccessRange &range);
+
     template <typename Action, typename RangeGen>
     void UpdateMemoryAccessState(const Action &action, RangeGen &range_gen);
 
   private:
-    template <typename Action>
-    static void UpdateMemoryAccessRangeState(ResourceAccessRangeMap &accesses, Action &action, const ResourceAccessRange &range);
 
     struct UpdateMemoryAccessStateFunctor {
         using Iterator = ResourceAccessRangeMap::iterator;
-        Iterator Infill(ResourceAccessRangeMap *accesses, const Iterator &pos, const ResourceAccessRange &range) const;
+        Iterator Infill(ResourceAccessRangeMap *accesses, const Iterator &pos_hint, const ResourceAccessRange &range) const;
         void operator()(const Iterator &pos) const;
         UpdateMemoryAccessStateFunctor(const AccessContext &context_, SyncAccessIndex usage_, SyncOrdering ordering_rule_,
                                        ResourceUsageTagEx tag_ex, SyncFlags flags = 0)
@@ -403,7 +378,8 @@ class AccessContext {
     };
 
     // Follow the context previous to access the access state, supporting "lazy" import into the context. Not intended for
-    // subpass layout transition, as the pending state handling is more complex
+    // subpass layout transition, as the pending state handling is more complex (TODO: check if previous statement is
+    // still true after pending barriers rework).
     // TODO: See if returning the lower_bound would be useful from a performance POV -- look at the lower_bound overhead
     // Would need to add a "hint" overload to parallel_iterator::invalidate_[AB] call, if so.
     void ResolvePreviousAccess(const ResourceAccessRange &range, ResourceAccessRangeMap *descent_map,
@@ -452,12 +428,12 @@ class AccessContext {
     HazardResult DetectPreviousHazard(Detector &detector, const ResourceAccessRange &range) const;
 
     ResourceAccessRangeMap access_state_map_;
-    std::vector<TrackBack> prev_;
-    std::vector<TrackBack *> prev_by_subpass_;
+    std::vector<SubpassBarrierTrackback> prev_;
+    std::vector<SubpassBarrierTrackback *> prev_by_subpass_;
     // These contexts *must* have the same lifespan as this context, or be cleared, before the referenced contexts can expire
     std::vector<AsyncReference> async_;
-    TrackBack *src_external_;
-    TrackBack dst_external_;
+    SubpassBarrierTrackback *src_external_;
+    SubpassBarrierTrackback dst_external_;
     ResourceUsageTag start_tag_;
 };
 
@@ -487,30 +463,17 @@ struct ActionToOpsAdapter {
 };
 
 template <typename Action>
-void AccessContext::ApplyToContext(const Action &barrier_action) {
-    // Note: Barriers do *not* cross context boundaries, applying to accessess within.... (at least for renderpass subpasses)
-    UpdateMemoryAccessRangeState(access_state_map_, barrier_action, kFullRange);
-}
-
-template <typename Action>
-void AccessContext::UpdateMemoryAccessRangeState(ResourceAccessRangeMap &accesses, Action &action,
-                                                 const ResourceAccessRange &range) {
+void AccessContext::UpdateMemoryAccessRangeState(Action &action, const ResourceAccessRange &range) {
     ActionToOpsAdapter<Action> ops{action};
-    infill_update_range(accesses, range, ops);
+    infill_update_range(access_state_map_, range, ops);
 }
 
 template <typename Action, typename RangeGen>
 void AccessContext::UpdateMemoryAccessState(const Action &action, RangeGen &range_gen) {
     ActionToOpsAdapter<Action> ops{action};
-    infill_update_rangegen(access_state_map_, range_gen, ops);
-}
-
-template <typename Action>
-void AccessContext::ApplyUpdateAction(const AttachmentViewGen &view_gen, AttachmentViewGen::Gen gen_type, const Action &action) {
-    const std::optional<ImageRangeGen> &ref_range_gen = view_gen.GetRangeGen(gen_type);
-    if (ref_range_gen) {
-        ImageRangeGen range_gen(*ref_range_gen);
-        UpdateMemoryAccessState(action, range_gen);
+    auto pos = access_state_map_.lower_bound(*range_gen);
+    for (; range_gen->non_empty(); ++range_gen) {
+        pos = infill_update_range(access_state_map_, pos, *range_gen, ops);
     }
 }
 
@@ -577,6 +540,24 @@ HazardResult AccessContext::DetectHazardOneRange(Detector &detector, bool detect
     return hazard;
 }
 
+// A wrapper for a single range with the same semantics as other non-trivial range generators
+template <typename KeyType>
+class SingleRangeGenerator {
+  public:
+    using RangeType = KeyType;
+    SingleRangeGenerator(const KeyType &range) : current_(range) {}
+    const KeyType &operator*() const { return current_; }
+    const KeyType *operator->() const { return &current_; }
+    SingleRangeGenerator &operator++() {
+        current_ = KeyType();  // just one real range
+        return *this;
+    }
+
+  private:
+    SingleRangeGenerator() = default;
+    KeyType current_;
+};
+
 template <typename Detector>
 HazardResult AccessContext::DetectHazardRange(Detector &detector, const ResourceAccessRange &range, DetectOptions options) const {
     SingleRangeGenerator range_gen(range);
