@@ -19,10 +19,14 @@
 #include "sync/sync_common.h"
 
 class ResourceAccessState;
-class WriteState;
+struct WriteState;
 struct ReadState;
 struct ResourceFirstAccess;
 
+namespace syncval_stats {
+struct AccessContextStats;
+}  // namespace syncval_stats
+
 // NOTE: the attachement read flag is put *only* in the access scope and not in the exect scope, since the ordering
 //       rules apply only to this specific access for this stage, and not the stage as a whole. The ordering detection
 //       also reflects this special case for read hazard detection (using access instead of exec scope)
@@ -66,8 +70,10 @@ enum class SyncOrdering : uint8_t {
 
 struct SyncFlag {
     enum : uint32_t {
-        kLoadOp = 1u << 0,
-        kStoreOp = 1u << 1,
+        kLoadOp = 0x01,
+        kStoreOp = 0x02,
+        kPresent = 0x04,
+        kMarker = 0x08,
     };
 };
 using SyncFlags = uint32_t;
@@ -136,13 +142,9 @@ class HazardResult {
 };
 
 struct SyncExecScope {
-    VkPipelineStageFlags2 mask_param;  // the xxxStageMask parameter passed by the caller
-    VkPipelineStageFlags2 exec_scope;  // all earlier or later stages that would be affected by a barrier using this scope.
-    SyncAccessFlags valid_accesses;    // all valid accesses that can be used with this scope.
-
-    SyncExecScope() : mask_param(0), exec_scope(0), valid_accesses(0) {}
-    SyncExecScope(VkPipelineStageFlags2 mask_param, VkPipelineStageFlags2 exec_scope, const SyncAccessFlags &valid_accesses)
-        : mask_param(mask_param), exec_scope(exec_scope), valid_accesses(valid_accesses) {}
+    VkPipelineStageFlags2 mask_param = 0;  // the xxxStageMask parameter passed by the caller
+    VkPipelineStageFlags2 exec_scope = 0;  // all earlier or later stages that would be affected by a barrier using this scope.
+    SyncAccessFlags valid_accesses;        // all valid accesses that can be used with this scope.
 
     static SyncExecScope MakeSrc(VkQueueFlags queue_flags, VkPipelineStageFlags2 src_stage_mask,
                                  const VkPipelineStageFlags2 disabled_feature_mask = 0);
@@ -171,16 +173,37 @@ struct SyncBarrier {
     SyncBarrier(const std::vector<SyncBarrier> &barriers);
 };
 
+// Defines the source scope of the barrier.
+// ReadState and WriteState have InBarrierSourceScope() that checks if corresponding access is in the barrier source scope.
+struct BarrierScope {
+    VkPipelineStageFlagBits2 src_exec_scope;
+    SyncAccessFlags src_access_scope;
+
+    // Scope queue is used to include accesses only from the specific queue.
+    // The check against queue scope is unified for all cases. During record time the scope queue
+    // has default value (Invalid). This matches how the queue member of read/write accesses is
+    // initialized during recording, so (access_queue == scope_queue) evaluates to true during record time.
+    QueueId scope_queue = kQueueIdInvalid;
+
+    // The tag is needed for the event scope logic. The scope tag is defined by the "set event" command.
+    // The check against scope tag is unified for all cases. For non event code the scope tag is uint64-max
+    // value, so (access_tag < scope_tag) evaluates to true for non event code.
+    ResourceUsageTag scope_tag = kInvalidTag;
+
+    BarrierScope(const SyncBarrier &barrier, QueueId scope_queue = kQueueIdInvalid, ResourceUsageTag scope_tag = kInvalidTag);
+};
+
 struct ResourceFirstAccess {
     const SyncAccessInfo *usage_info;
     ResourceUsageTag tag;
     uint32_t handle_index;
     SyncOrdering ordering_rule;
+    SyncFlags flags;
 
-    ResourceFirstAccess(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncOrdering ordering_rule)
-        : usage_info(&usage_info), tag(tag_ex.tag), handle_index(tag_ex.handle_index), ordering_rule(ordering_rule) {}
+    ResourceFirstAccess(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncOrdering ordering_rule, SyncFlags flags)
+        : usage_info(&usage_info), tag(tag_ex.tag), handle_index(tag_ex.handle_index), ordering_rule(ordering_rule), flags(flags) {}
     bool operator==(const ResourceFirstAccess &rhs) const {
-        return (tag == rhs.tag) && (usage_info == rhs.usage_info) && (ordering_rule == rhs.ordering_rule);
+        return (tag == rhs.tag) && (usage_info == rhs.usage_info) && (ordering_rule == rhs.ordering_rule) && flags == rhs.flags;
     }
     ResourceUsageTagEx TagEx() const { return {tag, handle_index}; }
 };
@@ -203,6 +226,8 @@ struct OrderingBarrier {
     }
 };
 
+const OrderingBarrier &GetOrderingRules(SyncOrdering ordering_enum);
+
 using ResourceUsageTagSet = CachedInsertSet<ResourceUsageTag, 4>;
 
 // Mutliple read operations can be simlutaneously (and independently) synchronized,
@@ -210,26 +235,21 @@ using ResourceUsageTagSet = CachedInsertSet<ResourceUsageTag, 4>;
 // but only up to one per pipeline stage (as another read from the *same* stage become more recent,
 // and applicable one for hazard detection
 struct ReadState {
-    VkPipelineStageFlags2 stage;        // The stage of this read
+    VkPipelineStageFlagBits2 stage;     // The stage of this read
     SyncAccessIndex access_index;       // TODO: Revisit whether this needs to support multiple reads per stage
     VkPipelineStageFlags2 barriers;     // all applicable barriered stages
     VkPipelineStageFlags2 sync_stages;  // reads known to have happened after this
     ResourceUsageTag tag;
     uint32_t handle_index;
     QueueId queue;
-    VkPipelineStageFlags2 pending_dep_chain;  // Should be zero except during barrier application
-                                              // Excluded from comparison
-    ReadState() = default;
-    ReadState(VkPipelineStageFlags2 stage, SyncAccessIndex access_index, ResourceUsageTagEx tag_ex);
-    void Set(VkPipelineStageFlags2 stage, SyncAccessIndex access_index, ResourceUsageTagEx tag_ex);
+
+    void Set(VkPipelineStageFlagBits2 stage, SyncAccessIndex access_index, ResourceUsageTagEx tag_ex);
 
     ResourceUsageTagEx TagEx() const { return {tag, handle_index}; }
     bool operator==(const ReadState &rhs) const {
         return (stage == rhs.stage) && (access_index == rhs.access_index) && (barriers == rhs.barriers) &&
-               (sync_stages == rhs.sync_stages) && (tag == rhs.tag) && (queue == rhs.queue) &&
-               (pending_dep_chain == rhs.pending_dep_chain);
+               (sync_stages == rhs.sync_stages) && (tag == rhs.tag) && (queue == rhs.queue);
     }
-    void Normalize() { pending_dep_chain = VK_PIPELINE_STAGE_2_NONE; }
     bool IsReadBarrierHazard(VkPipelineStageFlags2 src_exec_scope) const {
         // If the read stage is not in the src sync scope
         // *AND* not execution chained with an existing sync barrier (that's the or)
@@ -258,89 +278,103 @@ struct ReadState {
 
         return (src_exec_scope & (queue_ordered_stage | barriers)) == 0;
     }
-    bool ReadInScopeOrChain(VkPipelineStageFlags2 exec_scope) const { return (exec_scope & (stage | barriers)) != 0; }
-    bool ReadInQueueScopeOrChain(QueueId queue, VkPipelineStageFlags2 exec_scope) const;
-    bool ReadInEventScope(VkPipelineStageFlags2 exec_scope, QueueId scope_queue, ResourceUsageTag scope_tag) const {
-        // If this read is the same one we included in the set event and in scope, then apply the execution barrier...
-        // NOTE: That's not really correct... this read stage might *not* have been included in the setevent, and the barriers
-        // representing the chain might have changed since then (that would be an odd usage), so as a first approximation
-        // we'll assume the barriers *haven't* been changed since (if the tag hasn't), and while this could be a false
-        // positive in the case of Set; SomeBarrier; Wait; we'll live with it until we can add more state to the first scope
-        // capture (the specific write and read stages that *were* in scope at the moment of SetEvents.
-        return (tag < scope_tag) && ReadInQueueScopeOrChain(scope_queue, exec_scope);
-    }
-    void ApplyReadBarrier(VkPipelineStageFlags2 dst_scope) { pending_dep_chain |= dst_scope; }
-    VkPipelineStageFlags2 ApplyPendingBarriers();
+    bool ReadOrDependencyChainInSourceScope(QueueId queue, VkPipelineStageFlags2 src_exec_scope) const;
+    bool InBarrierSourceScope(const BarrierScope &barrier_scope) const;
 };
 
-class WriteState {
-  public:
-    WriteState() = default;
-    WriteState(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncFlags flags = 0);
+static_assert(std::is_trivially_copyable_v<ReadState>);
 
-    bool operator==(const WriteState &rhs) const {
-        return (access_ == rhs.access_) && (barriers_ == rhs.barriers_) && (tag_ == rhs.tag_) && (queue_ == rhs.queue_) &&
-               (dependency_chain_ == rhs.dependency_chain_);
-    }
-    bool WriteInChain(VkPipelineStageFlags2 src_exec_scope) const;
-    bool WriteInScope(const SyncAccessFlags &src_access_scope) const;
-    bool WriteInSourceScopeOrChain(VkPipelineStageFlags2 src_exec_scope, SyncAccessFlags src_access_scope) const;
-    bool WriteInQueueSourceScopeOrChain(QueueId queue, VkPipelineStageFlags2 src_exec_scope,
-                                        const SyncAccessFlags &src_access_scope) const;
+struct WriteState {
+    SyncAccessIndex access_index;
+    SyncFlags flags;
 
-    bool WriteInEventScope(VkPipelineStageFlags2 src_exec_scope, const SyncAccessFlags &src_access_scope, QueueId scope_queue,
-                           ResourceUsageTag scope_tag) const;
+    // Union of applicable barrier masks since last write
+    SyncAccessFlags barriers;
+
+    // Accumulates the dstStages of barriers if they chain
+    VkPipelineStageFlags2 dependency_chain;
+
+    ResourceUsageTag tag;
+    uint32_t handle_index;
+    QueueId queue;
+
+    void Set(SyncAccessIndex access_index, ResourceUsageTagEx tag_ex, SyncFlags flags);
+    void SetQueueId(QueueId id);
+    void MergeBarriers(const WriteState &other);
+
+    bool operator==(const WriteState &rhs) const;
+
+    bool DependencyChainInSourceScope(VkPipelineStageFlags2 src_exec_scope) const;
+    bool WriteInSourceScope(const SyncAccessFlags &src_access_scope) const;
+    bool WriteOrDependencyChainInSourceScope(QueueId queue_id, VkPipelineStageFlags2 src_exec_scope,
+                                             const SyncAccessFlags &src_access_scope) const;
+    bool InBarrierSourceScope(const BarrierScope &barrier_scope) const;
 
-    SyncAccessIndex Index() const { return access_->access_index; }
-    bool IsIndex(SyncAccessIndex access_index) const { return Index() == access_index; }
-    bool IsQueue(QueueId other_queue) const { return queue_ == other_queue; }
-    const SyncAccessInfo &Access() const { return *access_; }
-    const SyncAccessFlags &Barriers() const { return barriers_; }
-    ResourceUsageTag Tag() const { return tag_; }
-    ResourceUsageTagEx TagEx() const { return {tag_, handle_index_}; }
     bool IsWriteHazard(const SyncAccessInfo &usage_info) const;
-    bool IsOrdered(const OrderingBarrier &ordering, QueueId queue_id) const;
-    bool IsLoadOp() const { return flags_ & SyncFlag::kLoadOp; }
-
     bool IsWriteBarrierHazard(QueueId queue_id, VkPipelineStageFlags2 src_exec_scope,
                               const SyncAccessFlags &src_access_scope) const;
 
-    void SetQueueId(QueueId id);
-    void Set(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncFlags flags);
-    void MergeBarriers(const WriteState &other);
-    void OffsetTag(ResourceUsageTag offset) { tag_ += offset; }
+    bool IsOrdered(const OrderingBarrier &ordering, QueueId queue_id) const;
+    bool IsLoadOp() const { return flags & SyncFlag::kLoadOp; }
+    bool IsPresent() const { return flags & SyncFlag::kPresent; }
 
-    bool HasPendingState() const { return pending_barriers_.any() || (0 != pending_dep_chain_); }
-    void UpdatePendingBarriers(const SyncBarrier &barrier);
-    void ApplyPendingBarriers();
-    void UpdatePendingLayoutOrdering(const SyncBarrier &barrier);
-    const OrderingBarrier &GetPendingLayoutOrdering() const { return pending_layout_ordering_; }
+    ResourceUsageTagEx TagEx() const { return {tag, handle_index}; }
+};
 
-  private:
-    const SyncAccessInfo *access_;
-    SyncAccessFlags barriers_;  // union of applicable barrier masks since last write
-    ResourceUsageTag tag_;
-    uint32_t handle_index_;
-    QueueId queue_;
-    SyncFlags flags_;
-    // intially zero, but accumulating the dstStages of barriers if they chain.
-    VkPipelineStageFlags2 dependency_chain_;
+static_assert(std::is_trivially_copyable_v<WriteState>);
 
-    // Write specific layout state
-    OrderingBarrier pending_layout_ordering_;
-    VkPipelineStageFlags2 pending_dep_chain_;
-    SyncAccessFlags pending_barriers_;
+enum class PendingBarrierType : uint8_t { ReadAccessBarrier, WriteAccessBarrier, LayoutTransition };
 
-    friend ResourceAccessState;
+struct PendingBarrierInfo {
+    PendingBarrierType type;
+    uint32_t index;  // indexes array determined by 'type'
+    ResourceAccessState *access_state;
+};
+
+struct PendingReadBarrier {
+    VkPipelineStageFlags2 barriers;
+    uint32_t last_reads_index;  // indexes ResourceAccessState::last_reads
+};
+
+struct PendingWriteBarrier {
+    SyncAccessFlags barriers;
+    VkPipelineStageFlags2 dependency_chain;
+};
+
+struct PendingLayoutTransition {
+    OrderingBarrier ordering;
+    uint32_t handle_index;
+};
+
+// PendingBarriers stores the results of independent barrier applications, so the applied barriers
+// do not interact (for example, they do not create execution dependencies between themselves).
+// Apply() stores the final result in the access state. Independent barrier application is required
+// by various sync APIs, such as vkCmdPipelineBarrier.
+//
+// A naive approach to applying a set of independent barriers is to apply them directly to the access
+// state one at a time. This creates dependencies. PendingBarriers solves this by delaying updates to
+// the access state until all barriers have been processed.
+struct PendingBarriers {
+    std::vector<PendingBarrierInfo> infos;
+    std::vector<PendingReadBarrier> read_barriers;
+    std::vector<PendingWriteBarrier> write_barriers;
+    std::vector<PendingLayoutTransition> layout_transitions;
+
+    // Store result of barrier application as PendingBarriers state
+    void AddReadBarrier(ResourceAccessState *access_state, uint32_t last_reads_index, const SyncBarrier &barrier);
+    void AddWriteBarrier(ResourceAccessState *access_state, const SyncBarrier &barrier);
+    void AddLayoutTransition(ResourceAccessState *access_state, const SyncBarrier &barrier,
+                             uint32_t layout_transition_handle_index);
+
+    // Update accesss state with collected barriers
+    void Apply(const ResourceUsageTag exec_tag);
 };
 
 class ResourceAccessState {
-  protected:
-    using OrderingBarriers = std::array<OrderingBarrier, static_cast<size_t>(SyncOrdering::kNumOrderings)>;
-    using FirstAccesses = small_vector<ResourceFirstAccess, 3>;
-
   public:
     HazardResult DetectHazard(const SyncAccessInfo &usage_info) const;
+    HazardResult DetectMarkerHazard() const;
+
     HazardResult DetectHazard(const SyncAccessInfo &usage_info, const OrderingBarrier &ordering, SyncFlags flags,
                               QueueId queue_id) const;
     HazardResult DetectHazard(const ResourceAccessState &recorded_use, QueueId queue_id, const ResourceUsageRange &tag_range) const;
@@ -356,17 +390,29 @@ class ResourceAccessState {
                                      QueueId event_queue, ResourceUsageTag event_tag) const;
 
     void Update(const SyncAccessInfo &usage_info, SyncOrdering ordering_rule, ResourceUsageTagEx tag_ex, SyncFlags flags = 0);
-    void SetWrite(const SyncAccessInfo &usage_info, ResourceUsageTagEx tag_ex, SyncFlags flags = 0);
+    void SetWrite(SyncAccessIndex access_index, ResourceUsageTagEx tag_ex, SyncFlags flags = 0);
     void ClearWrite();
     void ClearRead();
     void ClearFirstUse();
     void Resolve(const ResourceAccessState &other);
-    void ApplyBarriers(const std::vector<SyncBarrier> &barriers, bool layout_transition);
-    void ApplyBarriersImmediate(const SyncBarrier &barriers);
-    template <typename ScopeOps>
-    void ApplyBarrier(ScopeOps &&scope, const SyncBarrier &barrier, bool layout_transition,
-                      uint32_t layout_transition_handle_index = vvl::kNoIndex32);
-    void ApplyPendingBarriers(ResourceUsageTag tag);
+
+    // Apply a single barrier to the access state
+    void ApplyBarrier(const BarrierScope &barrier_scope, const SyncBarrier &barrier, bool layout_transition = false,
+                      uint32_t layout_transition_handle_index = vvl::kNoIndex32,
+                      ResourceUsageTag layout_transition_tag = kInvalidTag);
+
+    // Store the result of barrier application in PendingBarriers.
+    // Does not update the access state (as ApplyBarrier does).
+    // Used for applying multiple barriers independently.
+    void CollectPendingBarriers(const BarrierScope &barrier_scope, const SyncBarrier &barrier, bool layout_transition,
+                                uint32_t layout_transition_handle_index, PendingBarriers &pending_barriers);
+
+    // Apply pending barriers to the access state.
+    // Called after all barrier application results are collected in PendingBarriers.
+    void ApplyPendingReadBarrier(const PendingReadBarrier &read_barrier, ResourceUsageTag tag);
+    void ApplyPendingWriteBarrier(const PendingWriteBarrier &write_barrier);
+    void ApplyPendingLayoutTransition(const PendingLayoutTransition &layout_transition, ResourceUsageTag layout_transition_tag);
+
     void ApplySemaphore(const SemaphoreScope &signal, const SemaphoreScope wait);
 
     struct WaitQueueTagPredicate {
@@ -392,19 +438,23 @@ class ResourceAccessState {
     };
     friend WaitAcquirePredicate;
 
+    // Clear read/write accesses that satisfy the predicate
+    // (predicate says which accesses should be considered synchronized).
+    // Return true if all accesses were cleared and access state is empty
     template <typename Predicate>
-    bool ApplyPredicatedWait(Predicate &predicate);
+    bool ClearPredicatedAccesses(Predicate &predicate);
 
     bool FirstAccessInTagRange(const ResourceUsageRange &tag_range) const;
 
     void OffsetTag(ResourceUsageTag offset);
     ResourceAccessState();
 
-    bool HasPendingState() const { return (0 != pending_layout_transition) || (last_write && last_write->HasPendingState()); }
     bool HasWriteOp() const { return last_write.has_value(); }
-    SyncAccessIndex LastWriteOp() const { return last_write.has_value() ? last_write->Index() : SYNC_ACCESS_INDEX_NONE; }
-    bool IsLastWriteOp(SyncAccessIndex access_index) const { return LastWriteOp() == access_index; }
-    ResourceUsageTag LastWriteTag() const { return last_write.has_value() ? last_write->Tag() : ResourceUsageTag(0); }
+    bool IsLastWriteOp(SyncAccessIndex access_index) const {
+        return last_write.has_value() && last_write->access_index == access_index;
+    }
+    ResourceUsageTag LastWriteTag() const { return last_write.has_value() ? last_write->tag : ResourceUsageTag(0); }
+    const WriteState &LastWrite() const;
     bool operator==(const ResourceAccessState &rhs) const {
         const bool write_same = (read_execution_barriers == rhs.read_execution_barriers) &&
                                 (input_attachment_read == rhs.input_attachment_read) && (last_write == rhs.last_write);
@@ -419,67 +469,20 @@ class ResourceAccessState {
     }
     bool operator!=(const ResourceAccessState &rhs) const { return !(*this == rhs); }
     VkPipelineStageFlags2 GetReadBarriers(SyncAccessIndex access_index) const;
-    SyncAccessFlags GetWriteBarriers() const { return last_write.has_value() ? last_write->Barriers() : SyncAccessFlags(); }
+    SyncAccessFlags GetWriteBarriers() const { return last_write.has_value() ? last_write->barriers : SyncAccessFlags(); }
     void SetQueueId(QueueId id);
 
     bool IsWriteBarrierHazard(QueueId queue_id, VkPipelineStageFlags2 src_exec_scope,
                               const SyncAccessFlags &src_access_scope) const;
-    bool WriteInSourceScopeOrChain(VkPipelineStageFlags2 src_exec_scope, SyncAccessFlags src_access_scope) const;
-    bool WriteInQueueSourceScopeOrChain(QueueId queue, VkPipelineStageFlags2 src_exec_scope,
-                                        const SyncAccessFlags &src_access_scope) const;
-    bool WriteInEventScope(VkPipelineStageFlags2 src_exec_scope, const SyncAccessFlags &src_access_scope, QueueId scope_queue,
-                           ResourceUsageTag scope_tag) const;
-
-    struct UntaggedScopeOps {
-        bool WriteInScope(const SyncBarrier &barrier, const ResourceAccessState &access) const {
-            return access.WriteInSourceScopeOrChain(barrier.src_exec_scope.exec_scope, barrier.src_access_scope);
-        }
-        bool ReadInScope(const SyncBarrier &barrier, const ReadState &read_state) const {
-            return read_state.ReadInScopeOrChain(barrier.src_exec_scope.exec_scope);
-        }
-    };
-
-    struct QueueScopeOps {
-        bool WriteInScope(const SyncBarrier &barrier, const ResourceAccessState &access) const {
-            return access.WriteInQueueSourceScopeOrChain(queue, barrier.src_exec_scope.exec_scope, barrier.src_access_scope);
-        }
-        bool ReadInScope(const SyncBarrier &barrier, const ReadState &read_state) const {
-            return read_state.ReadInQueueScopeOrChain(queue, barrier.src_exec_scope.exec_scope);
-        }
-        QueueScopeOps(QueueId scope_queue) : queue(scope_queue) {}
-        QueueId queue;
-    };
-
-    struct EventScopeOps {
-        bool WriteInScope(const SyncBarrier &barrier, const ResourceAccessState &access) const {
-            return access.WriteInEventScope(barrier.src_exec_scope.exec_scope, barrier.src_access_scope, scope_queue, scope_tag);
-        }
-        bool ReadInScope(const SyncBarrier &barrier, const ReadState &read_state) const {
-            return read_state.ReadInEventScope(barrier.src_exec_scope.exec_scope, scope_queue, scope_tag);
-        }
-        EventScopeOps(QueueId qid, ResourceUsageTag event_tag) : scope_queue(qid), scope_tag(event_tag) {}
-        QueueId scope_queue;
-        ResourceUsageTag scope_tag;
-    };
-
     void Normalize();
     void GatherReferencedTags(ResourceUsageTagSet &used) const;
 
-    static const OrderingBarrier &GetOrderingRules(SyncOrdering ordering_enum) {
-        return kOrderingRules[static_cast<size_t>(ordering_enum)];
-    }
+    void UpdateStats(syncval_stats::AccessContextStats &stats) const;
 
   private:
     static constexpr VkPipelineStageFlags2 kInvalidAttachmentStage = ~VkPipelineStageFlags2(0);
     bool IsRAWHazard(const SyncAccessInfo &usage_info) const;
 
-    bool WriteInScope(const SyncAccessFlags &src_access_scope) const;
-    // Apply ordering scope to write hazard detection
-
-    bool ReadInSourceScopeOrChain(VkPipelineStageFlags2 src_exec_scope) const {
-        return (0 != (src_exec_scope & (last_read_stages | read_execution_barriers)));
-    }
-
     static bool IsReadHazard(VkPipelineStageFlags2 stage_mask, const VkPipelineStageFlags2 barriers) {
         return stage_mask != (stage_mask & barriers);
     }
@@ -489,9 +492,8 @@ class ResourceAccessState {
     }
     VkPipelineStageFlags2 GetOrderedStages(QueueId queue_id, const OrderingBarrier &ordering, SyncFlags flags) const;
 
-    void UpdateFirst(ResourceUsageTagEx tag_ex, const SyncAccessInfo &usage_info, SyncOrdering ordering_rule);
+    void UpdateFirst(ResourceUsageTagEx tag_ex, const SyncAccessInfo &usage_info, SyncOrdering ordering_rule, SyncFlags flags = 0);
     void TouchupFirstForLayoutTransition(ResourceUsageTag tag, const OrderingBarrier &layout_ordering);
-    void MergePending(const ResourceAccessState &other);
     void MergeReads(const ResourceAccessState &other);
 
     // TODO: Add a NONE (zero) enum to SyncStageAccessFlags for input_attachment_read and last_write
@@ -506,90 +508,40 @@ class ResourceAccessState {
 
     VkPipelineStageFlags2 last_read_stages;
     VkPipelineStageFlags2 read_execution_barriers;
-    using ReadStates = small_vector<ReadState, 3, uint32_t>;
+
+    // NOTE: default capacity is set to 2. A single read is the most common case.
+    // Two reads occur sometimes and more than two is rare in practice.
+    // Syncval performance is sensitive to memory usage (there are many access objects).
+    // The early tests show that capacity of 1 can further improve performance and in some
+    // apps reduced memory bandwidth outweight the cost of additional allocations.
+    // More testing is needed.
+    using ReadStates = small_vector<ReadState, 2, uint32_t>;
+
     ReadStates last_reads;
 
     // TODO Input Attachment cleanup for multiple reads in a given stage
     // Tracks whether the fragment shader read is input attachment read
     bool input_attachment_read;
 
-    // Not part of the write state, logically.  Can exist when !last_write
-    // Pending execution state to support independent parallel barriers
-    bool pending_layout_transition;
-    uint32_t pending_layout_transition_handle_index = vvl::kNoIndex32;
+    // Reserve capacity for 2 first accesses, more than that is not very common
+    using FirstAccesses = small_vector<ResourceFirstAccess, 2>;
 
     FirstAccesses first_accesses_;
     VkPipelineStageFlags2 first_read_stages_;
     OrderingBarrier first_write_layout_ordering_;
     bool first_access_closed_;
-
-    static OrderingBarriers kOrderingRules;
 };
 using ResourceAccessStateFunction = std::function<void(ResourceAccessState *)>;
 using ResourceAccessRangeMap = sparse_container::range_map<ResourceAddress, ResourceAccessState>;
 using ResourceRangeMergeIterator = sparse_container::parallel_iterator<ResourceAccessRangeMap, const ResourceAccessRangeMap>;
 
-// Apply the memory barrier without updating the existing barriers.  The execution barrier
-// changes the "chaining" state, but to keep barriers independent, we defer this until all barriers
-// of the batch have been processed. Also, depending on whether layout transition happens, we'll either
-// replace the current write barriers or add to them, so accumulate to pending as well.
-template <typename ScopeOps>
-void ResourceAccessState::ApplyBarrier(ScopeOps &&scope, const SyncBarrier &barrier, bool layout_transition,
-                                       uint32_t layout_transition_handle_index) {
-    // For independent barriers we need to track what the new barriers and dependency chain *will* be when we're done
-    // applying the memory barriers
-    // NOTE: We update the write barrier if the write is in the first access scope or if there is a layout
-    //       transistion, under the theory of "most recent access".  If the resource acces  *isn't* safe
-    //       vs. this layout transition DetectBarrierHazard should report it.  We treat the layout
-    //       transistion *as* a write and in scope with the barrier (it's before visibility).
-    if (layout_transition) {
-        if (!last_write.has_value()) {
-            last_write.emplace(GetAccessInfo(SYNC_ACCESS_INDEX_NONE), ResourceUsageTagEx{0U});
-        }
-        last_write->UpdatePendingBarriers(barrier);
-        last_write->UpdatePendingLayoutOrdering(barrier);
-        pending_layout_transition = true;
-        pending_layout_transition_handle_index = layout_transition_handle_index;
-    } else {
-        if (scope.WriteInScope(barrier, *this)) {
-            last_write->UpdatePendingBarriers(barrier);
-        }
-
-        if (!pending_layout_transition) {
-            // Once we're dealing with a layout transition (which is modelled as a *write*) then the last reads/chains
-            // don't need to be tracked as we're just going to clear them.
-            VkPipelineStageFlags2 stages_in_scope = VK_PIPELINE_STAGE_2_NONE;
-
-            for (auto &read_access : last_reads) {
-                // The | implements the "dependency chain" logic for this access, as the barriers field stores the second sync
-                // scope
-                if (scope.ReadInScope(barrier, read_access)) {
-                    // We'll apply the barrier in the next loop, because it's DRY'r to do it one place.
-                    stages_in_scope |= read_access.stage;
-                }
-            }
-
-            for (auto &read_access : last_reads) {
-                if (0 != ((read_access.stage | read_access.sync_stages) & stages_in_scope)) {
-                    // If this stage, or any stage known to be synchronized after it are in scope, apply the barrier to this
-                    // read NOTE: Forwarding barriers to known prior stages changes the sync_stages from shallow to deep,
-                    // because the
-                    //       barriers used to determine sync_stages have been propagated to all known earlier stages
-                    read_access.ApplyReadBarrier(barrier.dst_exec_scope.exec_scope);
-                }
-            }
-        }
-    }
-}
-
-// Return if the resulting state is "empty"
 template <typename Predicate>
-bool ResourceAccessState::ApplyPredicatedWait(Predicate &predicate) {
+bool ResourceAccessState::ClearPredicatedAccesses(Predicate &predicate) {
     VkPipelineStageFlags2 sync_reads = VK_PIPELINE_STAGE_2_NONE;
 
     // Use the predicate to build a mask of the read stages we are synchronizing
     // Use the sync_stages to also detect reads known to be before any synchronized reads (first pass)
-    for (auto &read_access : last_reads) {
+    for (const auto &read_access : last_reads) {
         if (predicate(read_access)) {
             // If we know this stage is before any stage we syncing, or if the predicate tells us that we are waited for..
             sync_reads |= read_access.stage;
@@ -599,7 +551,7 @@ bool ResourceAccessState::ApplyPredicatedWait(Predicate &predicate) {
     // Now that we know the reads directly in scopejust need to go over the list again to pick up the "known earlier" stages.
     // NOTE: sync_stages is "deep" catching all stages synchronized after it because we forward barriers
     uint32_t unsync_count = 0;
-    for (auto &read_access : last_reads) {
+    for (const auto &read_access : last_reads) {
         if (0 != ((read_access.stage | read_access.sync_stages) & sync_reads)) {
             // This is redundant in the "stage" case, but avoids a second branch to get an accurate count
             sync_reads |= read_access.stage;
@@ -642,3 +594,9 @@ bool ResourceAccessState::ApplyPredicatedWait(Predicate &predicate) {
     }
     return all_clear;
 }
+
+// A helper function to apply multiple barriers.
+// NOTE: That's for use cases when BarrierScope does not use queue id or tag (record time, not-event barriers).
+// This can be extended if necessary to provide BarrierScope for each barrier.
+void ApplyBarriers(ResourceAccessState &access_state, const std::vector<SyncBarrier> &barriers, bool layout_transition = false,
+                   ResourceUsageTag layout_transition_tag = kInvalidTag);
