@@ -19,68 +19,61 @@
 
 #include "generated/error_location_helper.h"
 #include "generated/sync_validation_types.h"
-#include "generated/vk_object_types.h"
 
-namespace vvl {
-class CommandBuffer;
-class StateObject;
-class Image;
-class Queue;
-}  // namespace vvl
+class CommandExecutionContext;
+class HazardResult;
 class Logger;
-class DebugReport;
 class SyncValidator;
-struct DeviceFeatures;
-struct DeviceExtensions;
+
+// Collection of named values that describe key information associated with an error message.
+// This can be useful to filter out messages or for quick inspection as a more structured (but lose)
+// representation of the main error message text. The main error message might change relatively
+// often due to wording improvements and minor fixes (e.g. punctuation fix), but the associated
+// properties will change less frequently and are more suited for automatic processing.
+struct ReportProperties {
+    struct NameValue {
+        std::string name;
+        std::string value;
+    };
+    std::vector<NameValue> name_values;
+
+    void Add(std::string_view property_name, std::string_view value);
+    void Add(std::string_view property_name, uint64_t value);
+    std::string FormatExtraPropertiesSection() const;
+};
+
+// Customization options to modify the standard form of the synchronization error message.
+struct AdditionalMessageInfo {
+    // These are message-specific properties. They are combined with common message properties.
+    ReportProperties properties;
+
+    // When we need something more complex than vvl::Func
+    std::string access_initiator;
+
+    // Replaces standard "writes to"/"reads" access wording.
+    // For example, "clears" for a clear operation might be more specific than a write
+    std::string access_action;
+
+    std::string hazard_overview;
+    std::string brief_description_end_text;
+    std::string pre_synchronization_text;
+    std::string message_end_text;
+};
+
+ReportProperties GetErrorMessageProperties(const HazardResult &hazard, const CommandExecutionContext &context, vvl::Func command,
+                                           const char *message_type, const AdditionalMessageInfo &additional_info);
+
+std::string FormatErrorMessage(const HazardResult &hazard, const CommandExecutionContext &context, vvl::Func command,
+                               const std::string &resouce_description, const AdditionalMessageInfo &additional_info);
+
+std::string FormatSyncAccesses(const SyncAccessFlags &sync_accesses, const SyncValidator &device, VkQueueFlags allowed_queue_flags,
+                               bool format_as_extra_property);
 
 void FormatVideoPictureResouce(const Logger &logger, const VkVideoPictureResourceInfoKHR &video_picture, std::stringstream &ss);
 void FormatVideoQuantizationMap(const Logger &logger, const VkVideoEncodeQuantizationMapInfoKHR &quantization_map,
                                 std::stringstream &ss);
 
-struct SyncNodeFormatter {
-    const DebugReport *debug_report;
-    const vvl::StateObject *node;
-    const char *label;
-
-    SyncNodeFormatter(const SyncValidator &sync_state, const vvl::CommandBuffer *cb_state);
-    SyncNodeFormatter(const SyncValidator &sync_state, const vvl::Image *image);
-    SyncNodeFormatter(const SyncValidator &sync_state, const vvl::Queue *q_state);
-    SyncNodeFormatter(const SyncValidator &sync_state, const vvl::StateObject *state_object, const char *label_ = nullptr);
-};
-std::string FormatStateObject(const SyncNodeFormatter &formatter);
-
-struct ReportKeyValues {
-    struct KeyValue {
-        std::string key;
-        std::string value;
-    };
-    std::vector<KeyValue> key_values;
-
-    void Add(std::string_view key, std::string_view value);
-    void Add(std::string_view key, uint64_t value);
-
-    std::string GetExtraPropertiesSection(bool pretty_print) const;
-    const std::string *FindProperty(const std::string &key) const;
-};
-
-struct ReportUsageInfo {
-    vvl::Func command = vvl::Func::Empty;
-    VulkanTypedHandle resource_handle;
-    std::string debug_region_name;
-    const vvl::CommandBuffer *cb = nullptr;
-    const vvl::Queue *queue = nullptr;
-    uint64_t submit_index = 0;
-    uint32_t batch_index = 0;
-};
-
-std::vector<std::pair<VkPipelineStageFlags2, VkAccessFlags2>> ConvertSyncAccessesToCompactVkForm(
-    const SyncAccessFlags &sync_accesses, VkQueueFlags allowed_queue_flags, const DeviceFeatures &features,
-    const DeviceExtensions &device_extensions);
-
-std::string FormatSyncAccesses(const SyncAccessFlags &sync_accesses, VkQueueFlags allowed_queue_flags,
-                               const DeviceFeatures &features, const DeviceExtensions &device_extensions,
-                               bool format_as_extra_property);
-
+// Common properties
 inline constexpr const char *kPropertyMessageType = "message_type";
 inline constexpr const char *kPropertyHazardType = "hazard_type";
 inline constexpr const char *kPropertyCommand = "command";
@@ -91,6 +84,8 @@ inline constexpr const char *kPropertyAccess = "access";
 inline constexpr const char *kPropertyPriorAccess = "prior_access";
 inline constexpr const char *kPropertyReadBarriers = "read_barriers";
 inline constexpr const char *kPropertyWriteBarriers = "write_barriers";
+
+// Message-specific properties
 inline constexpr const char *kPropertyRegionIndex = "region_index";
 inline constexpr const char *kPropertyLoadOp = "load_op";
 inline constexpr const char *kPropertyStoreOp = "store_op";
@@ -107,8 +102,7 @@ inline constexpr const char *kPropertyBatchIndex = "batch_index";
 inline constexpr const char *kPropertyCommandBufferIndex = "command_buffer_index";
 inline constexpr const char *kPropertySwapchainIndex = "swapchain_index";
 
-// debug properties
+// Debug properties
 inline constexpr const char *kPropertySeqNo = "seq_no";
-inline constexpr const char *kPropertySubCmd = "subcmd";
 inline constexpr const char *kPropertyResetNo = "reset_no";
 inline constexpr const char *kPropertyBatchTag = "batch_tag";
