@@ -23,6 +23,11 @@ class TestIcdGenerator(BaseGenerator):
         BaseGenerator.__init__(self)
 
         self.manual_functions = [
+            'vkGetClusterAccelerationStructureBuildSizesNV',
+            'vkGetTensorMemoryRequirementsARM',
+            'vkGetPhysicalDeviceExternalTensorPropertiesARM',
+            'vkGetDataGraphPipelineSessionBindPointRequirementsARM',
+            'vkGetDataGraphPipelineSessionMemoryRequirementsARM',
             'vkCreateInstance',
             'vkDestroyInstance',
             'vkAllocateCommandBuffers',
@@ -120,6 +125,7 @@ class TestIcdGenerator(BaseGenerator):
             'vkRegisterDisplayEventEXT',
             'vkCreatePipelineBinariesKHR',
             'vkGetPipelineBinaryDataKHR',
+            'vkGetPartitionedAccelerationStructuresBuildSizesNV',
         ]
 
     def generate(self):
