@@ -3,6 +3,7 @@
  * Copyright (c) 2015-2025 Valve Corporation
  * Copyright (c) 2015-2025 LunarG, Inc.
  * Copyright (c) 2015-2025 Google, Inc.
+ * Copyright (C) 2025 Arm Limited.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -241,8 +242,8 @@ class GpuAVDescriptorIndexingTest : public GpuAVTest {
 
 class GpuAVDescriptorClassGeneralBuffer : public GpuAVTest {
   public:
-    void ComputeStorageBufferTest(const char *shader, bool is_glsl, VkDeviceSize buffer_size, const char *expected_error = nullptr,
-                                  uint32_t error_count = 1);
+    void ComputeStorageBufferTest(const char *shader, int source_type, VkDeviceSize buffer_size,
+                                  const char *expected_error = nullptr, uint32_t error_count = 1);
 };
 
 class GpuAVRayQueryTest : public GpuAVTest {
@@ -261,6 +262,11 @@ class AndroidExternalResolveTest : public VkLayerTest {
     bool nullColorAttachmentWithExternalFormatResolve;
 };
 
+class DeprecationTest : public VkLayerTest {
+  public:
+    void CreateRenderPass();
+};
+
 class DescriptorBufferTest : public VkLayerTest {
   public:
     void InitBasicDescriptorBuffer(void *instance_pnext = nullptr);
@@ -332,6 +338,11 @@ class ImageDrmTest : public VkLayerTest {
     std::vector<uint64_t> GetFormatModifier(VkFormat format, VkFormatFeatureFlags2 features, uint32_t plane_count = 1);
 };
 
+class MeshTest : public virtual VkLayerTest {
+  public:
+    void InitBasicMeshAndTask();
+};
+
 class QueryTest : public VkLayerTest {
   public:
     bool HasZeroTimestampValidBits();
@@ -367,6 +378,41 @@ class SyncObjectTest : public VkLayerTest {
 #endif
 };
 
+class TensorTest : public VkLayerTest {
+  public:
+    void InitBasicTensor();
+    static VkTensorDescriptionARM DefaultDesc();
+    static VkTensorCreateInfoARM DefaultCreateInfo(VkTensorDescriptionARM *desc = nullptr);
+
+    const char *tensor_shader_source = R"glsl(
+      #version 450
+      #extension GL_ARM_tensors : require
+      #extension GL_EXT_shader_explicit_arithmetic_types : require
+      layout(local_size_x = 1, local_size_y = 1, local_size_z = 1) in;
+      layout(set=0, binding=0) uniform tensorARM<int32_t, 1> tens;
+      layout(set=0, binding=1, std430) buffer asd {
+        int32_t out_data[];
+      };
+      void main()
+      {
+        const uint size_x = tensorSizeARM(tens, 0);
+        const uint x = gl_GlobalInvocationID.x % size_x;
+        const uint out_index = gl_GlobalInvocationID.x;
+
+        tensorReadARM(tens, uint[](x), out_data[out_index]);
+      }
+    )glsl";
+};
+
+class DataGraphTest : public VkLayerTest {
+  public:
+    void InitBasicDataGraph(bool init_types = false);
+    static void CheckSessionMemory(const vkt::DataGraphPipelineSession& session);
+    static std::vector<VkBindDataGraphPipelineSessionMemoryInfoARM> InitSessionBindInfo(const vkt::DataGraphPipelineSession& session, const std::vector<vkt::DeviceMemory>& device_mem);
+
+    static const std::string IncorrectSpirvMessage;
+};
+
 class WsiTest : public VkLayerTest {
   protected:
     // Find physical device group that contains physical device selected by the test framework
