@@ -1,7 +1,7 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -17,9 +17,24 @@
  */
 
 #include "shader_helper.h"
+
+#include "test_common.h"
 #include "glslang/SPIRV/GlslangToSpv.h"
 #include <glslang/Public/ShaderLang.h>
 
+#ifdef VVL_USE_SLANG
+#pragma push_macro("None")
+#pragma push_macro("Bool")
+#undef None
+#undef Bool
+
+#include "slang.h"
+#include "slang-com-ptr.h"
+
+#pragma pop_macro("None")
+#pragma pop_macro("Bool")
+#endif
+
 static void ProcessConfigFile(const VkPhysicalDeviceLimits &device_limits, TBuiltInResource &out_resources) {
     // These are the default resources for TBuiltInResources.
     out_resources.maxLights = 32;
@@ -304,12 +319,147 @@ bool ASMtoSPV(const spv_target_env target_env, const uint32_t options, const cha
     return true;
 }
 
+void CheckSlangSupport() {
+#ifndef VVL_USE_SLANG
+    GTEST_SKIP() << "Slang not supported on this platform";
+#endif
+}
+
+bool SlangToSPV(const char *slang_shader, const char *entry_point_name, std::vector<uint8_t> &out_bytes) {
+#ifndef VVL_USE_SLANG
+    (void)slang_shader;
+    (void)entry_point_name;
+    (void)out_bytes;
+    return false;
+#else
+    // Function adapted from
+    // https://github.com/shader-slang/slang/blob/master/examples/hello-world/main.cpp#L114
+    // Used https://github.com/shader-slang/slang/issues/6678 to figure out how to use
+    // `slang::IModule::loadModuleFromSourceString`
+
+    using namespace Slang;
+
+    // First we need to create slang global session with work with the Slang API.
+    ComPtr<slang::IGlobalSession> slang_session;
+    {
+        SlangGlobalSessionDesc slang_session_desc = {};
+        // slang_session_desc.enableGLSL = true; // Could be needed in the future
+        slang::createGlobalSession(&slang_session_desc, slang_session.writeRef());
+    }
+
+    // Next we create a compilation session to generate SPIRV code from Slang source.
+    slang::TargetDesc targetDesc = {};
+    targetDesc.format = SLANG_SPIRV;
+    targetDesc.profile = slang_session->findProfile("glsl_460");  // todo what spirv profile ?
+    targetDesc.flags = 0;
+    slang::SessionDesc sessionDesc = {};
+    sessionDesc.targets = &targetDesc;
+    sessionDesc.targetCount = 1;
+
+    std::vector<slang::CompilerOptionEntry> options;
+    options.push_back(
+        {slang::CompilerOptionName::EmitSpirvDirectly, {slang::CompilerOptionValueKind::Int, 1, 0, nullptr, nullptr}});
+    // https://github.com/shader-slang/slang/issues/6248
+    options.push_back(
+        {slang::CompilerOptionName::VulkanUseEntryPointName, {slang::CompilerOptionValueKind::Int, 1, 0, nullptr, nullptr}});
+    sessionDesc.compilerOptionEntries = options.data();
+    sessionDesc.compilerOptionEntryCount = options.size();
+
+    ComPtr<slang::ISession> session;
+    SlangResult result = slang_session->createSession(sessionDesc, session.writeRef());
+    if (result != 0) {
+        ADD_FAILURE() << "Slang failure: slang_session->createSession()";
+    }
+
+    // Handle errors
+    Slang::ComPtr<ISlangBlob> diagnostics;
+
+    // Compile the source code
+    // Once the session has been obtained, we can start loading code into it.
+    // Here we use `loadModuleFromSourceString` and not `loadModule`.
+    // moduleName and path are just placeholders
+    Slang::ComPtr<slang::IModule> slang_module;
+    slang_module = session->loadModuleFromSourceString("my_shader", "my_shader.slang", slang_shader, diagnostics.writeRef());
+    if (slang_module == NULL) {
+        ADD_FAILURE() << "Slang failure: loadModuleFromSourceString()\n" << ((const char *)diagnostics->getBufferPointer());
+        return false;
+    }
+
+    // Loading the module will compile and check all the shader code in it,
+    // including the shader entry points we want to use. Now that the module is loaded
+    // we can look up those entry points by name.
+    //
+    // Note: If you are using this `loadModule` approach to load your shader code it is
+    // important to tag your entry point functions with the `[shader("...")]` attribute
+    // (e.g., `[shader("compute")] void computeMain(...)`). Without that information there
+    // is no unambiguous way for the compiler to know which functions represent entry
+    // points when it parses your code via `loadModule()`.
+    //
+    ComPtr<slang::IEntryPoint> entry_point;
+    result = slang_module->findEntryPointByName(entry_point_name, entry_point.writeRef());
+    if (result != 0) {
+        ADD_FAILURE() << "Slang failure: loadModuleFromSourceString()\nCould not find entry point \"" << entry_point_name << "\"";
+        return false;
+    }
+
+    // At this point we have a few different Slang API objects that represent
+    // pieces of our code: `module`, `vertexEntryPoint`, and `fragmentEntryPoint`.
+    //
+    // A single Slang module could contain many different entry points (e.g.,
+    // four vertex entry points, three fragment entry points, and two compute
+    // shaders), and before we try to generate output code for our target API
+    // we need to identify which entry points we plan to use together.
+    //
+    // Modules and entry points are both examples of *component types* in the
+    // Slang API. The API also provides a way to build a *composite* out of
+    // other pieces, and that is what we are going to do with our module
+    // and entry points.
+    //
+    std::vector<slang::IComponentType *> componentTypes;
+    componentTypes.emplace_back(slang_module);
+    componentTypes.emplace_back(entry_point);
+
+    // Actually creating the composite component type is a single operation
+    // on the Slang session, but the operation could potentially fail if
+    // something about the composite was invalid (e.g., you are trying to
+    // combine multiple copies of the same module), so we need to deal
+    // with the possibility of diagnostic output.
+    //
+    ComPtr<slang::IComponentType> composedProgram;
+    {
+        result = session->createCompositeComponentType(componentTypes.data(), (SlangInt)componentTypes.size(),
+                                                       composedProgram.writeRef(), diagnostics.writeRef());
+        if (result != 0) {
+            ADD_FAILURE() << "Slang failure: createCompositeComponentType()\n" << ((const char *)diagnostics->getBufferPointer());
+            return false;
+        }
+    }
+
+    // Now we can call `composedProgram->getEntryPointCode()` to retrieve the
+    // compiled SPIRV code that we will use to create a vulkan compute pipeline.
+    // This will trigger the final Slang compilation and spirv code generation.
+    ComPtr<slang::IBlob> spirvCode;
+    {
+        result = composedProgram->getEntryPointCode(0, 0, spirvCode.writeRef(), diagnostics.writeRef());
+        if (result != 0) {
+            ADD_FAILURE() << "Slang failure: createCompositeComponentType()\n" << ((const char *)diagnostics->getBufferPointer());
+            return false;
+        }
+    }
+
+    out_bytes.resize(spirvCode->getBufferSize());
+    std::memcpy(out_bytes.data(), spirvCode->getBufferPointer(), spirvCode->getBufferSize());
+
+    return true;
+#endif
+}
+
 VkPipelineShaderStageCreateInfo const &VkShaderObj::GetStageCreateInfo() const { return m_stage_info; }
 
-VkShaderObj::VkShaderObj(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage, const spv_target_env env,
-                         SpvSourceType source_type, const VkSpecializationInfo *spec_info, char const *entry_point,
+VkShaderObj::VkShaderObj(vkt::Device &device, const char *source, VkShaderStageFlagBits stage, const spv_target_env env,
+                         SpvSourceType source_type, const VkSpecializationInfo *spec_info, const char *entry_point,
                          const void *pNext)
-    : m_device(*(framework->DeviceObj())), m_source(source), m_spv_env(env) {
+    : m_device(&device), m_source(source), m_spv_env(env) {
     m_stage_info = vku::InitStructHelper();
     m_stage_info.flags = 0;
     m_stage_info.stage = stage;
@@ -320,19 +470,26 @@ VkShaderObj::VkShaderObj(VkRenderFramework *framework, const char *source, VkSha
         InitFromGLSL(pNext);
     } else if (source_type == SPV_SOURCE_ASM) {
         InitFromASM();
+    } else if (source_type == SPV_SOURCE_SLANG) {
+        InitFromSlang();
     }
 }
 
+VkShaderObj::VkShaderObj(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage, const spv_target_env env,
+                         SpvSourceType source_type, const VkSpecializationInfo *spec_info, const char *entry_point,
+                         const void *pNext)
+    : VkShaderObj(*framework->DeviceObj(), source, stage, env, source_type, spec_info, entry_point, pNext) {}
+
 bool VkShaderObj::InitFromGLSL(const void *pNext) {
     std::vector<uint32_t> spv;
-    GLSLtoSPV(m_device.Physical().limits_, m_stage_info.stage, m_source, spv, m_spv_env);
+    GLSLtoSPV(m_device->Physical().limits_, m_stage_info.stage, m_source, spv, m_spv_env);
 
     VkShaderModuleCreateInfo moduleCreateInfo = vku::InitStructHelper();
     moduleCreateInfo.pNext = pNext;
     moduleCreateInfo.codeSize = spv.size() * sizeof(uint32_t);
     moduleCreateInfo.pCode = spv.data();
 
-    init(m_device, moduleCreateInfo);
+    Init(*m_device, moduleCreateInfo);
     m_stage_info.module = handle();
     return VK_NULL_HANDLE != handle();
 }
@@ -344,14 +501,14 @@ VkResult VkShaderObj::InitFromGLSLTry(const vkt::Device *custom_device) {
     std::vector<uint32_t> spv;
     // 99% of tests just use the framework's VkDevice, but this allows for tests to use custom device object
     // Can't set at contructor time since all reference members need to be initialized then.
-    VkPhysicalDeviceLimits limits = (custom_device) ? custom_device->Physical().limits_ : m_device.Physical().limits_;
+    VkPhysicalDeviceLimits limits = (custom_device) ? custom_device->Physical().limits_ : m_device->Physical().limits_;
     GLSLtoSPV(limits, m_stage_info.stage, m_source, spv, m_spv_env);
 
     VkShaderModuleCreateInfo moduleCreateInfo = vku::InitStructHelper();
     moduleCreateInfo.codeSize = spv.size() * sizeof(uint32_t);
     moduleCreateInfo.pCode = spv.data();
 
-    const auto result = InitTry(((custom_device) ? *custom_device : m_device), moduleCreateInfo);
+    const auto result = InitTry(((custom_device) ? *custom_device : *m_device), moduleCreateInfo);
     m_stage_info.module = handle();
     return result;
 }
@@ -364,7 +521,7 @@ bool VkShaderObj::InitFromASM() {
     moduleCreateInfo.codeSize = spv.size() * sizeof(uint32_t);
     moduleCreateInfo.pCode = spv.data();
 
-    init(m_device, moduleCreateInfo);
+    Init(*m_device, moduleCreateInfo);
     m_stage_info.module = handle();
     return VK_NULL_HANDLE != handle();
 }
@@ -377,28 +534,46 @@ VkResult VkShaderObj::InitFromASMTry() {
     moduleCreateInfo.codeSize = spv.size() * sizeof(uint32_t);
     moduleCreateInfo.pCode = spv.data();
 
-    const auto result = InitTry(m_device, moduleCreateInfo);
+    const auto result = InitTry(*m_device, moduleCreateInfo);
     m_stage_info.module = handle();
     return result;
 }
 
+bool VkShaderObj::InitFromSlang() {
+#ifndef VVL_USE_SLANG
+    return false;
+#else
+    std::vector<uint8_t> bytes;
+    if (!SlangToSPV(m_source, m_stage_info.pName, bytes)) {
+        return false;
+    }
+    VkShaderModuleCreateInfo module_ci = vku::InitStructHelper();
+    module_ci.codeSize = bytes.size();
+    module_ci.pCode = (uint32_t *)bytes.data();
+
+    const auto result = InitTry(*m_device, module_ci);
+    m_stage_info.module = handle();
+    return result == VK_SUCCESS;
+#endif
+}
+
 // static
-std::unique_ptr<VkShaderObj> VkShaderObj::CreateFromGLSL(VkRenderFramework *framework, const char *source,
-                                                         VkShaderStageFlagBits stage, const spv_target_env spv_env,
-                                                         const VkSpecializationInfo *spec_info, const char *entry_point) {
-    auto shader = std::make_unique<VkShaderObj>(framework, source, stage, spv_env, SPV_SOURCE_GLSL_TRY, spec_info, entry_point);
-    if (VK_SUCCESS == shader->InitFromGLSLTry()) {
+VkShaderObj VkShaderObj::CreateFromGLSL(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
+                                        const spv_target_env spv_env, const VkSpecializationInfo *spec_info,
+                                        const char *entry_point) {
+    auto shader = VkShaderObj(framework, source, stage, spv_env, SPV_SOURCE_GLSL_TRY, spec_info, entry_point);
+    if (VK_SUCCESS == shader.InitFromGLSLTry()) {
         return shader;
     }
     return {};
 }
 
 // static
-std::unique_ptr<VkShaderObj> VkShaderObj::CreateFromASM(VkRenderFramework *framework, const char *source,
-                                                        VkShaderStageFlagBits stage, const spv_target_env spv_env,
-                                                        const VkSpecializationInfo *spec_info, const char *entry_point) {
-    auto shader = std::make_unique<VkShaderObj>(framework, source, stage, spv_env, SPV_SOURCE_ASM_TRY, spec_info, entry_point);
-    if (VK_SUCCESS == shader->InitFromASMTry()) {
+VkShaderObj VkShaderObj::CreateFromASM(VkRenderFramework *framework, const char *source, VkShaderStageFlagBits stage,
+                                       const spv_target_env spv_env, const VkSpecializationInfo *spec_info,
+                                       const char *entry_point) {
+    auto shader = VkShaderObj(framework, source, stage, spv_env, SPV_SOURCE_ASM_TRY, spec_info, entry_point);
+    if (VK_SUCCESS == shader.InitFromASMTry()) {
         return shader;
     }
     return {};
