@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (C) 2015-2024 Google Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (C) 2015-2025 Google Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,10 +22,9 @@
 #include <unordered_map>
 #include <vector>
 
-#include "containers/custom_containers.h"
 #include "generated/vk_dispatch_table_helper.h"
-#include "utils/vk_layer_utils.h"
 #include "vk_lunarg_device_profile_api_layer.h"
+#include "utils/dispatch_utils.h"
 #include <vulkan/utility/vk_struct_helper.hpp>
 
 namespace device_profile_api {
@@ -348,9 +347,9 @@ VKAPI_ATTR VkResult VKAPI_CALL EnumerateInstanceLayerProperties(uint32_t *pCount
 
 VKAPI_ATTR VkResult VKAPI_CALL EnumerateInstanceExtensionProperties(const char *pLayerName, uint32_t *pCount,
                                                                     VkExtensionProperties *pProperties) {
-    if (pLayerName && !strcmp(pLayerName, device_profile_api_LayerProps.layerName))
+    if (pLayerName && !strcmp(pLayerName, device_profile_api_LayerProps.layerName)) {
         return EnumerateProperties<VkExtensionProperties>(0, NULL, pCount, pProperties);
-
+    }
     return VK_ERROR_LAYER_NOT_PRESENT;
 }
 
