@@ -12,8 +12,8 @@
  */
 
 #include "../framework/layer_validation_tests.h"
-#include "../framework/barrier_queue_family.h"
 #include "../framework/pipeline_helper.h"
+#include "../framework/sync_helper.h"
 
 class PositiveBuffer : public VkLayerTest {};
 
@@ -60,11 +60,8 @@ TEST_F(PositiveBuffer, TexelBufferAlignmentIn13) {
     if (minTexelBufferOffsetAlignment == 1) {
         GTEST_SKIP() << "Test requires minTexelOffsetAlignment to not be equal to 1";
     }
-
-    VkFormatProperties format_properties;
-    vk::GetPhysicalDeviceFormatProperties(Gpu(), VK_FORMAT_R8G8B8A8_UNORM, &format_properties);
-    if (!(format_properties.bufferFeatures & VK_FORMAT_FEATURE_UNIFORM_TEXEL_BUFFER_BIT)) {
-        GTEST_SKIP() << "Test requires support for VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT";
+    if (!BufferFormatAndFeaturesSupported(Gpu(), VK_FORMAT_R8G8B8A8_UNORM, VK_FORMAT_FEATURE_UNIFORM_TEXEL_BUFFER_BIT)) {
+        GTEST_SKIP() << "Test requires support for VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT";
     }
 
     VkPhysicalDeviceVulkan13Properties props_1_3 = vku::InitStructHelper();
@@ -81,7 +78,7 @@ TEST_F(PositiveBuffer, TexelBufferAlignmentIn13) {
     VkBufferViewCreateInfo buff_view_ci = vku::InitStructHelper();
     buff_view_ci.format = VK_FORMAT_R8G8B8A8_UNORM;
     buff_view_ci.range = VK_WHOLE_SIZE;
-    buff_view_ci.buffer = buffer.handle();
+    buff_view_ci.buffer = buffer;
     buff_view_ci.offset = minTexelBufferOffsetAlignment + block_size;
     vkt::BufferView buffer_view(*m_device, buff_view_ci);
 }
@@ -117,7 +114,7 @@ TEST_F(PositiveBuffer, DISABLED_PerfGetBufferAddressWorstCase) {
         vkt::Buffer &buffer = buffers[i];
         buffer_ci.size = (i + 1) * 4096;
         buffer.InitNoMemory(*m_device, buffer_ci);
-        vk::BindBufferMemory(device(), buffer.handle(), buffer_memory.handle(), 0);
+        vk::BindBufferMemory(device(), buffer, buffer_memory, 0);
         VkDeviceAddress addr = buffer.Address();
         if (ref_address == 0) {
             ref_address = addr;
@@ -157,7 +154,7 @@ TEST_F(PositiveBuffer, DISABLED_PerfGetBufferAddressGoodCase) {
         vkt::Buffer &buffer = buffers[i];
         buffer.InitNoMemory(*m_device, buffer_ci);
         // Consecutive offsets
-        vk::BindBufferMemory(device(), buffer.handle(), buffer_memory.handle(), i * buffer_ci.size);
+        vk::BindBufferMemory(device(), buffer, buffer_memory, i * buffer_ci.size);
         (void)buffer.Address();
     }
 }
@@ -175,9 +172,9 @@ TEST_F(PositiveBuffer, IndexBuffer2Size) {
     m_command_buffer.Begin();
     m_command_buffer.BeginRenderPass(m_renderPassBeginInfo);
 
-    vk::CmdBindIndexBuffer2KHR(m_command_buffer.handle(), buffer.handle(), 4, 8, VK_INDEX_TYPE_UINT32);
+    vk::CmdBindIndexBuffer2KHR(m_command_buffer, buffer, 4, 8, VK_INDEX_TYPE_UINT32);
 
-    vk::CmdBindIndexBuffer2KHR(m_command_buffer.handle(), buffer.handle(), 0, buffer_size, VK_INDEX_TYPE_UINT32);
+    vk::CmdBindIndexBuffer2KHR(m_command_buffer, buffer, 0, buffer_size, VK_INDEX_TYPE_UINT32);
 
     m_command_buffer.EndRenderPass();
     m_command_buffer.End();
@@ -195,11 +192,9 @@ TEST_F(PositiveBuffer, IndexBufferNull) {
 
     m_command_buffer.Begin();
     m_command_buffer.BeginRenderPass(m_renderPassBeginInfo);
-    vk::CmdBindPipeline(m_command_buffer.handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.Handle());
-    vk::CmdDrawIndexed(m_command_buffer.handle(), 0, 1, 0, 0, 0);
-
-    vk::CmdBindIndexBuffer(m_command_buffer.handle(), VK_NULL_HANDLE, 0, VK_INDEX_TYPE_UINT32);
-    vk::CmdDrawIndexed(m_command_buffer.handle(), 0, 1, 0, 0, 0);
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_GRAPHICS, pipe);
+    vk::CmdBindIndexBuffer(m_command_buffer, VK_NULL_HANDLE, 0, VK_INDEX_TYPE_UINT32);
+    vk::CmdDrawIndexed(m_command_buffer, 0, 1, 0, 0, 0);
     m_command_buffer.EndRenderPass();
     m_command_buffer.End();
 }
@@ -219,7 +214,7 @@ TEST_F(PositiveBuffer, BufferViewUsageBasic) {
     VkBufferViewCreateInfo buffer_view_ci = vku::InitStructHelper(&buffer_usage_flags);
     buffer_view_ci.format = VK_FORMAT_R8G8B8A8_UNORM;
     buffer_view_ci.range = VK_WHOLE_SIZE;
-    buffer_view_ci.buffer = buffer.handle();
+    buffer_view_ci.buffer = buffer;
     vkt::BufferView buffer_view(*m_device, buffer_view_ci);
 }
 
@@ -238,7 +233,7 @@ TEST_F(PositiveBuffer, BufferUsageFlags2Subset) {
     VkBufferViewCreateInfo buffer_view_ci = vku::InitStructHelper(&buffer_usage_flags);
     buffer_view_ci.format = VK_FORMAT_R8G8B8A8_UNORM;
     buffer_view_ci.range = VK_WHOLE_SIZE;
-    buffer_view_ci.buffer = buffer.handle();
+    buffer_view_ci.buffer = buffer;
     vkt::BufferView buffer_view(*m_device, buffer_view_ci);
 }
 
