@@ -410,6 +410,55 @@ TEST_F(PositiveCopyBufferImage, UncompressedToCompressedImage2) {
     m_command_buffer.End();
 }
 
+TEST_F(PositiveCopyBufferImage, UncompressedToCompressedImage3) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-Docs/issues/1637");
+    RETURN_IF_SKIP(Init());
+
+    if (!FormatFeaturesAreSupported(Gpu(), VK_FORMAT_R32G32B32A32_UINT, VK_IMAGE_TILING_OPTIMAL, kSrcDstFeature) ||
+        !FormatFeaturesAreSupported(Gpu(), VK_FORMAT_BC6H_SFLOAT_BLOCK, VK_IMAGE_TILING_OPTIMAL, kSrcDstFeature)) {
+        GTEST_SKIP() << "Required formats/features not supported";
+    }
+
+    auto image_ci = vkt::Image::ImageCreateInfo2D(2, 2, 2, 1, VK_FORMAT_R32G32B32A32_UINT, kSrcDstUsage);
+    vkt::Image uncomp_image(*m_device, image_ci);
+
+    image_ci.format = VK_FORMAT_BC6H_SFLOAT_BLOCK;
+    image_ci.extent = {8, 8, 1};
+    image_ci.mipLevels = 4;
+    vkt::Image comp_image(*m_device, image_ci);
+    if (!uncomp_image.initialized() || !comp_image.initialized()) {
+        GTEST_SKIP() << "Unable to initialize surfaces";
+    }
+
+    VkImageCopy copy_region = {};
+    copy_region.srcSubresource = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 0, 1};
+    copy_region.dstSubresource = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 0, 1};
+    copy_region.srcOffset = {0, 0, 0};
+    copy_region.dstOffset = {0, 0, 0};
+
+    m_command_buffer.Begin();
+
+    copy_region.extent = {2, 2, 1};
+    vk::CmdCopyImage(m_command_buffer, uncomp_image, VK_IMAGE_LAYOUT_GENERAL, comp_image, VK_IMAGE_LAYOUT_GENERAL, 1, &copy_region);
+    m_command_buffer.FullMemoryBarrier();
+
+    copy_region.extent = {1, 1, 1};
+    copy_region.srcSubresource.mipLevel = 1;
+    copy_region.dstSubresource.mipLevel = 1;
+    vk::CmdCopyImage(m_command_buffer, uncomp_image, VK_IMAGE_LAYOUT_GENERAL, comp_image, VK_IMAGE_LAYOUT_GENERAL, 1, &copy_region);
+    m_command_buffer.FullMemoryBarrier();
+
+    copy_region.dstSubresource.mipLevel = 2;
+    vk::CmdCopyImage(m_command_buffer, uncomp_image, VK_IMAGE_LAYOUT_GENERAL, comp_image, VK_IMAGE_LAYOUT_GENERAL, 1, &copy_region);
+    m_command_buffer.FullMemoryBarrier();
+
+    copy_region.dstSubresource.mipLevel = 3;
+    vk::CmdCopyImage(m_command_buffer, uncomp_image, VK_IMAGE_LAYOUT_GENERAL, comp_image, VK_IMAGE_LAYOUT_GENERAL, 1, &copy_region);
+    m_command_buffer.FullMemoryBarrier();
+
+    m_command_buffer.End();
+}
+
 TEST_F(PositiveCopyBufferImage, UncompressedToCompressedNonPowerOfTwo) {
     RETURN_IF_SKIP(Init());
 
@@ -815,8 +864,7 @@ TEST_F(PositiveCopyBufferImage, Compressed2DToUncompressed1D) {
     m_command_buffer.End();
 }
 
-// TODO - Need WG agreement how this works still
-TEST_F(PositiveCopyBufferImage, DISABLED_CopyCompress1DTo2D) {
+TEST_F(PositiveCopyBufferImage, CopyCompress1DTo2D) {
     TEST_DESCRIPTION("Copy a 1D compressed format to a 2D compressed format");
     SetTargetApiVersion(VK_API_VERSION_1_1);
     AddRequiredExtensions(VK_KHR_MAINTENANCE_5_EXTENSION_NAME);
@@ -901,25 +949,14 @@ TEST_F(PositiveCopyBufferImage, CopyBufferTo1DCompressedImage) {
     m_command_buffer.End();
 }
 
-// TODO - Need WG agreement how this works still
-TEST_F(PositiveCopyBufferImage, DISABLED_CopyCompress2DTo1D) {
+TEST_F(PositiveCopyBufferImage, CopyCompress2DTo1D) {
     TEST_DESCRIPTION("Copy a compressed 2D image to a compressed 1D image");
     SetTargetApiVersion(VK_API_VERSION_1_1);
     AddRequiredExtensions(VK_KHR_MAINTENANCE_5_EXTENSION_NAME);
     AddRequiredFeature(vkt::Feature::maintenance5);
     RETURN_IF_SKIP(Init());
 
-    VkImageCreateInfo image_ci = vku::InitStructHelper();
-    image_ci.flags = 0u;
-    image_ci.imageType = VK_IMAGE_TYPE_2D;
-    image_ci.format = VK_FORMAT_BC1_RGBA_UNORM_BLOCK;
-    image_ci.extent = {64u, 64u, 1u};
-    image_ci.mipLevels = 1u;
-    image_ci.arrayLayers = 1u;
-    image_ci.samples = VK_SAMPLE_COUNT_1_BIT;
-    image_ci.tiling = VK_IMAGE_TILING_OPTIMAL;
-    image_ci.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
-    image_ci.usage = VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
+    auto image_ci = vkt::Image::ImageCreateInfo2D(64, 64, 1, 1, VK_FORMAT_BC1_RGBA_UNORM_BLOCK, VK_IMAGE_USAGE_TRANSFER_SRC_BIT);
     if (!ImageFormatIsSupported(instance(), Gpu(), image_ci, VK_FORMAT_FEATURE_TRANSFER_SRC_BIT)) {
         GTEST_SKIP() << "image format not supported";
     }
@@ -1069,19 +1106,19 @@ TEST_F(PositiveCopyBufferImage, CompressedImageMip) {
     vk::CmdCopyBufferToImage(m_command_buffer, buffer_16, image, VK_IMAGE_LAYOUT_GENERAL, 1, &region);
 }
 
-// TODO - Need WG agreement how this works still
-TEST_F(PositiveCopyBufferImage, DISABLED_CopyCompressToCompress) {
+TEST_F(PositiveCopyBufferImage, CopyCompressToCompress) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-Docs/issues/1005");
     RETURN_IF_SKIP(Init());
 
     VkImageCreateInfo image_ci = vku::InitStructHelper();
     image_ci.imageType = VK_IMAGE_TYPE_2D;
-    image_ci.format = VK_FORMAT_BC1_RGBA_UNORM_BLOCK;
-    image_ci.mipLevels = 4u;
+    image_ci.format = VK_FORMAT_BC2_UNORM_BLOCK;
+    image_ci.mipLevels = 2u;  // [30x30], [15x15]
     image_ci.arrayLayers = 1u;
     image_ci.samples = VK_SAMPLE_COUNT_1_BIT;
     image_ci.tiling = VK_IMAGE_TILING_OPTIMAL;
     image_ci.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
-    image_ci.extent = {16, 16u, 1u};
+    image_ci.extent = {30, 30, 1};
     image_ci.usage = kSrcDstUsage;
     if (!ImageFormatIsSupported(instance(), Gpu(), image_ci, kSrcDstFeature)) {
         GTEST_SKIP() << "image format not supported";
@@ -1096,12 +1133,25 @@ TEST_F(PositiveCopyBufferImage, DISABLED_CopyCompressToCompress) {
     VkImageCopy image_copy;
     image_copy.srcSubresource = {1u, 0u, 0u, 1u};
     image_copy.srcOffset = {0u, 0u, 0u};
-    image_copy.dstSubresource = {1u, 3u, 0u, 1u};  // last mip
+    image_copy.dstSubresource = {1u, 0u, 0u, 1u};
     image_copy.dstOffset = {0u, 0u, 0u};
-    image_copy.extent = {2u, 2u, 1u};  // single texel
 
-    vk::CmdCopyImage(m_command_buffer, src_image, VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL, dst_image,
-                     VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 1u, &image_copy);
+    {
+        image_copy.srcSubresource.mipLevel = 0;
+        image_copy.dstSubresource.mipLevel = 1;
+        image_copy.extent = {16u, 16u, 1u};
+        vk::CmdCopyImage(m_command_buffer, src_image, VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL, dst_image,
+                         VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 1u, &image_copy);
+    }
+    m_command_buffer.FullMemoryBarrier();
+    {
+        image_copy.srcSubresource.mipLevel = 1;
+        image_copy.dstSubresource.mipLevel = 0;
+        image_copy.extent = {15u, 15u, 1u};
+        vk::CmdCopyImage(m_command_buffer, src_image, VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL, dst_image,
+                         VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL, 1u, &image_copy);
+    }
+
     m_command_buffer.End();
 }
 
@@ -1743,3 +1793,111 @@ TEST_F(PositiveCopyBufferImage, Transition3dImageSlices) {
     m_command_buffer.End();
     m_default_queue->SubmitAndWait(m_command_buffer);
 }
+
+TEST_F(PositiveCopyBufferImage, MemoryIndirect) {
+    AddRequiredExtensions(VK_KHR_COPY_MEMORY_INDIRECT_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::indirectMemoryCopy);
+    AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
+    RETURN_IF_SKIP(Init());
+
+    VkCopyMemoryIndirectCommandKHR cmds[2] = {
+        {4, 4, 0},
+        {0, 0, 0},
+    };
+    const VkDeviceSize buffer_size = sizeof(cmds);
+
+    vkt::Buffer indirect_buffer(*m_device, buffer_size, 0, vkt::device_address);
+    void *indirect_buffer_data = indirect_buffer.Memory().Map();
+    memcpy(indirect_buffer_data, cmds, buffer_size);
+
+    VkStridedDeviceAddressRangeKHR address_range = {};
+    address_range.address = indirect_buffer.Address();
+    address_range.size = buffer_size;
+    address_range.stride = sizeof(VkCopyMemoryIndirectCommandKHR);
+
+    VkCopyMemoryIndirectInfoKHR copy_info = vku::InitStructHelper();
+    copy_info.copyCount = 2;
+    copy_info.copyAddressRange = address_range;
+    copy_info.srcCopyFlags = VK_ADDRESS_COPY_DEVICE_LOCAL_BIT_KHR;
+    copy_info.dstCopyFlags = VK_ADDRESS_COPY_DEVICE_LOCAL_BIT_KHR;
+
+    m_command_buffer.Begin();
+    vk::CmdCopyMemoryIndirectKHR(m_command_buffer, &copy_info);
+    m_command_buffer.End();
+}
+
+TEST_F(PositiveCopyBufferImage, MemoryToImageIndirect) {
+    AddRequiredExtensions(VK_KHR_COPY_MEMORY_INDIRECT_EXTENSION_NAME);
+    AddRequiredFeature(vkt::Feature::indirectMemoryToImageCopy);
+    AddRequiredFeature(vkt::Feature::bufferDeviceAddress);
+    RETURN_IF_SKIP(Init());
+
+    VkCopyMemoryToImageIndirectCommandKHR cmd1 = {};
+    cmd1.srcAddress = 0;
+    cmd1.bufferRowLength = 8;
+    cmd1.bufferImageHeight = 8;
+    cmd1.imageSubresource = {};
+    cmd1.imageOffset = {0, 0, 0};
+    cmd1.imageExtent = {8, 8, 1};
+
+    VkCopyMemoryToImageIndirectCommandKHR cmd2 = {};
+    cmd2.srcAddress = 1024;
+    cmd2.bufferRowLength = 4;
+    cmd2.bufferImageHeight = 4;
+    cmd2.imageSubresource = {};
+    cmd2.imageOffset = {0, 0, 0};
+    cmd2.imageExtent = {4, 4, 1};
+
+    VkCopyMemoryToImageIndirectCommandKHR cmds[2] = {cmd1, cmd2};
+
+    vkt::Buffer indirect_buffer(*m_device, sizeof(cmds), 0, vkt::device_address);
+    void *indirect_buffer_data = indirect_buffer.Memory().Map();
+    memcpy(indirect_buffer_data, cmds, sizeof(cmds));
+
+    VkImageSubresourceLayers res_layer = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 0, 1};
+    VkImageSubresourceLayers res_layers[2] = {res_layer, res_layer};
+
+    m_command_buffer.Begin();
+
+    VkImageCreateInfo image_create_info = vku::InitStructHelper();
+    image_create_info.imageType = VK_IMAGE_TYPE_2D;
+    image_create_info.format = VK_FORMAT_R8G8B8A8_UNORM;
+    image_create_info.extent = {32, 32, 1};
+    image_create_info.mipLevels = 1;
+    image_create_info.arrayLayers = 4;
+    image_create_info.samples = VK_SAMPLE_COUNT_1_BIT;
+    image_create_info.tiling = VK_IMAGE_TILING_OPTIMAL;
+    image_create_info.usage = VK_IMAGE_USAGE_TRANSFER_SRC_BIT;
+    image_create_info.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+    image_create_info.flags = 0;
+    vkt::Image src_image(*m_device, image_create_info, vkt::set_layout);
+
+    image_create_info.usage = VK_IMAGE_USAGE_TRANSFER_DST_BIT;
+    vkt::Image dst_image(*m_device, image_create_info, vkt::set_layout);
+
+    VkImageCopy copy_region;
+    copy_region.srcSubresource = {VK_IMAGE_ASPECT_COLOR_BIT, 0, 0, 1};
+    copy_region.dstSubresource = copy_region.srcSubresource;
+    copy_region.srcOffset = {0, 0, 0};
+    copy_region.dstOffset = {0, 0, 0};
+    copy_region.extent = {1, 1, 1};
+
+    const uint32_t stride = sizeof(VkCopyMemoryToImageIndirectCommandKHR);
+
+    VkStridedDeviceAddressRangeKHR address_range = {};
+    address_range.address = indirect_buffer.Address();
+    address_range.size = sizeof(cmds);
+    address_range.stride = stride;
+
+    VkCopyMemoryToImageIndirectInfoKHR copy_info = vku::InitStructHelper();
+    copy_info.copyCount = 2;
+    copy_info.copyAddressRange = address_range;
+    copy_info.srcCopyFlags = VK_ADDRESS_COPY_DEVICE_LOCAL_BIT_KHR;
+    copy_info.dstImage = dst_image;
+    copy_info.dstImageLayout = VK_IMAGE_LAYOUT_GENERAL;
+    copy_info.pImageSubresources = res_layers;
+
+    vk::CmdCopyImage(m_command_buffer, src_image, VK_IMAGE_LAYOUT_GENERAL, dst_image, VK_IMAGE_LAYOUT_GENERAL, 1, &copy_region);
+    vk::CmdCopyMemoryToImageIndirectKHR(m_command_buffer, &copy_info);
+    m_command_buffer.End();
+}
