@@ -202,7 +202,7 @@ TEST_F(PositiveDescriptorBuffer, Basic) {
     void *mapped_descriptor_data = descriptor_buffer.Memory().Map();
     vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize, mapped_descriptor_data);
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout (set = 0, binding = 0) buffer SSBO_0 {
             uint a;
@@ -267,25 +267,23 @@ TEST_F(PositiveDescriptorBuffer, BasicSampler) {
     uint8_t *mapped_descriptor_data = (uint8_t *)descriptor_buffer.Memory().Map();
 
     vkt::DescriptorGetInfo get_info_sampler(&sampler.handle());
-    vk::GetDescriptorEXT(device(), get_info_sampler, descriptor_buffer_properties.samplerDescriptorSize, mapped_descriptor_data);
+    vk::GetDescriptorEXT(device(), get_info_sampler, descriptor_buffer_properties.samplerDescriptorSize,
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(0));
 
     vkt::DescriptorGetInfo get_info_image(VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, VK_NULL_HANDLE, image_view, VK_IMAGE_LAYOUT_GENERAL);
-    VkDeviceSize ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(1);
     vk::GetDescriptorEXT(device(), get_info_image, descriptor_buffer_properties.sampledImageDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(1));
 
     vkt::DescriptorGetInfo get_info_combined(VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER, sampler, image_view,
                                              VK_IMAGE_LAYOUT_GENERAL);
-    ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(2);
     vk::GetDescriptorEXT(device(), get_info_combined, descriptor_buffer_properties.combinedImageSamplerDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(2));
 
     vkt::DescriptorGetInfo get_info_buffer(VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, result_buffer, 32);
-    ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(3);
     vk::GetDescriptorEXT(device(), get_info_buffer, descriptor_buffer_properties.storageBufferDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(3));
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout(set = 0, binding = 0) uniform sampler s;
         layout(set = 0, binding = 1) uniform texture2D t; // sampled image
@@ -347,18 +345,18 @@ TEST_F(PositiveDescriptorBuffer, MultipleDescriptors) {
 
     uint8_t *mapped_descriptor_data = (uint8_t *)resource_descriptor_buffer.Memory().Map();
     vkt::DescriptorGetInfo get_info_image(VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE, VK_NULL_HANDLE, image_view, VK_IMAGE_LAYOUT_GENERAL);
-    vk::GetDescriptorEXT(device(), get_info_image, descriptor_buffer_properties.sampledImageDescriptorSize, mapped_descriptor_data);
+    vk::GetDescriptorEXT(device(), get_info_image, descriptor_buffer_properties.sampledImageDescriptorSize,
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(0));
 
     vkt::DescriptorGetInfo get_info_buffer(VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, result_buffer, 32);
-    VkDeviceSize ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(1);
     vk::GetDescriptorEXT(device(), get_info_buffer, descriptor_buffer_properties.storageBufferDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(1));
 
     mapped_descriptor_data = (uint8_t *)sampler_descriptor_buffer.Memory().Map();
     vkt::DescriptorGetInfo get_info_sampler(&sampler.handle());
     vk::GetDescriptorEXT(device(), get_info_sampler, descriptor_buffer_properties.samplerDescriptorSize, mapped_descriptor_data);
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout(set = 0, binding = 2) uniform sampler s;
         layout(set = 0, binding = 0) uniform texture2D t;
@@ -435,7 +433,7 @@ TEST_F(PositiveDescriptorBuffer, MultipleSet) {
     mapped_descriptor_data += ds_layout_size;
     vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize, mapped_descriptor_data);
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout (set = 0, binding = 0) buffer SSBO_0 {
             uint a;
@@ -503,21 +501,20 @@ TEST_F(PositiveDescriptorBuffer, MultipleBinding) {
     vkt::DescriptorGetInfo get_info(VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, buffer_data, 4);
     uint8_t *mapped_descriptor_data = (uint8_t *)descriptor_buffer.Memory().Map();
     // Sets data_buffer[0] to binding 0
-    vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize, mapped_descriptor_data);
+    vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize,
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(0));
 
     // Sets data_buffer[1] to binding 1
     get_info.address_info.address += 4;
-    VkDeviceSize ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(1);
     vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(1));
 
     // Sets data_buffer[2] to binding 2
     get_info.address_info.address += 4;
-    ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(2);
     vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(2));
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout (set = 0, binding = 0) buffer SSBO_0 {
             uint a;
@@ -599,7 +596,7 @@ TEST_F(PositiveDescriptorBuffer, DescriptorIndexing) {
     mapped_descriptor_data += descriptor_size;
     vk::GetDescriptorEXT(device(), get_info, descriptor_size, mapped_descriptor_data);
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout (set = 0, binding = 0) buffer SSBO_0 {
             uint data;
@@ -711,14 +708,13 @@ TEST_F(PositiveDescriptorBuffer, TexelBuffer) {
     vkt::DescriptorGetInfo get_info_s(VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER, storage_buffer, 32, VK_FORMAT_R32_UINT);
     uint8_t *mapped_descriptor_data = (uint8_t *)descriptor_buffer.Memory().Map();
     vk::GetDescriptorEXT(device(), get_info_s, descriptor_buffer_properties.storageTexelBufferDescriptorSize,
-                         mapped_descriptor_data);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(0));
 
     vkt::DescriptorGetInfo get_info_u(VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER, uniform_buffer, 32, VK_FORMAT_R32_UINT);
-    VkDeviceSize ds_layout_binding_offset = ds_layout.GetDescriptorBufferBindingOffset(1);
     vk::GetDescriptorEXT(device(), get_info_u, descriptor_buffer_properties.uniformTexelBufferDescriptorSize,
-                         mapped_descriptor_data + ds_layout_binding_offset);
+                         mapped_descriptor_data + ds_layout.GetDescriptorBufferBindingOffset(1));
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout(set = 0, binding = 0, r32ui) uniform uimageBuffer s_buffer;
         layout(set = 0, binding = 1) uniform usamplerBuffer u_buffer;
@@ -754,14 +750,13 @@ TEST_F(PositiveDescriptorBuffer, TexelBuffer) {
 
     m_default_queue->SubmitAndWait(m_command_buffer);
 
-    // TODO - Seems on NVIDIA this is not working, need to investigate, but still want rest of testing for API usage
-    // data = (uint32_t *)storage_buffer.Memory().Map();
-    // if (!IsPlatformMockICD()) {
-    //     ASSERT_TRUE(data[0] == 8);
-    //     ASSERT_TRUE(data[1] == 12);
-    //     ASSERT_TRUE(data[2] == 1);
-    //     ASSERT_TRUE(data[3] == 4);
-    // }
+    data = (uint32_t *)storage_buffer.Memory().Map();
+    if (!IsPlatformMockICD()) {
+        ASSERT_TRUE(data[0] == 8);
+        ASSERT_TRUE(data[1] == 12);
+        ASSERT_TRUE(data[2] == 1);
+        ASSERT_TRUE(data[3] == 4);
+    }
 }
 
 TEST_F(PositiveDescriptorBuffer, BindingOffsets) {
@@ -791,7 +786,7 @@ TEST_F(PositiveDescriptorBuffer, BindingOffsets) {
     mapped_descriptor_data += ds_layout_size;
     vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize, mapped_descriptor_data);
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout (set = 0, binding = 0) buffer SSBO_0 {
             uint data;
@@ -865,7 +860,7 @@ TEST_F(PositiveDescriptorBuffer, ShaderObject) {
     void *mapped_descriptor_data = descriptor_buffer.Memory().Map();
     vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize, mapped_descriptor_data);
 
-    char const *cs_source = R"glsl(
+    const char *cs_source = R"glsl(
         #version 450
         layout (set = 0, binding = 0) buffer SSBO_0 {
             uint a;
@@ -904,4 +899,55 @@ TEST_F(PositiveDescriptorBuffer, ShaderObject) {
         ASSERT_TRUE(data[1] == 12);
         ASSERT_TRUE(data[2] == 20);
     }
-}
\ No newline at end of file
+}
+
+TEST_F(PositiveDescriptorBuffer, NotInvalidatedLegacy) {
+    TEST_DESCRIPTION("Case 3 from https://gitlab.khronos.org/vulkan/vulkan/-/merge_requests/7504#note_549388");
+    RETURN_IF_SKIP(InitBasicDescriptorBuffer());
+
+    vkt::Buffer legacy_buffer(*m_device, 16, VK_BUFFER_USAGE_STORAGE_BUFFER_BIT);
+    VkDescriptorSetLayoutBinding binding = {0, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, 1, VK_SHADER_STAGE_ALL, nullptr};
+    OneOffDescriptorSet legacy_ds(m_device, {binding});
+    vkt::PipelineLayout pipeline_layout(*m_device, {&legacy_ds.layout_});
+    legacy_ds.WriteDescriptorBufferInfo(0, legacy_buffer, 0, VK_WHOLE_SIZE, VK_DESCRIPTOR_TYPE_STORAGE_BUFFER);
+    legacy_ds.UpdateDescriptorSets();
+
+    vkt::Buffer buffer_data(*m_device, 16, 0, vkt::device_address);
+    vkt::DescriptorSetLayout ds_layout(*m_device, binding, VK_DESCRIPTOR_SET_LAYOUT_CREATE_DESCRIPTOR_BUFFER_BIT_EXT);
+
+    VkDeviceSize ds_layout_size = ds_layout.GetDescriptorBufferSize();
+    vkt::Buffer descriptor_buffer(*m_device, ds_layout_size, VK_BUFFER_USAGE_RESOURCE_DESCRIPTOR_BUFFER_BIT_EXT,
+                                  vkt::device_address);
+
+    vkt::DescriptorGetInfo get_info(VK_DESCRIPTOR_TYPE_STORAGE_BUFFER, buffer_data, 16);
+
+    void *mapped_descriptor_data = descriptor_buffer.Memory().Map();
+    vk::GetDescriptorEXT(device(), get_info, descriptor_buffer_properties.storageBufferDescriptorSize, mapped_descriptor_data);
+
+    const char *cs_source = R"glsl(
+        #version 450
+        layout (set = 0, binding = 0) buffer SSBO_0 { uint x; };
+        void main() {
+            x = 0;
+        }
+    )glsl";
+
+    CreateComputePipelineHelper pipe(*this);
+    pipe.cs_ = VkShaderObj(this, cs_source, VK_SHADER_STAGE_COMPUTE_BIT, SPV_ENV_VULKAN_1_2);
+    pipe.cp_ci_.layout = pipeline_layout;
+    pipe.CreateComputePipeline();
+
+    m_command_buffer.Begin();
+    vk::CmdBindPipeline(m_command_buffer, VK_PIPELINE_BIND_POINT_COMPUTE, pipe);
+
+    vk::CmdBindDescriptorSets(m_command_buffer, VK_PIPELINE_BIND_POINT_COMPUTE, pipeline_layout, 0, 1, &legacy_ds.set_, 0, nullptr);
+
+    // Does not invalidate the legacy by itself
+    VkDescriptorBufferBindingInfoEXT descriptor_buffer_binding_info = vku::InitStructHelper();
+    descriptor_buffer_binding_info.address = descriptor_buffer.Address();
+    descriptor_buffer_binding_info.usage = VK_BUFFER_USAGE_RESOURCE_DESCRIPTOR_BUFFER_BIT_EXT;
+    vk::CmdBindDescriptorBuffersEXT(m_command_buffer, 1, &descriptor_buffer_binding_info);
+
+    vk::CmdDispatch(m_command_buffer, 1, 1, 1);
+    m_command_buffer.End();
+}
