@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2015-2024 The Khronos Group Inc.
- * Copyright (c) 2015-2024 Valve Corporation
- * Copyright (c) 2015-2024 LunarG, Inc.
- * Copyright (c) 2015-2024 Google, Inc.
+ * Copyright (c) 2015-2025 The Khronos Group Inc.
+ * Copyright (c) 2015-2025 Valve Corporation
+ * Copyright (c) 2015-2025 LunarG, Inc.
+ * Copyright (c) 2015-2025 Google, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -47,6 +47,8 @@ class PositiveImageDrm : public ImageDrmTest {};
 
 TEST_F(PositiveImageDrm, Basic) {
     // See https://github.com/KhronosGroup/Vulkan-ValidationLayers/pull/2610
+    // for more detailed checking, we could export the image to dmabuf
+    // and then import it again (using VkImageDrmFormatModifierExplicitCreateInfoEXT)
     TEST_DESCRIPTION("Create image and imageView using VK_EXT_image_drm_format_modifier");
     RETURN_IF_SKIP(InitBasicImageDrm());
 
@@ -85,52 +87,18 @@ TEST_F(PositiveImageDrm, Basic) {
         mod_list.drmFormatModifierCount = mods.size();
         ci.pNext = &mod_list;
 
-        VkImage image;
-        VkResult err = vk::CreateImage(device(), &ci, nullptr, &image);
-        ASSERT_EQ(VK_SUCCESS, err);
+        vkt::Image image(*m_device, ci, vkt::no_mem);
 
-        // bind memory
-        VkPhysicalDeviceMemoryProperties phys_mem_props;
-        vk::GetPhysicalDeviceMemoryProperties(Gpu(), &phys_mem_props);
-        VkMemoryRequirements mem_reqs;
-        vk::GetImageMemoryRequirements(device(), image, &mem_reqs);
-        VkDeviceMemory mem_obj = VK_NULL_HANDLE;
-        VkMemoryPropertyFlagBits mem_props = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
-
-        for (uint32_t type = 0; type < phys_mem_props.memoryTypeCount; type++) {
-            if ((mem_reqs.memoryTypeBits & (1 << type)) &&
-                ((phys_mem_props.memoryTypes[type].propertyFlags & mem_props) == mem_props)) {
-                VkMemoryAllocateInfo alloc_info = vku::InitStructHelper();
-                alloc_info.allocationSize = mem_reqs.size;
-                alloc_info.memoryTypeIndex = type;
-                ASSERT_EQ(VK_SUCCESS, vk::AllocateMemory(device(), &alloc_info, nullptr, &mem_obj));
-                break;
-            }
+        VkMemoryRequirements mem_reqs = image.MemoryRequirements();
+        VkMemoryAllocateInfo alloc_info = vku::InitStructHelper();
+        alloc_info.allocationSize = mem_reqs.size;
+        if (!m_device->Physical().SetMemoryType(mem_reqs.memoryTypeBits, &alloc_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)) {
+            GTEST_SKIP() << "Memory type not found";
         }
 
-        ASSERT_NE((VkDeviceMemory)VK_NULL_HANDLE, mem_obj);
-        ASSERT_EQ(VK_SUCCESS, vk::BindImageMemory(device(), image, mem_obj, 0));
-
-        // create image view
-        VkImageViewCreateInfo ivci = {
-            VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO,
-            nullptr,
-            0,
-            image,
-            VK_IMAGE_VIEW_TYPE_2D,
-            format,
-            {VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY,
-             VK_COMPONENT_SWIZZLE_IDENTITY},
-            {VK_IMAGE_ASPECT_COLOR_BIT, 0, 1, 0, 1},
-        };
-
-        CreateImageViewTest(*this, &ivci);
-
-        // for more detailed checking, we could export the image to dmabuf
-        // and then import it again (using VkImageDrmFormatModifierExplicitCreateInfoEXT)
-
-        vk::FreeMemory(device(), mem_obj, nullptr);
-        vk::DestroyImage(device(), image, nullptr);
+        vkt::DeviceMemory memory(*m_device, alloc_info);
+        ASSERT_EQ(VK_SUCCESS, vk::BindImageMemory(device(), image, memory, 0));
+        vkt::ImageView image_view = image.CreateView();
     }
 }
 
@@ -151,9 +119,6 @@ TEST_F(PositiveImageDrm, ExternalMemory) {
     VkExternalMemoryImageCreateInfo external_info = vku::InitStructHelper();
     external_info.handleTypes = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT;
 
-    // handleTypes needs to be assigned to trigger the behavior we want
-    assert(external_info.handleTypes);
-
     VkImageDrmFormatModifierListCreateInfoEXT drm_info = vku::InitStructHelper(&external_info);
     drm_info.drmFormatModifierCount = size32(mods);
     drm_info.pDrmFormatModifiers = mods.data();
@@ -197,8 +162,19 @@ TEST_F(PositiveImageDrm, ExternalMemory) {
             GTEST_SKIP() << "Unable to create image, VkExternalMemoryImageCreateInfo::handleTypes not supported";
         }
     }
+    vkt::Image image(*m_device, ci, vkt::no_mem);
 
-    CreateImageTest(*this, &ci);
+    VkMemoryDedicatedAllocateInfo dedicated_info = vku::InitStructHelper();
+    dedicated_info.image = image;
+    VkMemoryAllocateInfo alloc_info = vku::InitStructHelper(&dedicated_info);
+    VkMemoryRequirements mem_reqs = image.MemoryRequirements();
+    alloc_info.allocationSize = mem_reqs.size;
+    if (!m_device->Physical().SetMemoryType(mem_reqs.memoryTypeBits, &alloc_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)) {
+        GTEST_SKIP() << "Memory type not found";
+    }
+
+    vkt::DeviceMemory memory(*m_device, alloc_info);
+    ASSERT_EQ(VK_SUCCESS, vk::BindImageMemory(device(), image, memory, 0));
 }
 
 TEST_F(PositiveImageDrm, GetImageSubresourceLayoutPlane) {
@@ -249,7 +225,7 @@ TEST_F(PositiveImageDrm, GetImageSubresourceLayoutPlane) {
 
     VkImageSubresource subresource{VK_IMAGE_ASPECT_MEMORY_PLANE_1_BIT_EXT, 0, 0};
     VkSubresourceLayout layout{};
-    vk::GetImageSubresourceLayout(m_device->handle(), image.handle(), &subresource, &layout);
+    vk::GetImageSubresourceLayout(*m_device, image, &subresource, &layout);
 }
 
 TEST_F(PositiveImageDrm, MutableFormat) {
@@ -282,7 +258,96 @@ TEST_F(PositiveImageDrm, MutableFormat) {
     image_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
     image_info.sharingMode = VK_SHARING_MODE_EXCLUSIVE;
     image_info.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
-    CreateImageTest(*this, &image_info);
+    vkt::Image image(*m_device, image_info, vkt::no_mem);
+
+    VkMemoryRequirements mem_reqs = image.MemoryRequirements();
+    VkMemoryAllocateInfo alloc_info = vku::InitStructHelper();
+    alloc_info.allocationSize = mem_reqs.size;
+    if (!m_device->Physical().SetMemoryType(mem_reqs.memoryTypeBits, &alloc_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)) {
+        GTEST_SKIP() << "Memory type not found";
+    }
+
+    vkt::DeviceMemory memory(*m_device, alloc_info);
+    ASSERT_EQ(VK_SUCCESS, vk::BindImageMemory(device(), image, memory, 0));
+}
+
+TEST_F(PositiveImageDrm, MutableExternalMemory) {
+    TEST_DESCRIPTION("https://github.com/KhronosGroup/Vulkan-ValidationLayers/issues/8627");
+    AddRequiredExtensions(VK_KHR_EXTERNAL_MEMORY_FD_EXTENSION_NAME);
+    RETURN_IF_SKIP(InitBasicImageDrm());
+
+    const VkFormat format = VK_FORMAT_R8G8B8A8_UNORM;
+    std::vector<uint64_t> mods = GetFormatModifier(format, VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT);
+    if (mods.empty()) {
+        GTEST_SKIP() << "No valid Format Modifier found";
+    }
+
+    VkFormat formats = VK_FORMAT_R8G8B8A8_SNORM;
+    VkImageFormatListCreateInfo format_list = vku::InitStructHelper();
+    format_list.viewFormatCount = 1;
+    format_list.pViewFormats = &formats;
+
+    VkExternalMemoryImageCreateInfo external_info = vku::InitStructHelper(&format_list);
+    external_info.handleTypes = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT;
+
+    VkImageDrmFormatModifierListCreateInfoEXT drm_info = vku::InitStructHelper(&external_info);
+    drm_info.drmFormatModifierCount = size32(mods);
+    drm_info.pDrmFormatModifiers = mods.data();
+
+    VkImageCreateInfo ci = vku::InitStructHelper(&drm_info);
+    ci.flags = VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT;
+    ci.imageType = VK_IMAGE_TYPE_2D;
+    ci.format = format;
+    ci.extent = {128, 128, 1};
+    ci.mipLevels = 1;
+    ci.arrayLayers = 1;
+    ci.samples = VK_SAMPLE_COUNT_1_BIT;
+    ci.tiling = VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT;
+    ci.usage = VK_IMAGE_USAGE_TRANSFER_SRC_BIT | VK_IMAGE_USAGE_TRANSFER_DST_BIT | VK_IMAGE_USAGE_SAMPLED_BIT |
+               VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT;
+    ci.sharingMode = VK_SHARING_MODE_EXCLUSIVE;
+    ci.initialLayout = VK_IMAGE_LAYOUT_UNDEFINED;
+
+    {
+        VkPhysicalDeviceImageDrmFormatModifierInfoEXT drm_format_modifier = vku::InitStructHelper(&format_list);
+        drm_format_modifier.sharingMode = ci.sharingMode;
+        drm_format_modifier.queueFamilyIndexCount = ci.queueFamilyIndexCount;
+        drm_format_modifier.pQueueFamilyIndices = ci.pQueueFamilyIndices;
+        VkPhysicalDeviceExternalImageFormatInfo external_image_info = vku::InitStructHelper(&drm_format_modifier);
+        external_image_info.handleType = static_cast<VkExternalMemoryHandleTypeFlagBits>(external_info.handleTypes);
+        VkPhysicalDeviceImageFormatInfo2 image_info = vku::InitStructHelper(&external_image_info);
+        image_info.format = ci.format;
+        image_info.type = ci.imageType;
+        image_info.tiling = ci.tiling;
+        image_info.usage = ci.usage;
+        image_info.flags = ci.flags;
+
+        VkExternalImageFormatProperties external_image_properties = vku::InitStructHelper();
+        VkImageFormatProperties2 image_properties = vku::InitStructHelper(&external_image_properties);
+
+        if (const auto result = vk::GetPhysicalDeviceImageFormatProperties2(Gpu(), &image_info, &image_properties);
+            result != VK_SUCCESS) {
+            GTEST_SKIP() << "Unable to create image. VkResult = " << string_VkResult(result);
+        }
+
+        if ((external_image_properties.externalMemoryProperties.compatibleHandleTypes & external_info.handleTypes) !=
+            external_info.handleTypes) {
+            GTEST_SKIP() << "Unable to create image, VkExternalMemoryImageCreateInfo::handleTypes not supported";
+        }
+    }
+    vkt::Image image(*m_device, ci, vkt::no_mem);
+
+    VkMemoryDedicatedAllocateInfo dedicated_info = vku::InitStructHelper();
+    dedicated_info.image = image;
+    VkMemoryAllocateInfo alloc_info = vku::InitStructHelper(&dedicated_info);
+    VkMemoryRequirements mem_reqs = image.MemoryRequirements();
+    alloc_info.allocationSize = mem_reqs.size;
+    if (!m_device->Physical().SetMemoryType(mem_reqs.memoryTypeBits, &alloc_info, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT)) {
+        GTEST_SKIP() << "Memory type not found";
+    }
+
+    vkt::DeviceMemory memory(*m_device, alloc_info);
+    ASSERT_EQ(VK_SUCCESS, vk::BindImageMemory(device(), image, memory, 0));
 }
 
 TEST_F(PositiveImageDrm, GetImageDrmFormatModifierProperties) {
@@ -312,7 +377,7 @@ TEST_F(PositiveImageDrm, GetImageDrmFormatModifierProperties) {
     vkt::Image image(*m_device, image_info);
 
     VkImageDrmFormatModifierPropertiesEXT props = vku::InitStructHelper();
-    vk::GetImageDrmFormatModifierPropertiesEXT(device(), image.handle(), &props);
+    vk::GetImageDrmFormatModifierPropertiesEXT(device(), image, &props);
 }
 
 TEST_F(PositiveImageDrm, PhysicalDeviceImageDrmFormatModifierInfoExclusive) {
