@@ -150,7 +150,7 @@ TEST_F(NegativeShaderStorageImage, MissingFormatReadForFormat) {
 
     struct {
         VkFormat format;
-        VkFormatProperties3KHR props;
+        VkFormatProperties3 props;
     } tests[2] = {};
     int n_tests = 0;
     bool has_without_format_test = false, has_with_format_test = false;
@@ -160,7 +160,7 @@ TEST_F(NegativeShaderStorageImage, MissingFormatReadForFormat) {
         if (has_without_format_test && has_with_format_test) break;
         if (!vkuFormatIsSampledFloat((VkFormat)fmt)) continue;
 
-        VkFormatProperties3KHR fmt_props_3 = vku::InitStructHelper();
+        VkFormatProperties3 fmt_props_3 = vku::InitStructHelper();
         VkFormatProperties2 fmt_props = vku::InitStructHelper(&fmt_props_3);
 
         vk::GetPhysicalDeviceFormatProperties2(Gpu(), (VkFormat)fmt, &fmt_props);
@@ -298,7 +298,7 @@ TEST_F(NegativeShaderStorageImage, MissingFormatWriteForFormat) {
 
     struct {
         VkFormat format;
-        VkFormatProperties3KHR props;
+        VkFormatProperties3 props;
     } tests[2] = {};
     int n_tests = 0;
     bool has_without_format_test = false, has_with_format_test = false;
@@ -308,7 +308,7 @@ TEST_F(NegativeShaderStorageImage, MissingFormatWriteForFormat) {
         if (has_without_format_test && has_with_format_test) break;
         if (!vkuFormatIsSampledFloat((VkFormat)fmt)) continue;
 
-        VkFormatProperties3KHR fmt_props_3 = vku::InitStructHelper();
+        VkFormatProperties3 fmt_props_3 = vku::InitStructHelper();
         VkFormatProperties2 fmt_props = vku::InitStructHelper(&fmt_props_3);
 
         vk::GetPhysicalDeviceFormatProperties2(Gpu(), (VkFormat)fmt, &fmt_props);
@@ -764,7 +764,7 @@ TEST_F(NegativeShaderStorageImage, UnknownWriteLessComponent) {
         GTEST_SKIP() << "Format doesn't support storage image";
     }
 
-    VkFormatProperties3KHR fmt_props_3 = vku::InitStructHelper();
+    VkFormatProperties3 fmt_props_3 = vku::InitStructHelper();
     VkFormatProperties2 fmt_props = vku::InitStructHelper(&fmt_props_3);
     vk::GetPhysicalDeviceFormatProperties2(Gpu(), format, &fmt_props);
     if ((fmt_props_3.optimalTilingFeatures & VK_FORMAT_FEATURE_2_STORAGE_WRITE_WITHOUT_FORMAT_BIT) == 0) {
@@ -843,7 +843,7 @@ TEST_F(NegativeShaderStorageImage, UnknownWriteComponentA8Unorm) {
         GTEST_SKIP() << "Format doesn't support storage image";
     }
 
-    VkFormatProperties3KHR fmt_props_3 = vku::InitStructHelper();
+    VkFormatProperties3 fmt_props_3 = vku::InitStructHelper();
     VkFormatProperties2 fmt_props = vku::InitStructHelper(&fmt_props_3);
     vk::GetPhysicalDeviceFormatProperties2(Gpu(), format, &fmt_props);
     if ((fmt_props_3.optimalTilingFeatures & VK_FORMAT_FEATURE_2_STORAGE_WRITE_WITHOUT_FORMAT_BIT) == 0) {
