@@ -1348,7 +1348,7 @@ TEST_F(NegativeSparseImage, UnalignedBindOffsets) {
     // size of the required alignment, to ensure we can bind both
     // allocated memory blocks to the image on aligned offsets.
     while (memory_reqs.size < (memory_reqs.alignment * 2)) {
-        image.destroy();
+        image.Destroy();
         image_create_info.extent.width *= 2;
         image_create_info.extent.height *= 2;
         image.InitNoMemory(*m_device, image_create_info);
